
;**************************************************************************
;*                    MSSP Main Board                                     *
;*                    Version 2015/01                                     *
;**************************************************************************
;**************************************************************************
;*          v 3.37 - 12.08.2017                                           *
;*          MSSP demo board                                               *
;*                                                                        *
;*          LCD & PS/2 keyboard routines included                         *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC18LF26K22 used                                     *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************



;------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------
;change history
;
;  0.07 	- basic I2C communication is working on MSSP1
;  0.09 	- improved write to LCD - Flash is used to store texts as strings
;         	  and generic routinge w_send_txt implemented
;  0.12 	- keaboard routines added
;  0.13 	- I2C Port Menu added
;  0.18 	- i2c_send and i2c_start finilized (with recognizion of ACK)
;  0.21 	- Ports functionality completely upported
;  0.22 	- Temperature sensors added
;  0.25 	- Ports & Temp sensors upport finished
;       	- OneWire upport finished (and working with new I2C communication start & send routines
;  0.26		- PCA9546 included - basic routines added
;  0.27		- entry menu included
;  0.29		- I2C diagnostic included and cutting off the "stuck" lines
;  0.30		- I2C EEPROM upport
;  0.32		- internal PCF8574 introduced to maintain SPI CS signals
;  0.33		- first SPI setup done
;  1.35		- I2C single command editor included
;  1.38		- Microcontroler P18LF25K22.INC used instead of P18F24K22.INC
;  1.40		- SPI single command editor included
;  1.41 	- wait time in the routine ds1820_conv_spu increased
;  1.43		- SPI setup included - it is possible to change to iddle stage of SPI Clock signal
;  1.44 - 1.46	- I2C Device List included
;  1.48 - 1.55  - 1-wire ROM Search implemented
;  3.01         - RTC upport
;               - buf in 1-wire search fixed
;  3.02         - "recovery" version
;  3.03 - 3.10  - IR Port included - first part of IR keaboard supported via interrupt routine
;  3.11         - Menu part IR included - to display received IR tripple-byte from IR-sensor
;  3.12	- 3.14	- RTC chip ISL12022 included
;  3.15		- support of PCA9534A included (Ports - Group "B" chip)
;  3.16		- support of basic functions of MCP23008 included
;  3.17		- detailed information about INT-related registers for MCP23008 added
;  3.18		- dailight saving maintenance for ISL12022 based on European convention included
;  3.19         - big in DST for ISL12022 corrected (after the DST switsch doesn't occur...)
;  3.20		- test routines for Freescale Sensor MPL3115A2
;  3.21		- 1w DOW CRC calculaiton included (using table look-up method)
;  3.22         - SPI EEPROM JEDEC-ID read implemented
;  3.23		- JEDEC identification for some Microchip and Adesto devices implemented
;  3.24 - 3.26  - SPI implementation - AMS driver included
; 		- SPI parameters extended + SPI test provided
;		- display of RTC information to AMS display implemented
;  3.27 - 3.28  - Setup menu "others" added
;  3.29         - SPI Setup extended - now it is possible to define defualt value for SDI-Line (pull-up / pull-down / nothing)
;  3.30         - AMS output shifted to SPI1
;               - SPI Editor is able to work with CS0 and CS1
;  3.31         - for AMS Output the SPI MSSP is always set to the required configuration
;		- routines (and menu entry) for SPI RTC DS1306 included
;  3.33		- temperature meassurement with ISL12022 implemented
;  3.34		- RTC with NXP PCA2129T with SPI bus implemented
;  3.35	- 3.37	- Demo Routines for MAX6969 included
;		- SPI menu extended
;------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------



;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)

;keyboard
;	kbdclk  -- RB7
;	kbddata -- RB6


 INCLUDE "P18LF26K22.INC"

 CONFIG		FOSC	= INTIO67
 CONFIG		BORV	= 190
 CONFIG		BOREN	= OFF
 CONFIG		PWRTEN	= ON
 CONFIG		WDTEN	= OFF
 CONFIG		PBADEN	= OFF
 CONFIG		MCLRE	= INTMCLR
 CONFIG		STVREN	= OFF
 CONFIG		XINST	= ON
 CONFIG		DEBUG	= OFF
 CONFIG		CP0	= OFF
 CONFIG		CP1	= OFF
 CONFIG		CPB	= OFF
 CONFIG		CPD	= OFF
 CONFIG		WRT0	= OFF
 CONFIG		WRT1	= OFF
 CONFIG		WRTB	= OFF
 CONFIG		WRTC	= OFF
 CONFIG		WRTD	= OFF
 CONFIG		EBTR0	= OFF
 CONFIG		EBTR1	= OFF
 CONFIG		EBTRB	= OFF


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'00'		;delay routine
TIMER2		EQU	H'01'		; "	"	"

v_ttt		EQU	H'02'
c_tmp		EQU	H'03'
data_tmp	EQU	H'04'
disp_pos	EQU	H'05'

kbd_buffer	EQU	H'06'
kbd_out		EQU	H'07'
kbd_parity	EQU	H'08'
kbd_bb		EQU	H'09'		;keyboard bit buffer

v_p_device	EQU	H'0A'
v_p0		EQU	H'0B'
v_p1		EQU	H'0C'
v_cursor2	EQU	H'0D'

v_r0		EQU	H'0E'
v_r1		EQU	H'0F'

v_pwm0		EQU	H'10'
v_pwm1		EQU	H'11'
v_psc0		EQU	H'12'
v_psc1		EQU	H'13'

v_comm		EQU	H'14'
v_b_in		EQU	H'15'

tmp_ot		EQU	H'16'
v_cursor	EQU	H'17'
b_ax_h		EQU	H'18'
b_ax		EQU	H'19'

v_pca953x	EQU	H'1A'
v_ls0		EQU	H'1B'
v_ls1		EQU	H'1C'
v_ls2		EQU	H'1D'
v_ls3		EQU	H'1E'
v_lsx		EQU	H'1F'

v_F0		EQU	H'20'

v_tst1		EQU	H'21'
v_tst2		EQU	H'22'
v_tst3		EQU	H'23'

v_tst1k		EQU	H'2C'		;redifining ow_ROM_0
v_tst2k		EQU	H'2D'		;redifining ow_ROM_1
v_tst3k		EQU	H'2E'		;redifining ow_ROM_2

v_i2c_add	EQU	H'24'

v_temp1		EQU	H'25'
v_temp2		EQU	H'26'
v_tempx		EQU	H'27'
v_temp1s	EQU	H'28'
v_temp2s	EQU	H'29'

v_board_LEDs	EQU	H'2A'

ow_buffer	EQU	H'2B'
ow_ROM_0	EQU	H'2C'
ow_ROM_1	EQU	H'2D'
ow_ROM_2	EQU	H'2E'
ow_ROM_3	EQU	H'2F'
ow_ROM_4	EQU	H'30'
ow_ROM_5	EQU	H'31'
ow_ROM_6	EQU	H'32'
ow_ROM_7	EQU	H'33'

ow_m1		EQU	H'34'
ow_m2		EQU	H'35'
ow_m3		EQU	H'36'
ow_m4		EQU	H'37'
ow_m5		EQU	H'38'
ow_m6		EQU	H'39'
ow_m7		EQU	H'3A'
ow_m8		EQU	H'3B'
ow_m9		EQU	H'3C'

ds2484_dc	EQU	H'3D'
ds2484_pc	EQU	H'3E'
ds2484_st	EQU	H'3F'
ds2484_data	EQU	H'40'

brd_i2c_config	EQU	H'41'

;---------------------------------------------------------------------
;EEPROM variables
v_ee_type	EQU	H'42'
	;selected EEPROM type:
	;	01 = 24LC01  (A:1010000; Add.bytes:1)		128 x 8		8
	;	02 = 24LC02  (A:1010000; Add.bytes:1)		256 x 8		8
	;	03 = 24LC04  (A:101000a; Add.bytes:1)		512 x 8		4
	;	04 = 24LC08  (A:10100aa; Add.bytes:1)		1024 x 8	2
	;	05 = 24LC16  (A:1010aaa; Add.bytes:1)		2048 x 8	1

	;	06 = 24LC32  (A:1010000; Add.bytes:2)		4096 x 8	8
	; 	07 = 24LC64  (A:1010000; Add.bytes:2)		8192 x 8	8
	;	08 = 24LC128 (A:1010000; Add.bytes:2)		16384 x 8	8
	;	09 = 24LC256 (A:1010000; Add.bytes:2)		32768 x 8	8
	;	0A = 24LC512 (A:1010000; Add.bytes:2)		65536 x 8	8

	;	0B = DS1624
	;	0C = AT28C256 connected via slave board with PIC16F724

	;	81 = SPI 23LC512


;EEPROM communication buffer - 8 bytes
v_eed0		EQU	H'43'
v_eed1		EQU	H'44'
v_eed2		EQU	H'45'
v_eed3		EQU	H'46'
v_eed4		EQU	H'37'
v_eed5		EQU	H'48'
v_eed6		EQU	H'49'
v_eed7		EQU	H'4A'

v_ee_add	EQU	H'4B'		;number of address bytes
v_ee_i2c	EQU	H'4C'		;EEPROM I2C address merged with memory address
v_ee_w		EQU	H'4D'		;write mode active

v_ad0		EQU	H'4E'		;EEPROM address - lowest 8 bits
v_ad1		EQU	H'4F'		;EEPROM address - highest bits

;--------------------------------------------------
;EEPROM communication buffer - old values - 8 bytes
o_eed0		EQU	H'50'
o_eed1		EQU	H'51'
o_eed2		EQU	H'52'
o_eed3		EQU	H'53'
o_eed4		EQU	H'54'
o_eed5		EQU	H'55'
o_eed6		EQU	H'56'
o_eed7		EQU	H'57'

v_spi_buffi	EQU	H'58'
v_spi_conf	EQU	H'5F'		;SPI control parameters
					;	bit 4 = bit 4 from the SSP2CON1 register - Clock Polarity Select bit
					;	bit 6 = bit 6 from the SSP2STAT register - SPI Clock Edge Select bit

					;       bit 0 = 0 -> no default value for SDI line
					;       bit 0 = 1 & bit 1 = 0 -- add pull down do SDI
					;       bit 0 = 1 & bit 1 = 1 -- add pull up do SDI

v_xxx_conf	EQU	H'5E'		;another settings

			;bit 0 = 1 --> for RTC the hh:mm:ss is set to the AMS1107 display driver via SPI
;---------------------------------------------------
;---------------------------------------------------
;redefining EEPROM variables -----------------------
;---------------------------------------------------
i2ce_cursor	EQU	H'42'
i2ce_stage	EQU	H'43'
i2ce_op		EQU	H'44'
;---------------------------------------------------
;---------------------------------------------------
;redefining 1-wire variables -----------------------
;---------------------------------------------------
v_i2c_devs	EQU	H'34'
v_i2c_actd	EQU	H'35'
;---------------------------------------------------
;---------------------------------------------------

;---------------------------------------------------
;---------------------------------------------------
;used for 1-wire Search ROM algorithm
;---------------------------------------------------
v_ow_dc		EQU	H'42'		;number of 1-wire devices found
v_ow_act_dir	EQU	H'43'

v_owc0		EQU	H'44'
v_owc1		EQU	H'45'
v_owc2		EQU	H'46'
v_owc3		EQU	H'47'
v_owc4		EQU	H'48'
v_owc5		EQU	H'49'
v_owc6		EQU	H'4A'
v_owc7		EQU	H'4B'
v_owcc		EQU	H'4C'

v_ow_curr	EQU	H'4D'
v_ows_debug	EQU	H'4E'

v_owd0		EQU	H'50'
v_owd1		EQU	H'51'
v_owd2		EQU	H'52'
v_owd3		EQU	H'53'
v_owd4		EQU	H'54'
v_owd5		EQU	H'55'
v_owd6		EQU	H'56'
v_owd7		EQU	H'57'

v_owdc		EQU	H'58'		;direction for search - current byte
v_rom_id_c	EQU	H'59'		;current byte of the ROM ID
;---------------------------------------------------
;---------------------------------------------------
;RTC variables --------------------
v_isl12022	EQU	H'41'		;ISL12022 indicator
					;bit 0 indicates usage of ISL12022
					;bit 1 indicates, the SPI RTC DS1306 is active
					;bit 2 indicates, the SPI RTC PCA2129 is active

v_rtc_d1	EQU	H'42'
v_rtc_d2	EQU	H'43'
v_rtc_m1	EQU	H'44'
v_rtc_m2	EQU	H'45'
v_rtc_y1	EQU	H'46'
v_rtc_y2	EQU	H'47'
v_rtc_y3	EQU	H'48'
v_rtc_y4	EQU	H'49'
v_rtc_h1	EQU	H'4A'
v_rtc_h2	EQU	H'4B'
v_rtc_n1	EQU	H'4C'
v_rtc_n2	EQU	H'4D'
v_rtc_s1	EQU	H'4E'
v_rtc_s2	EQU	H'4F'

v_rtc_sec	EQU	H'50'
v_rtc_min	EQU	H'51'
v_rtc_hrs	EQU	H'52'
v_rtc_wday	EQU	H'53'	;day in the week
v_rtc_day	EQU	H'54'
v_rtc_mnt	EQU	H'55'
v_rtc_year	EQU	H'56'

v_rtc_wd	EQU	H'57'
;---------------------------------------------------
;---------------------------------------------------
v_tmp1		EQU	H'58'	;temp variable - general usage

v_rtc_t1	EQU	H'59'
v_rtc_t2	EQU	H'5A'

v_ir_kbd	EQU	H'5B'
v_ir_kbd_new	EQU	H'5C'

ow_crc_calc	EQU	H'5D'

;last used RAM position: H'5D'

;---------------------------------------------------
;---------------------------------------------------
;Interrupt variables
;
v_itimer1	EQU	H'00'
v_itimer2	EQU	H'01'

v_ir_byte	EQU	H'02'
v_ir_bit	EQU	H'03'

v_ir_byte1	EQU	H'10'
v_ir_byte2	EQU	H'11'
v_ir_byte3	EQU	H'12'

v_ir_byte1_s	EQU	H'13'
v_ir_byte2_s	EQU	H'14'
v_ir_byte3_s	EQU	H'15'

v_ir_tmp	EQU	H'00'
;---------------------------------------------------
;---------------------------------------------------

v_reg00		EQU	H'42'
v_reg01		EQU	H'43'
v_reg02		EQU	H'44'
v_reg03		EQU	H'45'
v_reg04		EQU	H'46'
v_reg05		EQU	H'47'
v_reg06		EQU	H'48'
v_reg07		EQU	H'49'
v_reg08		EQU	H'4A'
v_reg09		EQU	H'4B'
v_reg10		EQU	H'4C'
;---------------------------------------------------
;---------------------------------------------------

;last used RAM position: H'5D'


;---------------------------------------------------
;---------------------------------------------------
v_jedec_0	EQU	H'2B'		;replacing 1-wire variables
v_jedec_1	EQU	H'2C'
v_jedec_2	EQU	H'2D'
v_jedec_3	EQU	H'2E'
v_jedec_4	EQU	H'2F'


;==========================================================================
;     AMS Variable Definition
;==========================================================================
ams_v_dig0		EQU	H'30'
ams_v_dig1		EQU	H'31'
ams_v_dig2		EQU	H'32'
ams_v_dig3		EQU	H'33'
ams_v_dig4		EQU	H'34'
ams_v_dig5		EQU	H'35'
ams_v_dig6		EQU	H'36'
ams_v_dig7		EQU	H'37'

ams_v_MSB		EQU	H'38'
ams_v_LSB		EQU	H'39'

ams_err_flag		EQU	H'3A'

ams_v_value		EQU	H'3B'
ams_v_tmp		EQU	H'3C'
ams_v_tmp2		EQU	H'3D'
;----------------------------------------------------------------------------------

;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	8		;Interrupt vector (we are using compatability mode --> Vector 0018h is not used
		goto	ir_main


;==========================================================================
; Text definitions
;==========================================================================
		ORG	100


t_release1	da	"MSSP Demo board 2015/01 ", H'FE'
t_release2	da	"Firmware 3.37-12.08.2017", H'FE'

t_menu_main1	da	"(B)asic   functions  I2C", H'FE'
t_menu_main2	da	"(S)pecial functions  I2C", H'FE'

;-----------------------------------------------------------
t_spi_max6969	da	"Demo (1) / (2) / (3)    ", H'FE'

t_spi_port1	da	"MAX696(9)               ", H'FE'
t_spi_port2	da	"                        ", H'FE'

t_spi_rtc1	da	"(D)S1306     / (P)CA2129", H'FE'
t_spi_rtc2	da	"                        ", H'FE'

t_spi1		da	"(R)TC        / (P)orts  ", H'FE'
t_spi2		da	"(S)PI Editor / (E)EPROM ", H'FE'

t_spi_ee	da	"(J)EDEC ID read", H'FE'
;-----------------------------------------------------------

t_kbd_wait	da	"waiting for keyboard...", H'FE'

t_ir_m1		da	"IR code:", H'FE'
t_ir_m2		da	"Last valid key: ", H'FE'

t_i2c_s1	da	"(R)TC     / (T)emperature", H'FE'
t_i2c_s2	da	"(E)EEPROM / (P)orts ", H'FE'

t_ports_m1	da	"Group selection:", H'FE'
t_ports_m2	da	"(A) / (B) / (C) / (X) ", H'FE'

t_ports_ti1	da	"PCF857(4) / PCF857(5) ", H'FE'

t_pcf8574	da	"PCF8574 - A:0100 000", H'FE'
t_tpic2810	da	"TPIC2810 - A:1100 000 ", H'FE'
t_pca6107	da	"PCA6107 - A:0011 000", H'FE'
t_pca9534A	da	"PCA9534A - A:0111 000 ", H'FE'
t_mcp23008	da	"MCP23008 - A:0100 000 ", H'FE'

t_ports_nxp1	da	"PCA953(0) / PCA953(1) ", H'FE'
t_ports_nxp2	da	"PCA953(2) ", H'FE'

t_tm1		da	"(1-2)TMP100/AT30 (3)LM76", H'FE'
t_tm2		da	"(4)DS1624    (5)ISL12022", H'FE'

t_owm1		da	"(B)asic functions ", H'FE'
t_owm2		da	"(D)evices / (RO)M Search", H'FE'

t_ow_bafu1	da	"(B)ridge registers", H'FE'
t_ow_bafu2	da	"(R)om id", H'FE'

t_ds1820	da	"ROM code(CRC*SN--Fam):", H'FE'

t_mm1		da	"(I)2C      (S)PI    I(R)", H'FE'
t_mm2		da	"(1)-wire   Set(u)p", H'FE'

t_s1		da	"(I)2C Chanel Selection", H'FE'
t_s2		da	"(S)PI         (O)thers", H'FE'

t_s_others1	da	"RTC to (A)MS / SPI1:", H'FE'

t_i2c_cee1	da	"I2C critical error", H'FE'
t_i2c_ee1	da	"I2C communication error ", H'FE'

t_eeprom01	da	"01 -- 128x8   ", H'FE'
t_eeprom02	da	"02 -- 256x8   ", H'FE'
t_eeprom04	da	"04 -- 512x8   ", H'FE'
t_eeprom08	da	"08 -- 1024x8  ", H'FE'
t_eeprom16	da	"16 -- 2048x8  ", H'FE'
t_eeprom32	da	"32 -- 4096x8  ", H'FE'
t_eeprom64	da	"64 -- 8192x8  ", H'FE'
t_eeprom128	da	"128 -- 16384x8", H'FE'
t_eeprom256	da	"256 -- 32768x8", H'FE'
t_eeprom512	da	"512 -- 65536x8", H'FE'
t_ds1624	da	"DS1624 -- 256x8 ", H'FE'
t_28c256	da	"28C256 -- 32768x8 ", H'FE'

t_ee_save	da	"Saving data...", H'FE'
t_ee_addr	da	"A:1010000  Add:", H'FE'

t_i2ce1		da	"I2CE", H'FE'

t_spi_par1	da	"(1) - Transmit", H'FE'
t_spi_par2	da	"(2) - Others", H'FE'

t_spi_par1_1	da	"(I)dle CLK:      T(e)st ", H'FE'
t_spi_par1_2	da	"(T)ransmit: ", H'FE'

t_spi_par2_1	da	"(I)dle SDI Value: ", H'FE'
t_spi_par2_2	da	"  ", H'FE'


t_spi_tt1	da	"Sending: 10xAAh, 10x55h ", H'FE'
t_spi_tt2	da	"Exit = ESC", H'FE'

t_sic2		da	"(E)ditor / (D)evice List", H'FE'

t_dl_nff	da	"No I2C device found", H'FE'
t_dl_s1		da	"Searching:", H'FE'
t_dl_s2		da	"Devices found: 000", H'FE'

t_dt_1010xxx	da	"EEPROM                  ", H'FE'
t_dt_0100xxx	da	"Port Expander (A)       ", H'FE'
t_dt_0011xxx	da	"Port Expander (B)       ", H'FE'
t_dt_1110xxx	da	"I2C Switch              ", H'FE'
t_dt_1001xxx	da	"Temperature Sensor      ", H'FE'
t_dt_ukn	da	"Unidentified            ", H'FE'

t_dti_PCA6107	da	"DB: PCA6107  Port Exp B ", H'FE'
t_dti_PCF8574	da	"DB: PCF8574  Port Exp A ", H'FE'
t_dti_PCA9546	da	"DB: PCA9546  I2C Switch ", H'FE'

t_ow_tg1	da	"Device CRC:  ", H'FE'
t_ow_tg2	da	"Computed CRC:", H'FE'

t_tmp_isl12022	da	"ISL12022       A:1101111", H'FE'
;==========================================================================
;==========================================================================


		ORG 0F00

;-----------------------------------------------------------------------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------------------------------------------------------------------
;1-wire CRC look-up table based on AN27 (Maxim Integrated / Dallas)
;"lazy definition"
;-----------------------------------------------------------------------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------------------------------------------------------------------
ow_crc_lt	da	D'000', D'094', D'188', D'226', D'097', D'063', D'221', D'131', D'194', D'156', D'126', D'032', D'163', D'253', D'031', D'065'
		da	D'157', D'195', D'033', D'127', D'252', D'162', D'064', D'030', D'095', D'001', D'227', D'189', D'062', D'096', D'130', D'220'
		da	D'035', D'125', D'159', D'193', D'066', D'028', D'254', D'160', D'225', D'191', D'093', D'003', D'128', D'222', D'060', D'098'
		da	D'190', D'224', D'002', D'092', D'223', D'129', D'099', D'061', D'124', D'034', D'192', D'158', D'029', D'067', D'161', D'255'
		da	D'070', D'024', D'250', D'164', D'039', D'121', D'155', D'197', D'132', D'218', D'056', D'102', D'229', D'187', D'089', D'007'
		da	D'219', D'133', D'103', D'057', D'186', D'228', D'006', D'088', D'025', D'071', D'165', D'251', D'120', D'038', D'196', D'154'
		da	D'101', D'059', D'217', D'135', D'004', D'090', D'184', D'230', D'167', D'249', D'027', D'069', D'198', D'152', D'122', D'036'
		da	D'248', D'166', D'068', D'026', D'153', D'199', D'037', D'123', D'058', D'100', D'134', D'216', D'091', D'005', D'231', D'185'
		da	D'140', D'210', D'048', D'110', D'237', D'179', D'081', D'015', D'078', D'016', D'242', D'172', D'047', D'113', D'147', D'205'
		da	D'017', D'079', D'173', D'243', D'112', D'046', D'204', D'146', D'211', D'141', D'111', D'049', D'178', D'236', D'014', D'080'
		da	D'175', D'241', D'019', D'077', D'206', D'144', D'114', D'044', D'109', D'051', D'209', D'143', D'012', D'082', D'176', D'238'
		da	D'050', D'108', D'142', D'208', D'083', D'013', D'239', D'177', D'240', D'174', D'076', D'018', D'145', D'207', D'045', D'115'
		da	D'202', D'148', D'118', D'040', D'171', D'245', D'023', D'073', D'008', D'086', D'180', D'234', D'105', D'055', D'213', D'139'
		da	D'087', D'009', D'235', D'181', D'054', D'104', D'138', D'212', D'149', D'203', D'041', D'119', D'244', D'170', D'072', D'022'
		da	D'233', D'183', D'085', D'011', D'136', D'214', D'052', D'106', D'043', D'117', D'151', D'201', D'074', D'020', D'246', D'168'
		da	D'116', D'042', D'200', D'150', D'021', D'075', D'169', D'247', D'182', D'232', D'010', D'084', D'215', D'137', D'107', D'053'
;-----------------------------------------------------------------------------------------------------------------------------------------------------


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;	       ************************************
;              **  INTERRUPT :  ir main routine  **
;              ************************************
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;IR routine receives IR Codes from the Pollin SWK-8650 keyboard

;standard key hex codes:
;-----------------------
;		14-A3-87
;	1	14-AB-8F
;	2	14-AB-07
;	3	14-AA-87
;	4	14-AA-47
;	5	14-A2-4F
;	6	14-A3-E7
;	7	14-AB-EF
;	8	14-AA-E7
;	9	14-AA-6F
;	0	14-AB-67
;	?	14-A3-6F
;		14-A2-EF
;	BACK	14-A9-CF
;	TAB	14-A9-87
;	Q	14-AF-87
;	W	14-AF-0F
;	E	14-AE-8F
;	R	14-AE-4F
;	T	14-A8-4F
;	Z	14-A9-E7
;	U	14-AF-E7
;	I	14-AE-EF
;	O	14-AE-67
;	P	14-AF-6F
;	A	14-A7-8F
;	S	14-A7-07
;	D	14-A6-87
;	F	14-A6-47
;	G	14-AC-47
;	H	14-AD-EF
;	J	14-A7-EF
;	K	14-A6-E7
;	L	14-A6-6F
;	Y	14-A5-87
;	X	14-A5-0F
;	C	14-A4-8F
;	V	14-A4-4F
;	B	14-A0-47
;	N	14-A1-EF
;	M	14-A5-E7
;	->	14-A1-A7
;	<-	14-A1-2F
;	^	14-AD-2F (sipka hore)
;	v	14-A0-AF (sipka dole)

;	ESC	14-AD-8F
;	ENTER	14-A5-CF
;-----------------------

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

		ORG 1100

ir_main		nop
		btfss	INTCON,RBIF		;unkown event -- return
		goto	leave_ir

		bcf	INTCON,D'007'		;disable interrupts
		movf	PORTB,0
		bcf	INTCON,RBIF

		lfsr	FSR2, H'300'		;FSR2 to start of bank 3
;-----------------------------------------------------------------------------
;Interrupt business logic starts here....
;-----------------------------------------------------------------------------
		bsf	PORTC,D'000'		;switch the LED on - we got received signal...
		btfsc	PORTB,D'005'
		goto	ir_cancel		;we expect level 0 now...

		call	ir_get_byte		;receive 1st byte
		movf	v_ir_byte,0
		movwf	v_ir_byte1_s
		call	ir_get_byte		;receive 2nd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte2_s
		call	ir_get_byte		;receive 3rd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte3_s

		movf	v_ir_byte1_s,0
		xorlw	H'14'
		btfss	STATUS,Z
		goto	leave_ir		;not valid code received (first byte is always 14h) --> exit

		movf	v_ir_byte1_s,0
		movwf	v_ir_byte1
		movf	v_ir_byte2_s,0
		movwf	v_ir_byte2
		movf	v_ir_byte3_s,0
		movwf	v_ir_byte3

		call	ir_decode_key

		movwf	v_ir_tmp
		movf	v_ir_tmp,1
		btfsc	STATUS,Z
		goto	leave_ir2		;not valid code received (first byte is always 14h) --> exit


		lfsr	FSR2, H'000'		;FSR2 to start of bank 0
		movwf	v_ir_kbd
		clrf	v_ir_kbd_new		;store the code
		bsf	v_ir_kbd_new,D'000'	;set flag - new character received

;----------------------------------------
ir_cancel	bcf	PORTC,D'000'		;switch the LED off
;-----------------------------------------------------------------------------
;Interrupt business logic ends here....
;-----------------------------------------------------------------------------
leave_ir	lfsr	FSR2, H'000'		;FSR2 to start of bank 0

		bsf	INTCON,RBIE
		bsf	INTCON,D'007'
		RETFIE	FAST			;return from interrupt with restore of STAUS, WREG and BSR
;-----------------------------------------------------------------------------
leave_ir2	lfsr	FSR2, H'000'		;FSR2 to start of bank 0
		bcf	v_ir_kbd_new,D'000'	;clear flag - no valid character received
		goto	leave_ir
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_byte	clrf	v_ir_byte
		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'007'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'006'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'005'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'004'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'003'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'002'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'001'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'000'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_bit	clrf	v_ir_bit
		call	wait300us
		btfsc	PORTB,D'005'
		bsf	v_ir_bit,D'000'
		call	wait460us
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; interrupt delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d350		MOVLW   D'255'
		MOVWF   v_itimer2
iDEL_LOOP1	MOVLW   D'255'
		MOVWF   v_itimer1

iDEL_LOOP2	DECFSZ  v_itimer1,F
		GOTO    iDEL_LOOP2
		DECFSZ  v_itimer2,F
		GOTO    iDEL_LOOP1
		return
;----------------------------------------------------------------------------
wait300us	movlw	D'001'
		movwf	v_itimer2
iw3_l2		movlw	D'210'
		movwf	v_itimer1

iw3_l1		decfsz	v_itimer1,1		;0,5us
		goto	iw3_l1			;1,0us

		decfsz	v_itimer2,1
		goto	iw3_l2
		return

;----------------------------------------------------------------------------
wait460us	movlw	D'002'
		movwf	v_itimer2
iw4_l2		movlw	D'155'
		movwf	v_itimer1

iw4_l1		decfsz	v_itimer1,1		;0,5us
		goto	iw4_l1			;1,0us

		decfsz	v_itimer2,1
		goto	iw4_l2
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;	B	14-A0-47
;	v	14-A0-AF (sipka dole)

;	N	14-A1-EF
;	->	14-A1-A7
;	<-	14-A1-2F


;	5	14-A2-4F
;		14-A2-EF


;		14-A3-87
;	?	14-A3-6F
;	6	14-A3-E7


;	C	14-A4-8F
;	V	14-A4-4F


;	Y	14-A5-87
;	X	14-A5-0F
;	M	14-A5-E7
;	ENTER	14-A5-CF


;	D	14-A6-87
;	F	14-A6-47
;	K	14-A6-E7
;	L	14-A6-6F



;	A	14-A7-8F
;	S	14-A7-07
;	J	14-A7-EF

;	T	14-A8-4F


;	Z	14-A9-E7
;	BACK	14-A9-CF
;	TAB	14-A9-87



;	3	14-AA-87
;	4	14-AA-47
;	8	14-AA-E7
;	9	14-AA-6F


;	1	14-AB-8F
;	2	14-AB-07
;	7	14-AB-EF
;	0	14-AB-67


;	G	14-AC-47


;	H	14-AD-EF
;	ESC	14-AD-8F
;	^	14-AD-2F (sipka hore)


;	R	14-AE-4F
;	E	14-AE-8F
;	I	14-AE-EF
;	O	14-AE-67

;	Q	14-AF-87
;	W	14-AF-0F
;	U	14-AF-E7
;	P	14-AF-6F


;----------------------------------------------------------------------------
ir_decode_key	movf	v_ir_byte2,0
		xorlw	H'A0'
		btfsc	STATUS,Z
		goto	ir_dc_A0

		movf	v_ir_byte2,0
		xorlw	H'A1'
		btfsc	STATUS,Z
		goto	ir_dc_A1

		movf	v_ir_byte2,0
		xorlw	H'A2'
		btfsc	STATUS,Z
		goto	ir_dc_A2

		movf	v_ir_byte2,0
		xorlw	H'A3'
		btfsc	STATUS,Z
		goto	ir_dc_A3

		movf	v_ir_byte2,0
		xorlw	H'A4'
		btfsc	STATUS,Z
		goto	ir_dc_A4

		movf	v_ir_byte2,0
		xorlw	H'A5'
		btfsc	STATUS,Z
		goto	ir_dc_A5

		movf	v_ir_byte2,0
		xorlw	H'A6'
		btfsc	STATUS,Z
		goto	ir_dc_A6

		movf	v_ir_byte2,0
		xorlw	H'A7'
		btfsc	STATUS,Z
		goto	ir_dc_A7

		movf	v_ir_byte2,0
		xorlw	H'A8'
		btfsc	STATUS,Z
		goto	ir_dc_A8

		movf	v_ir_byte2,0
		xorlw	H'A9'
		btfsc	STATUS,Z
		goto	ir_dc_A9

		movf	v_ir_byte2,0
		xorlw	H'AA'
		btfsc	STATUS,Z
		goto	ir_dc_AA

		movf	v_ir_byte2,0
		xorlw	H'AB'
		btfsc	STATUS,Z
		goto	ir_dc_AB

		movf	v_ir_byte2,0
		xorlw	H'AC'
		btfsc	STATUS,Z
		goto	ir_dc_AC

		movf	v_ir_byte2,0
		xorlw	H'AD'
		btfsc	STATUS,Z
		goto	ir_dc_AD

		movf	v_ir_byte2,0
		xorlw	H'AE'
		btfsc	STATUS,Z
		goto	ir_dc_AE

		movf	v_ir_byte2,0
		xorlw	H'AF'
		btfsc	STATUS,Z
		goto	ir_dc_AF

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A0	movf	v_ir_byte3,0
		xorlw	H'47'
		btfsc	STATUS,Z
		retlw	H'32'			;B

		movf	v_ir_byte3,0
		xorlw	H'AF'
		btfsc	STATUS,Z
		retlw	H'72'			;v

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A1	movf	v_ir_byte3,0
		xorlw	H'EF'
		btfsc	STATUS,Z
		retlw	H'31'			;N

		movf	v_ir_byte3,0
		xorlw	H'A7'
		btfsc	STATUS,Z
		retlw	H'74'			;->

		movf	v_ir_byte3,0
		xorlw	H'2F'
		btfsc	STATUS,Z
		retlw	H'6B'			;<-


		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A2	movf	v_ir_byte3,0
		xorlw	H'4F'
		btfsc	STATUS,Z
		retlw	H'2E'			;5

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A3	movf	v_ir_byte3,0
		xorlw	H'6F'
		btfsc	STATUS,Z
		retlw	H'4E'			;?

		movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'36'			;6

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A4	movf	v_ir_byte3,0
		xorlw	H'8F'
		btfsc	STATUS,Z
		retlw	H'21'			;C

		movf	v_ir_byte3,0
		xorlw	H'4F'
		btfsc	STATUS,Z
		retlw	H'2A'			;V

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A5	movf	v_ir_byte3,0
		xorlw	H'87'
		btfsc	STATUS,Z
		retlw	H'35'			;Y

		movf	v_ir_byte3,0
		xorlw	H'0F'
		btfsc	STATUS,Z
		retlw	H'22'			;X

		movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'3A'			;M

		movf	v_ir_byte3,0
		xorlw	H'CF'
		btfsc	STATUS,Z
		retlw	H'5A'			;ENTER

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A6	movf	v_ir_byte3,0
		xorlw	H'87'
		btfsc	STATUS,Z
		retlw	H'23'			;D

		movf	v_ir_byte3,0
		xorlw	H'47'
		btfsc	STATUS,Z
		retlw	H'2B'			;F

		movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'42'			;K

		movf	v_ir_byte3,0
		xorlw	H'6F'
		btfsc	STATUS,Z
		retlw	H'4B'			;L

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A7	movf	v_ir_byte3,0
		xorlw	H'8F'
		btfsc	STATUS,Z
		retlw	H'1C'			;A

		movf	v_ir_byte3,0
		xorlw	H'07'
		btfsc	STATUS,Z
		retlw	H'1B'			;S

		movf	v_ir_byte3,0
		xorlw	H'EF'
		btfsc	STATUS,Z
		retlw	H'3B'			;J
		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A8	movf	v_ir_byte3,0
		xorlw	H'4F'
		btfsc	STATUS,Z
		retlw	H'2C'			;T

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_A9	movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'1A'			;Z

		movf	v_ir_byte3,0
		xorlw	H'CF'
		btfsc	STATUS,Z
		retlw	H'66'			;BACK

		movf	v_ir_byte3,0
		xorlw	H'87'
		btfsc	STATUS,Z
		retlw	H'0D'			;TAB

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AA	movf	v_ir_byte3,0
		xorlw	H'87'
		btfsc	STATUS,Z
		retlw	H'26'			;3

		movf	v_ir_byte3,0
		xorlw	H'47'
		btfsc	STATUS,Z
		retlw	H'25'			;4

		movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'3E'			;8

		movf	v_ir_byte3,0
		xorlw	H'6F'
		btfsc	STATUS,Z
		retlw	H'46'			;9

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AB	movf	v_ir_byte3,0
		xorlw	H'67'
		btfsc	STATUS,Z
		retlw	H'45'			;0

		movf	v_ir_byte3,0
		xorlw	H'8F'
		btfsc	STATUS,Z
		retlw	H'16'			;1

		movf	v_ir_byte3,0
		xorlw	H'07'
		btfsc	STATUS,Z
		retlw	H'1E'			;2

		movf	v_ir_byte3,0
		xorlw	H'EF'
		btfsc	STATUS,Z
		retlw	H'3D'			;7

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AC	movf	v_ir_byte3,0
		xorlw	H'47'
		btfsc	STATUS,Z
		retlw	H'34'			;G

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AD	movf	v_ir_byte3,0
		xorlw	H'EF'
		btfsc	STATUS,Z
		retlw	H'33'			;H

		movf	v_ir_byte3,0
		xorlw	H'8F'
		btfsc	STATUS,Z
		retlw	H'76'			;ESC

		movf	v_ir_byte3,0
		xorlw	H'2F'
		btfsc	STATUS,Z
		retlw	H'75'			;^


		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AE	movf	v_ir_byte3,0
		xorlw	H'4F'
		btfsc	STATUS,Z
		retlw	H'2D'			;R

		movf	v_ir_byte3,0
		xorlw	H'8F'
		btfsc	STATUS,Z
		retlw	H'24'			;E

		movf	v_ir_byte3,0
		xorlw	H'EF'
		btfsc	STATUS,Z
		retlw	H'43'			;I

		movf	v_ir_byte3,0
		xorlw	H'67'
		btfsc	STATUS,Z
		retlw	H'44'			;O

		retlw	H'00'			;nothing known
;--------------------------------------
ir_dc_AF	movf	v_ir_byte3,0
		xorlw	H'87'
		btfsc	STATUS,Z
		retlw	H'15'			;Q

		movf	v_ir_byte3,0
		xorlw	H'0F'
		btfsc	STATUS,Z
		retlw	H'1D'			;W

		movf	v_ir_byte3,0
		xorlw	H'E7'
		btfsc	STATUS,Z
		retlw	H'3C'			;U

		movf	v_ir_byte3,0
		xorlw	H'6F'
		btfsc	STATUS,Z
		retlw	H'4D'			;P

		retlw	H'00'			;nothing known
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop
		movlb	H'0F'

		lfsr	FSR2, H'000'		;FSR2 to start of bank 0

;oscilator setup
		movlw	B'01100000'		;Frequency = 8MHz  ( OSCON<7:5> = 110 )
		movwf	OSCCON

		clrf	INTCON			;disable interrupts

;PORTA set-up
		clrf	PORTA
		clrf	LATA
		clrf	ANSELA

		clrf	PORTB
		clrf	LATB
		clrf	ANSELB
		movlw	B'00000000'
		movwf	TRISA
		movlw	B'11111111'
		movwf	TRISB

		movlw	B'11000000'
		movwf	WPUB			;week pull-ups for: keyboard (RB6, RB7)

;set-up for I2C Master operation for MSSP1
		clrf	ANSELC
		movlw	B'11111011'
		movwf	TRISC
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		call	LED_blink

		call	i2c_init
		call	spi_init
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		call	disp_init
		call	w_release
		call	LED_blink
		call	LED_blink


;wait for keaboard

		call	disp_clr1
		call	disp_clr2
		call	w_wkbd


;--- test ----
led_setup	movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'
		movwf	kbd_out
		call	send_ck


		movlw	H'FF'
		movwf	v_board_LEDs

		call	i2c_setup


		clrf	v_xxx_conf
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;set up interrupt registers for IR-module
;Data Link of the IR-Module is connected to RB5
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		clrf	v_ir_kbd
		clrf	v_ir_kbd_new

		bcf	INTCON,D'007'		;disable all interrupts

		bsf	INTCON,RBIE		;enable RBx Interrupt-On-Change

		bcf	INTCON2,D'007'		;enable pull-ups feature on PORTB
		bsf	WPUB,D'005'		;enable weak pull-up for RB5

		movlw	B'00100000'
		movwf	IOCB			;enable interrupt on change for RB5

		bcf	PORTC,D'000'
		bcf	TRISC,D'000'		;set PORTC0 to output - connected LED

		bsf	PORTC,D'000'
		call	d2
		bcf	PORTC,D'000'
		call	d2


		movf	PORTB,0
		bcf	INTCON,RBIF
		bsf	INTCON,D'007'		;enable interrupts

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	mm001			;start with main menu


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      IR Test Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_ir_test	movlw	B'00001100'
		call	send_comm

		call	disp_clr1
		call	disp_clr2
		call	w_ir_m


ir_mt001	call	w_ir_info

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back
		btfss	STATUS,Z
		goto	ir_mt001

		movlw	B'00001111'
		call	send_comm
		goto	mm001


;-----------------------------------------------------------------
w_ir_m		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ir_m1
		movwf	TBLPTRH
		movlw	low t_ir_m1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_ir_m2
		movwf	TBLPTRH
		movlw	low t_ir_m2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------
w_ir_info	movlw	B'11010000'
		call	send_comm

		movf	v_ir_kbd,0
		movwf	kbd_buffer
		call	write_hex

		movlw	B'10010000'
		call	send_comm
		lfsr	FSR2, H'300'		;FSR2 to start of bank 3
		movf	v_ir_byte1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_ir_byte2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_ir_byte3,0
		movwf	kbd_buffer
		call	write_hex
		lfsr	FSR2, H'000'		;FSR2 to start of bank 0

		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main entry screen
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
mm001		call	disp_clr1
		call	disp_clr2
		call	w_mm001

mm002		clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'43'			;I = I2C Main Menu
		btfsc	STATUS,Z
		goto	m001

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = SPI Main Menu
		btfsc	STATUS,Z
		goto	m_spi

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = IR Test Menu
		btfsc	STATUS,Z
		goto	m_ir_test

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = 1-wire Main Menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		movf	kbd_buffer,0
		xorlw	H'3C'			;U = Setup
		btfsc	STATUS,Z
		goto	m_setup

		goto	mm002

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi1
		movwf	TBLPTRH
		movlw	low t_spi1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi2
		movwf	TBLPTRH
		movlw	low t_spi2
		movwf	TBLPTRL
		call	w_send_txt
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_max6969	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_max6969
		movwf	TBLPTRH
		movlw	low t_spi_max6969
		movwf	TBLPTRL
		call	w_send_txt
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_port	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_port1
		movwf	TBLPTRH
		movlw	low t_spi_port1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_port2
		movwf	TBLPTRH
		movlw	low t_spi_port2
		movwf	TBLPTRL
		call	w_send_txt
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_rtc	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_rtc1
		movwf	TBLPTRH
		movlw	low t_spi_rtc1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_rtc2
		movwf	TBLPTRH
		movlw	low t_spi_rtc2
		movwf	TBLPTRL
		call	w_send_txt
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_eeprom	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_ee
		movwf	TBLPTRH
		movlw	low t_spi_ee
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; RTC
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
rtc_v_init	movlw	'-'
		movwf	v_rtc_d1
		movwf	v_rtc_d2
		movwf	v_rtc_m1
		movwf	v_rtc_m2
		movwf	v_rtc_y1
		movwf	v_rtc_y2
		movwf	v_rtc_y3
		movwf	v_rtc_y4
		movwf	v_rtc_h1
		movwf	v_rtc_h2
		movwf	v_rtc_n1
		movwf	v_rtc_n2
		movwf	v_rtc_s1
		movwf	v_rtc_s2

		movwf	v_rtc_t1
		movwf	v_rtc_t2

		return
;------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;	RTC routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_translate	nop
		movf	v_rtc_sec,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	rtc_v_init				;no RTC device connected

;day
		swapf	v_rtc_day,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_d1

		movf	v_rtc_day,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_d2


;month
		swapf	v_rtc_mnt,0
		andlw	B'00000001'
		addlw	'0'
		movwf	v_rtc_m1

		movf	v_rtc_mnt,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_m2

;year
		movlw	'2'
		movwf	v_rtc_y1
		movlw	'0'
		movwf	v_rtc_y2

		swapf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y3

		movf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y4

;hours
		swapf	v_rtc_hrs,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_h1

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_h2

;minutes
		swapf	v_rtc_min,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_n1

		movf	v_rtc_min,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_n2

;seconds
		swapf	v_rtc_sec,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_s1

		movf	v_rtc_sec,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_s2
;
;day in the week
		movf	v_rtc_wday,0
		andlw	B'00000111'
		movwf	v_rtc_wday

		movf	v_rtc_wday,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	rtc_set_Mo

		movf	v_rtc_wday,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	rtc_set_Tu

		movf	v_rtc_wday,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	rtc_set_We

		movf	v_rtc_wday,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	rtc_set_Th

		movf	v_rtc_wday,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	rtc_set_Fr

		movf	v_rtc_wday,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	rtc_set_Sa

		movf	v_rtc_wday,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	rtc_set_Su
;
		return

rtc_set_Mo	movlw	'M'
		movwf	v_rtc_t1
		movlw	'o'
		movwf	v_rtc_t2
		return

rtc_set_Tu	movlw	'T'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

rtc_set_We	movlw	'W'
		movwf	v_rtc_t1
		movlw	'e'
		movwf	v_rtc_t2
		return

rtc_set_Th	movlw	'T'
		movwf	v_rtc_t1
		movlw	'h'
		movwf	v_rtc_t2
		return

rtc_set_Fr	movlw	'F'
		movwf	v_rtc_t1
		movlw	'r'
		movwf	v_rtc_t2
		return

rtc_set_Sa	movlw	'S'
		movwf	v_rtc_t1
		movlw	'a'
		movwf	v_rtc_t2
		return

rtc_set_Su	movlw	'S'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

;-----------------------------------------------------------------------------
rtc_backtrs	nop
;date - day
		movlw	'0'
		subwf	v_rtc_d1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_d2,0
		addwf	v_tmp1,0

		movwf	v_rtc_day
;----------------------------------
;date - month
		movlw	'0'
		subwf	v_rtc_m1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_m2,0
		addwf	v_tmp1,0

		movwf	v_rtc_mnt
;----------------------------------
;date - year
		movlw	'0'
		subwf	v_rtc_y3,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_y4,0
		addwf	v_tmp1,0

		movwf	v_rtc_year
;----------------------------------
;date - day in the week - always use 1; and we will see...
		movlw	'0'
		subwf	v_rtc_wd,0
		movwf	v_rtc_wday

;----------------------------------
;time - hours
		movlw	'0'
		subwf	v_rtc_h1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_h2,0
		addwf	v_tmp1,0

		movwf	v_rtc_hrs
;----------------------------------
;time - minutes
		movlw	'0'
		subwf	v_rtc_n1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_n2,0
		addwf	v_tmp1,0

		movwf	v_rtc_min
;----------------------------------
;time - seconds
		movlw	'0'
		subwf	v_rtc_s1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_s2,0
		addwf	v_tmp1,0

		movwf	v_rtc_sec

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Command Editor
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
spi_editor_st	call	spi_reinit1
spi_editor	call	spi_cs_all_dis		;disable all CSx

		call	disp_clr2
		call	disp_clr1

		movlw	B'10000000'
		call	send_comm
		call	wm_C
		call	wm_S

ddq002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'45'			;0 = CS0
		btfsc	STATUS,Z
		goto	spi_edit_cs0

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = CS1
		btfsc	STATUS,Z
		goto	spi_edit_cs1

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	ddq002

		goto	m_spi
;------------------------------------------------------------------------------
spi_edit_cs0	call	wm_0
		call	wm_sip1
		call	spi_cs0_en		;activate - CS0

		call	d2
		call	d2
		call	d2

		goto	spi_edit2
;------------------------------------------------------------------------------
spi_edit_cs1	call	wm_1
		call	wm_sip1
		call	spi_cs1_en		;activate - CS1

		call	d2
		call	d2
		call	d2

		goto	spi_edit2

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
spi_edit2	movlw	D'003'
		movwf	i2ce_cursor

spi_edit3	movf	i2ce_cursor,0
		addlw	B'11000000'
		call	send_comm
		call	wm_dvojbodka

		movf	i2ce_cursor,0
		addlw	D'128'
		call	send_comm
		call	wm_sip1

		clrf	v_b_in
		movf	i2ce_cursor,0
		addlw	B'10000001'
		call	hex_input
;----------------------------------------------------------------------------
		movf	v_comm,0
		xorlw	H'01'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	spi_editor
;----------------------------------------------------------------------------
		movf	v_b_in,0
		call	spi_send
		call	d2

		movf	i2ce_cursor,0
		addlw	B'11000001'
		call	send_comm
		movf	v_spi_buffi,0
		movwf	kbd_buffer
		call	write_hex

		call	spi_mcc3

		goto	spi_edit3

;----------------------------------------------------------------------------
spi_mcc3	movlw	D'003'
		addwf	i2ce_cursor,1

;we want to have at least 3 characters left
		movf	i2ce_cursor,0
		movwf	data_tmp
		movlw	D'009'			;5 needs to be left, but we need 6 more (to be able to compare bit 5...
		addwf	data_tmp,1

		btfss	data_tmp,D'005'		;we passed the column 24...
		return

		call	d2
		call	d2
		movlw	D'003'
		movwf	i2ce_cursor
		call	disp_clr1_p4
		call	disp_clr2
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
jedec_id_read	call	spi_cs0_en

		bcf	TRISB,D'004'
		bsf	PORTB,D'004'		;use pull-up for SDI line (MISO)

		movlw	H'9F'
		call	spi_send

		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_jedec_0

		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_jedec_1

		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_jedec_2

		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_jedec_3

		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_jedec_4

		call	spi_cs_all_dis

		bsf	TRISB,D'004'		;deactivate pull-up
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_jedec_id	movlw	B'10000000'		;line 1 / char0
		call	send_comm
		call	wm_I
		call	wm_D
		call	wm_dvojbodka
		call	wm_space

		movf	v_jedec_0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_jedec_1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_jedec_2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_jedec_3,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_jedec_4,0
		movwf	kbd_buffer
		call	write_hex

;----------------------------------------------------------------
		movlw	B'11000000'		;Line 2 / Postion 1
		call	send_comm

		movf	v_jedec_0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_jedec_nop		;No answer

		movf	v_jedec_0,0
		xorlw	H'BF'
		btfsc	STATUS,Z
		goto	w_jedec_mcp		;Microchip

		movf	v_jedec_0,0
		xorlw	H'1F'
		btfsc	STATUS,Z
		goto	w_jedec_ads		;Adesto

		call	wm_O
		call	wm_t
		call	wm_h
		call	wm_e
		call	wm_r
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;------------------------------------------------------------------------------
w_jedec_nop	call	wm_N
		call	wm_o
		call	wm_space
		call	wm_A
		call	wm_s
		call	wm_w
		call	wm_e
		call	wm_r
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;JEDEC-ID: Microchip
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_jedec_mcp	call	wm_M
		call	wm_i
		call	wm_c
		call	wm_r
		call	wm_o
		call	wm_c
		call	wm_h
		call	wm_i
		call	wm_p

		movlw	B'11001010'		;Line 2 / Position 10
		call	send_comm

		movf	v_jedec_1,0
		xorlw	H'26'
		btfss	STATUS,Z
		goto	w_mcp_n1

		movf	v_jedec_2,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	w_mcp_mcp1		;SST26VF064B


w_mcp_n1	movf	v_jedec_1,0
		xorlw	H'25'
		btfss	STATUS,Z
		goto	w_mcp_n2

		movf	v_jedec_2,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	w_mcp_mcp2		;SST25VF064C

w_mcp_n2	movf	v_jedec_1,0
		xorlw	H'25'
		btfss	STATUS,Z
		goto	w_mcp_n3

		movf	v_jedec_2,0
		xorlw	H'8E'
		btfsc	STATUS,Z
		goto	w_mcp_mcp3		;SST25VF080B

w_mcp_n3	nop
		goto	w_jedec_other
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_mcp_mcp1	call	wm_S
		call	wm_S
		call	wm_T
		call	wm_2
		call	wm_6
		call	wm_V
		call	wm_F
		call	wm_0
		call	wm_6
		call	wm_4
		call	wm_B

		movlw	B'10010100'		;Line 1 / Position 20
		call	send_comm
		call	wm_6
		call	wm_4
		call	wm_M
		call	wm_b

		return
;------------------------------------------------------------------------------
w_mcp_mcp2	call	wm_S
		call	wm_S
		call	wm_T
		call	wm_2
		call	wm_5
		call	wm_V
		call	wm_F
		call	wm_0
		call	wm_6
		call	wm_4
		call	wm_C

		movlw	B'10010100'		;Line 1 / Position 20
		call	send_comm
		call	wm_6
		call	wm_4
		call	wm_M
		call	wm_b
		return
;------------------------------------------------------------------------------
w_mcp_mcp3	call	wm_S
		call	wm_S
		call	wm_T
		call	wm_2
		call	wm_5
		call	wm_V
		call	wm_F
		call	wm_0
		call	wm_8
		call	wm_0
		call	wm_B

		movlw	B'10010100'		;Line 1 / Position 20
		call	send_comm
		call	wm_space
		call	wm_8
		call	wm_M
		call	wm_b
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;JEDEC-ID: Adesto
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_jedec_ads	call	wm_A
		call	wm_d
		call	wm_e
		call	wm_s
		call	wm_t
		call	wm_o

		movlw	B'11001010'		;Line 2 / Position 10
		call	send_comm

		movf	v_jedec_1,0
		xorlw	H'44'
		btfss	STATUS,Z
		goto	w_ads_n1

		movf	v_jedec_2,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	w_ads_ads1		;AT25DF041B

;--------------------------------------------------------------
w_ads_n1	movf	v_jedec_1,0
		xorlw	H'86'
		btfss	STATUS,Z
		goto	w_ads_n2

		movf	v_jedec_2,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	w_ads_ads2		;AT25SF161

w_ads_n2	nop
		goto	w_jedec_other

;------------------------------------------------------------------------------
w_ads_ads1	call	wm_A
		call	wm_T
		call	wm_2
		call	wm_5
		call	wm_D
		call	wm_F
		call	wm_0
		call	wm_4
		call	wm_1
		call	wm_B

		movlw	B'10010100'		;Line 1 / Position 20
		call	send_comm
		call	wm_space
		call	wm_4
		call	wm_M
		call	wm_b
		return
;------------------------------------------------------------------------------
w_ads_ads2	call	wm_A
		call	wm_T
		call	wm_2
		call	wm_5
		call	wm_S
		call	wm_F
		call	wm_1
		call	wm_6
		call	wm_1

		movlw	B'10010100'		;Line 1 / Position 20
		call	send_comm
		call	wm_1
		call	wm_6
		call	wm_M
		call	wm_b
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_jedec_other	call	wm_O
		call	wm_t
		call	wm_h
		call	wm_e
		call	wm_r
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      JEDEC-ID read
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_jedec_id	call	disp_clr2
		call	disp_clr1

		call	jedec_id_read
		call	w_jedec_id


xxyx002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'3B'			;J = JEDEC ID read
		btfsc	STATUS,Z
		goto	m_jedec_id

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	xxyx002

		goto	m_spi_eem
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI EEPROM Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi_eem	call	disp_clr2
		call	disp_clr1
		call	w_spi_eeprom

xyyy002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'3B'			;J = JEDEC ID read
		btfsc	STATUS,Z
		goto	m_jedec_id

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	xyyy002

		goto	m_spi
;-------------------------------------------------------
;-------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Main Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi		call	disp_clr2
m_spi2		call	disp_clr1
		call	w_spi

yy002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'24'			;E = EEPROM (& NVRAM)
		btfsc	STATUS,Z
		goto	m_spi_eem
;;;;;		goto	spi_eeprom

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = SPI RTC
		btfsc	STATUS,Z
		goto	m_spi_rtc

		movf	kbd_buffer,0
		xorlw	H'4D'			;P = SPI Ports
		btfsc	STATUS,Z
		goto	m_spi_port

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = SPI Command Editor
		btfsc	STATUS,Z
		goto	spi_editor_st

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	yy002

		goto	mm001

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Ports Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi_port	call	disp_clr2
		call	disp_clr1
		call	w_spi_port

yy002port	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'46'			;9 = MAX6969
		btfsc	STATUS,Z
		goto	m_spi_max6969

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	yy002port

		goto	m_spi
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Ports - MAX6969
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi_max6969	call	disp_clr1
		call	disp_clr2
		call	w_spi_max6969

yy002port9	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = Demo1
		btfsc	STATUS,Z
		goto	max6969_d1

		movf	kbd_buffer,0
		xorlw	H'1E'			;2 = Demo2
		btfsc	STATUS,Z
		goto	max6969_d2

		movf	kbd_buffer,0
		xorlw	H'26'			;3 = Demo3
		btfsc	STATUS,Z
		goto	max6969_d3

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	yy002port9

		goto	m_spi
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
data1_max6969	da	B'0000000110000000'	;01
		da	B'0000001001000000'	;02
		da	B'0000010000100000'	;03
		da	B'0000100000010000'	;04
		da	B'0001000000001000'	;05
		da	B'0010000000000100'	;06
		da	B'0100000000000010'	;07
		da	B'1000000000000001'	;08
		da	B'0100000000000010'	;09
		da	B'0010000000000100'	;10
		da	B'0001000000001000'	;11
		da	B'0000100000010000'	;12
		da	B'0000010000100000'	;13
		da	B'0000001001000000'	;14
;------------------------------------------------------------
max6969_d1	movlw	high data1_max6969
		movwf	v_ad1
		movlw	low data1_max6969
		movwf	v_ad0

		movlw	D'014'
		movwf	v_eed0			;length

		goto	max6969_demo
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
data2_max6969	da	B'0000000100000000'	;01
		da	B'0000001000000000'	;02
		da	B'0000010000000000'	;03
		da	B'0000100000000000'	;04
		da	B'0001000000000000'	;05
		da	B'0010000000000000'	;06
		da	B'0100000000000000'	;07
		da	B'1000000000000000'	;08
		da	B'0000000000000001'	;09
		da	B'0000000000000010'	;10
		da	B'0000000000000100'	;11
		da	B'0000000000001000'	;12
		da	B'0000000000010000'	;13
		da	B'0000000000100000'	;14
		da	B'0000000001000000'	;15
		da	B'0000000010000000'	;16
		da	B'0000000001000000'	;17
		da	B'0000000000100000'	;18
		da	B'0000000000010000'	;19
		da	B'0000000000001000'	;20
		da	B'0000000000000100'	;21
		da	B'0000000000000010'	;22
		da	B'0000000000000001'	;23
		da	B'1000000000000000'	;24
		da	B'0100000000000000'	;25
		da	B'0010000000000000'	;26
		da	B'0001000000000000'	;27
		da	B'0000100000000000'	;28
		da	B'0000010000000000'	;29
		da	B'0000001000000000'	;30
;------------------------------------------------------------
max6969_d2	movlw	high data2_max6969
		movwf	v_ad1
		movlw	low data2_max6969
		movwf	v_ad0

		movlw	D'030'
		movwf	v_eed0			;length

		goto	max6969_demo
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
data3_max6969	da	B'0000000110000000'	;01
		da	B'0000001111000000'	;02
		da	B'0000011111100000'	;03
		da	B'0000111001110000'	;04
		da	B'0001110000111000'	;05
		da	B'0011100000011100'	;06
		da	B'0111000000001110'	;07
		da	B'1110000000000111'	;08
		da	B'1100000000000011'	;09
		da	B'1000000000000001'	;10
		da	B'0000000000000000'	;11
		da	B'1000000000000001'	;12
		da	B'1100000000000011'	;13
		da	B'1110000000000111'	;14
		da	B'0111000000001110'	;15
		da	B'0011100000011100'	;16
		da	B'0001110000111000'	;17
		da	B'0000111001110000'	;18
		da	B'0000011111100000'	;19
		da	B'0000001111000000'	;20
		da	B'0000000000000000'	;21
;------------------------------------------------------------
max6969_d3	movlw	high data3_max6969
		movwf	v_ad1
		movlw	low data3_max6969
		movwf	v_ad0

		movlw	D'021'
		movwf	v_eed0			;length

		goto	max6969_demo
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
max6969_demo	call	disp_clr1
		call	w_max6969_demo1
		call	w_m6969_dc2

max6969_demo2	movf	v_ad1,0
		movwf	TBLPTRH
		movf	v_ad0,0
		movwf	TBLPTRL
		movf	v_eed0,0
		movwf	v_eed1

max6969_demo_lp	call	max6969_send
		call	d2
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

ppswaitclk	btfss	PORTB,D'007'
		goto	ppswaitclk

ppswaitdat	btfss	PORTB,D'006'
		goto	ppswaitdat

ppswaitclk2	btfsc	PORTB,D'007'
		goto	ppswaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC -> exit
		btfsc	STATUS,Z
		goto	max6969_de
;-----------------------------------------------------
;-----------------------------------------------------
		decfsz	v_eed1,1
		goto	max6969_demo_lp
		goto	max6969_demo2
;-----------------------------------------------------
max6969_de	goto	m_spi_max6969
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
max6969_send	call	spi_cs0_en

		tblrd*+
		movf	TABLAT,0
		call	spi_send

		movlw	B'11001100'
		call	send_comm
		call	w_disp_dat
;------------------------------------------
		tblrd*+
		movf	TABLAT,0
		call	spi_send

		movlw	B'11000100'
		call	send_comm
		call	w_disp_dat

		movlw	B'11011000'
		call	send_comm		;just hide the cursor

		call	spi_cs_all_dis
		return
;---------------------------------------------------------------------------
w_disp_dat	btfss	TABLAT,D'000'
		call	wm_minus
		btfsc	TABLAT,D'000'
		call	wm_X

		btfss	TABLAT,D'001'
		call	wm_minus
		btfsc	TABLAT,D'001'
		call	wm_X

		btfss	TABLAT,D'002'
		call	wm_minus
		btfsc	TABLAT,D'002'
		call	wm_X

		btfss	TABLAT,D'003'
		call	wm_minus
		btfsc	TABLAT,D'003'
		call	wm_X

		btfss	TABLAT,D'004'
		call	wm_minus
		btfsc	TABLAT,D'004'
		call	wm_X

		btfss	TABLAT,D'005'
		call	wm_minus
		btfsc	TABLAT,D'005'
		call	wm_X

		btfss	TABLAT,D'006'
		call	wm_minus
		btfsc	TABLAT,D'006'
		call	wm_X

		btfss	TABLAT,D'007'
		call	wm_minus
		btfsc	TABLAT,D'007'
		call	wm_X

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
w_m6969_dc2	movlw	B'11000000'
		call	send_comm
		movlw	'>'
		call	send_data
		movlw	'>'
		call	send_data
		movlw	'>'
		call	send_data

		movlw	B'11010101'
		call	send_comm
		movlw	'<'
		call	send_data
		movlw	'<'
		call	send_data
		movlw	'<'
		call	send_data
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
w_max6969_demo1	movlw	B'10000000'
		call	send_comm
		movlw	'('
		call	send_data
		call	wm_E
		call	wm_S
		call	wm_C
		movlw	')'
		call	send_data
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI RTC Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi_rtc	call	disp_clr2
		call	disp_clr1
		call	w_spi_rtc

yy002rtc	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'23'			;D = DS1306 - SPI RTC
		btfsc	STATUS,Z
		goto	spi_ds1306

		movf	kbd_buffer,0
		xorlw	H'4D'			;P = PCA2129 - RTC configured with SPI
		btfsc	STATUS,Z
		goto	spi_pca2129

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	yy002rtc

		goto	m_spi
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;PCA2129 - SPI RTC
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
spi_pca2129	bsf	v_isl12022,D'002'	;PCA2129 is active
		goto	spi_rtc_in

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;DS1306 - SPI RTC
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
spi_ds1306	bcf	v_isl12022,D'002'
spi_rtc_in	bsf	v_isl12022,D'001'	;DS1306 is in use
		btfsc	v_xxx_conf,D'000'
		call	AMS_init

		call	disp_clr1
		call	disp_clr2
		call	w_rtcm

;----------------- RTC get data -----------------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
		call	d55

rtc_ml_ds1306	btfsc	v_isl12022,D'002'
		call	rtc_spi_read2		;read PCA2129
		btfss	v_isl12022,D'002'
		call	rtc_spi_read		;read DS1306
;
		call	rtc_translate			;translate RTC date from RTC device to display format

		call	w_rtcm_dt

		btfsc	v_xxx_conf,D'000'
		call	w_ams_rtc			;send the clock data to AMS display driver via SPI
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

dswaitclk	btfss	PORTB,D'007'
		goto	dswaitclk

dswaitdat	btfss	PORTB,D'006'
		goto	dswaitdat

dswaitclk2	btfsc	PORTB,D'007'
		goto	dswaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S - Set Date / time - keyboard input
		btfsc	STATUS,Z
		goto	rtc_set

		movf	kbd_buffer,0
		xorlw	H'24'			;E - Set Date / time example
		btfsc	STATUS,Z
		goto	rtc_set_e

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC -> exit
		btfsc	STATUS,Z
		goto	dstemp_exit

		goto	rtc_ml_ds1306

dstemp_exit	bcf	v_isl12022,D'001'	;clear the flag of usage of DS1306
		bcf	v_isl12022,D'002'	;clear the flag of usage of PCA2129
		btfsc	v_xxx_conf,D'000'
		call	w_ams_minus		;send the "minus" to AMS display driver via SPI

		goto	m_spi_rtc
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
spi_eeprom	movlw	H'81'			;SPI EEPROM 23LC512
		movwf	v_ee_type
		call	spi_reinit2
		goto	eeprom_disp
;-------------------------------------------------------
;-------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_par	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_par1
		movwf	TBLPTRH
		movlw	low t_spi_par1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_par2
		movwf	TBLPTRH
		movlw	low t_spi_par2
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_par2	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_par2_1
		movwf	TBLPTRH
		movlw	low t_spi_par2_1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_par2_2
		movwf	TBLPTRH
		movlw	low t_spi_par2_2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'10010001'		;row: 1 / col: 17
		call	send_comm

		btfss	v_spi_conf,D'000'
		goto	w_spi_par2_X

		btfss	v_spi_conf,D'001'
		goto	w_spi_par2_0
		call	wm_1

w_spi_par2_f	movlw	B'10100000'		;row: 1 / col: far away
		call	send_comm
		return
;---------------------------------------------------
w_spi_par2_X	call	wm_X
		goto	w_spi_par2_f
;---------------------------------------------------
w_spi_par2_0	call	wm_0
		goto	w_spi_par2_f
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_par1	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_par1_1
		movwf	TBLPTRH
		movlw	low t_spi_par1_1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_par1_2
		movwf	TBLPTRH
		movlw	low t_spi_par1_2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'10001100'		;row: 1 / col: 12
		call	send_comm

		movlw	'0'
		btfsc	v_spi_conf,D'004'
		movlw	'1'
		call	send_data

		movlw	B'11001100'		;row: 2 / col: 12
		call	send_comm

		btfss	v_spi_conf,D'006'
		call	w_spi_ia
		btfsc	v_spi_conf,D'006'
		call	w_spi_ai

		return
;------------------------------------------------------------------------------
w_spi_ai	call	wm_A
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_v
		call	wm_e
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_I
		call	wm_d
		call	wm_l
		call	wm_e
		return
;------------------------------------------------------------------------------
w_spi_ia	call	wm_I
		call	wm_d
		call	wm_l
		call	wm_e
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_A
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_v
		call	wm_e
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_t_test	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_tt1
		movwf	TBLPTRH
		movlw	low t_spi_tt1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_spi_tt2
		movwf	TBLPTRH
		movlw	low t_spi_tt2
		movwf	TBLPTRL
		call	w_send_txt

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Parameters Setup - Entry
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
s_spi_par	call	disp_clr1
		call	disp_clr2
		call	w_spi_par

smpp002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = SPI Parameters 1
		btfsc	STATUS,Z
		goto	s_spi_par1

		movf	kbd_buffer,0
		xorlw	H'1E'			;2 = SPI Parameters 2
		btfsc	STATUS,Z
		goto	s_spi_par2

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	smpp002

		call	spi_reinit1		;use selected parameters
		goto	m_setup
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Parameters Setup - Screen 2
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
s_spi_par2	call	disp_clr1
		call	disp_clr2
s_spi_par2_1	call	w_spi_par2

spp005		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = Idle Value for SDI Line
		btfsc	STATUS,Z
		goto	s_idle_sdi

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	spp005

		call	spi_reinit1		;use selected parameters
		goto	s_spi_par
;------------------------------------------------------------------------------
s_idle_sdi	btfss	v_spi_conf,D'000'
		goto	s_idle_sdi_0
		btfss	v_spi_conf,D'001'
		goto	s_idle_sdi_1
;off
		bcf	v_spi_conf,D'000'
		bcf	v_spi_conf,D'001'
		goto	s_spi_par2_1
;------------------------------------------------------------------------------
s_idle_sdi_0	bsf	v_spi_conf,D'000'
		bcf	v_spi_conf,D'001'
		goto	s_spi_par2_1
;------------------------------------------------------------------------------
s_idle_sdi_1	bsf	v_spi_conf,D'001'
		goto	s_spi_par2_1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Parameters Setup - Screen 1
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
s_spi_par1	call	disp_clr1
		call	disp_clr2
s_spi_par1_2	call	w_spi_par1

spp002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = Idle Clock State toggle
		btfsc	STATUS,Z
		goto	s_spi_icst

		movf	kbd_buffer,0
		xorlw	H'2C'			;T = Transmit time toggle
		btfsc	STATUS,Z
		goto	s_spi_ttt

		movf	kbd_buffer,0
		xorlw	H'24'			;E = Transmision test
		btfsc	STATUS,Z
		goto	s_spi_t_test

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	spp002

		call	spi_reinit1		;use selected parameters
		goto	s_spi_par

;------------------------------------------------------------------------------
s_spi_icst	btg	v_spi_conf,D'004'
		call	d2
		call	d2
		goto	s_spi_par1_2
;------------------------------------------------------------------------------
s_spi_ttt	btg	v_spi_conf,D'006'
		call	d2
		call	d2
		goto	s_spi_par1_2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_spi_t_test	call	spi_reinit1		;use selected parameters
		call	disp_clr1
		call	disp_clr2
		call	w_spi_t_test

s_spi_t_test_e	movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send
		movlw	B'10101010'
		call	spi_send

		call	d2

		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
		movlw	B'01010101'
		call	spi_send
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

uriwaitclk	btfss	PORTB,D'007'
		goto	uriwaitclk

uriwaitdat	btfss	PORTB,D'006'
		goto	uriwaitdat

uriwaitclk2	btfsc	PORTB,D'007'
		goto	uriwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC -> exit
		btfsc	STATUS,Z
		goto	s_spi_par1

		goto	s_spi_t_test_e
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Setup - Others
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
s_others	call	disp_clr1
		call	disp_clr2
s_others_001	call	w_s_others

s_others_002	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1C'			;A = Toggle RTC output to AMS
		btfsc	STATUS,Z
		goto	s_rtc_ams

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	s_others_002

		goto	m_setup
;------------------------------------------------------------------------------
s_rtc_ams	btg	v_xxx_conf,D'000'
		call	d2
		call	d2
		goto	s_others_001	
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_s_others	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_s_others1
		movwf	TBLPTRH
		movlw	low t_s_others1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'10010101'		;row: 1 / col: 21
		call	send_comm
		btfss	v_xxx_conf,D'000'
		goto	w_s_o_off
		call	wm_o
		call	wm_n
		call	wm_space
		return
;----------------------------------------------
w_s_o_off	call	wm_o
		call	wm_f
		call	wm_f
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      MSSP Board Setup Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_setup		call	disp_clr1
		call	disp_clr2
		call	w_brd_setup

ms002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = I2C Chanel Selection
		btfsc	STATUS,Z
		goto	s_i2ccs

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = SPI parameters
		btfsc	STATUS,Z
		goto	s_spi_par

		movf	kbd_buffer,0
		xorlw	H'44'			;O = Others
		btfsc	STATUS,Z
		goto	s_others

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	ms002

		goto	mm001


s_i2ccs		call	i2c_setup
		goto	m001			;jump to I2C Main menu
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      I2C Main entry screen
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m001		call	disp_clr1
		call	disp_clr2
		call	w_m001

m002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'32'			;B = Basic functions / circuits
		btfsc	STATUS,Z
		goto	main_escs

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = Special functions
		btfsc	STATUS,Z
		goto	sic001

		movf	kbd_buffer,0
		xorlw	H'2C'			;T = very special functions (Test)
		btfsc	STATUS,Z
		goto	test_fnc01

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	m002

		goto	mm001


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Special Circuits Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sic001		nop
		call	disp_clr1
		call	disp_clr2
		call	w_sic001

sic002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'		;I = INA219
		btfsc	STATUS,Z
		goto	ina219

		movf	kbd_buffer,0
		xorlw	H'24'		;E = I2C Command Editor
		btfsc	STATUS,Z
		goto	i2c_editor

		movf	kbd_buffer,0
		xorlw	H'23'		;D = I2C Device List
		btfsc	STATUS,Z
		goto	i2c_dev_list

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	sic002

		goto	m001

; --------------- dummies -------------
ina219		goto	sic001

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;                          I2C Device List Screens
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_dl_nff	movlw	B'10000000'
		call	send_comm
		movlw	high t_dl_nff
		movwf	TBLPTRH
		movlw	low t_dl_nff
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_dl_search	movlw	B'10000000'
		call	send_comm
		movlw	high t_dl_s1
		movwf	TBLPTRH
		movlw	low t_dl_s1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'
		call	send_comm
		movlw	high t_dl_s2
		movwf	TBLPTRH
		movlw	low t_dl_s2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_dev_list	movlw	B'10000000'
		call	send_comm
		call	wm_A
		call	wm_d
		call	wm_d
		call	wm_dvojbodka
		call	wm_space

		movf	v_i2c_add,0
		movwf	kbd_buffer
		call	write_bin7

		call	wm_space
		call	wm_zat1
		movf	v_i2c_actd,0
		movwf	kbd_buffer
		call	write_dec
		call	wm_sls
		movf	v_i2c_devs,0
		movwf	kbd_buffer
		call	write_dec
		call	wm_zat2

		movlw	B'11000000'
		call	send_comm
		call	wm_T
		call	wm_y
		call	wm_p
		call	wm_e
		call	wm_dvojbodka
		call	wm_space

		call	w_i2c_dev_type

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;recognized device types based on I2c address:
;	1010xxx		EEPROM
;	0100xxx		Port Expander
;	0111xxx		Port Expander
;	1110xxx		I2C Switch (e.g. PCA9546)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_i2c_dev_type	nop

;1st - check demo board internal chips -------------
		movf	v_i2c_add,0
		andlw	B'11111110'
		xorlw	B'00111110'
		btfsc	STATUS,Z
		goto	w_dti_PCA6107

		movf	v_i2c_add,0
		andlw	B'11111110'
		xorlw	B'01001110'
		btfsc	STATUS,Z
		goto	w_dti_PCF8574

		movf	v_i2c_add,0
		andlw	B'11111110'
		xorlw	B'11101110'
		btfsc	STATUS,Z
		goto	w_dti_PCA9546

;---------------------------------------------------

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'10100000'
		btfsc	STATUS,Z
		goto	w_dt_1010xxx

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'01000000'
		btfsc	STATUS,Z
		goto	w_dt_0100xxx

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'01110000'
		btfsc	STATUS,Z
		goto	w_dt_0100xxx

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'11100000'
		btfsc	STATUS,Z
		goto	w_dt_1110xxx

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'10010000'
		btfsc	STATUS,Z
		goto	w_dt_1001xxx

		movf	v_i2c_add,0
		andlw	B'11110000'
		xorlw	B'00110000'
		btfsc	STATUS,Z
		goto	w_dt_0011xxx

;unknown I2C device -----------------------------
		movlw	high t_dt_ukn
		movwf	TBLPTRH
		movlw	low t_dt_ukn
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_dti_PCA6107	movlw	B'11000000'
		call	send_comm

		movlw	high t_dti_PCA6107
		movwf	TBLPTRH
		movlw	low t_dti_PCA6107
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_dti_PCF8574	movlw	B'11000000'
		call	send_comm

		movlw	high t_dti_PCF8574
		movwf	TBLPTRH
		movlw	low t_dti_PCF8574
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_dti_PCA9546	movlw	B'11000000'
		call	send_comm

		movlw	high t_dti_PCA9546
		movwf	TBLPTRH
		movlw	low t_dti_PCA9546
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;I2C Switch
w_dt_1110xxx	movlw	high t_dt_1110xxx
		movwf	TBLPTRH
		movlw	low t_dt_1110xxx
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;Temperature Sensor
w_dt_1001xxx	movlw	high t_dt_1001xxx
		movwf	TBLPTRH
		movlw	low t_dt_1001xxx
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;EEPROM
w_dt_1010xxx	movlw	high t_dt_1010xxx
		movwf	TBLPTRH
		movlw	low t_dt_1010xxx
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;Ports / Group A
w_dt_0100xxx	movlw	high t_dt_0100xxx
		movwf	TBLPTRH
		movlw	low t_dt_0100xxx
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
;Ports / Group B
w_dt_0011xxx	movlw	high t_dt_0011xxx
		movwf	TBLPTRH
		movlw	low t_dt_0011xxx
		movwf	TBLPTRL
		call	w_send_txt
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;                          I2C Device List
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
i2c_dev_list	call	disp_clr1
		call	disp_clr2

		call	w_dl_search

		movlw	B'00000010'
		movwf	v_i2c_add

		clrf	v_i2c_devs

		lfsr	FSR0, H'200'		;RAM Bank 2 to store the I2C Adresses of Devices found

i2c_srch_next	movlw	B'10001100'
		call	send_comm
		movf	v_i2c_add,0
		movwf	kbd_buffer
		call	write_bin7

		call	i2c_start
		movf	v_i2c_add,0
		call	i2c_send
		btfss	SSP1CON2,ACKSTAT
		call	i2c_dev_found
		call	i2c_stop

		incf	v_i2c_add,1
		incf	v_i2c_add,1
		movf	v_i2c_add,0
		btfsc	STATUS,Z
		goto	i2c_srch_fin
		goto	i2c_srch_next
;-----------------------------------------------------------------------------
i2c_srch_fin	call	disp_clr1
		call	disp_clr2

		movf	v_i2c_devs,0
		btfsc	STATUS,Z
		goto	i2csf02

i2c_dl_1	movlw	D'001'
		movwf	v_i2c_actd
		lfsr	FSR0, H'200'

i2c_dl_2	movf	POSTINC0,0
		movwf	v_i2c_add

		call	w_dev_list

i2csf01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER - next device
		btfsc	STATUS,Z
		goto	i2c_dl_next

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	i2csf01

		goto	sic001
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
i2c_dl_next	call	d2
		call	d2
		movf	v_i2c_actd,0
		xorwf	v_i2c_devs,0
		btfsc	STATUS,Z
		goto	i2c_dl_1
		incf	v_i2c_actd,1
		goto	i2c_dl_2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;no I2C device found
;
i2csf02		call	w_dl_nff

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	i2csf02

		goto	sic001
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c_dev_found	nop
;-- store the I2C address ----------------
		movf	v_i2c_add,0
		movwf	POSTINC0
;-----------------------------------------
		movlw	B'11001111'
		call	send_comm

		incf	v_i2c_devs,1
		movf	v_i2c_devs,0
		movwf	kbd_buffer
		call	write_dec
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          I2C Command Line Editor
;-----------------------------------------------------------------------------
;Commands:
;     S -- Generate Start Condition
;     E -- Send Byte
;     R -- Receive Byte
;     A -- Generate ACK
;     N -- Generate NOT ACK
;     P -- Generate Stop Condition

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

i2c_editor	clrf	i2ce_stage		;I2C closed
		clrf	i2ce_op			;no operation
		movlw	D'001'
		movwf	i2ce_cursor

		call	disp_clr2
i2ce_m3		call	disp_clr1
		call	w_i2c_edit

i2ce_m		call	w_i2ce_curr

i2ce_m2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'		;S = Start
		btfsc	STATUS,Z
		goto	i2ce_st

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = Stop
		btfsc	STATUS,Z
		goto	i2ce_sp

		movf	kbd_buffer,0
		xorlw	H'1C'		;A = ACK
		btfsc	STATUS,Z
		goto	i2ce_ak

		movf	kbd_buffer,0
		xorlw	H'31'		;N = NOT ACK
		btfsc	STATUS,Z
		goto	i2ce_nk

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = Receive
		btfsc	STATUS,Z
		goto	i2ce_rc

		movf	kbd_buffer,0
		xorlw	H'24'		;E = Send
		btfsc	STATUS,Z
		goto	i2ce_sn

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	i2ce_m2

		goto	sic001
;-------------------------------------------------------------------------------
i2ce_sn		btfss	i2ce_stage,D'000'
		goto	i2ce_m

		call	wm_E
		call	wm_zat1

		call	d2
		call	d2

		clrf	v_b_in
		movf	i2ce_cursor,0
		addlw	B'11000010'
		call	hex_input
		movf	v_b_in,0
		call	i2c_send
		btfss	i2ce_stage,D'001'
		call	i2ce_store_op
		call	d2
		call	wm_zat2
		call	i2ce_mcc6
		goto	i2ce_m
;-------------------------------------------------------------------------------
i2ce_store_op	movf	v_b_in,0
		movwf	i2ce_op
		bsf	i2ce_stage,D'001'
		return
;-------------------------------------------------------------------------------
i2ce_rc		call	wm_R
		call	i2c_receive
		movwf	kbd_buffer
		call	d2
		call	wm_zat1
		call	write_hex
		call	wm_zat2
		call	i2ce_mcc6
		goto	i2ce_m
;-------------------------------------------------------------------------------
i2ce_nk		call	wm_N
		call	i2c_not_ack
		call	d2
		call	i2ce_mcc2
		goto	i2ce_m
;-------------------------------------------------------------------------------
i2ce_ak		call	wm_A
		call	i2c_ack
		call	d2
		call	i2ce_mcc2
		goto	i2ce_m
;-------------------------------------------------------------------------------
i2ce_sp		call	wm_P
		call	i2c_stop
		bcf	i2ce_stage,D'000'
		bcf	i2ce_stage,D'001'
		call	d2
		call	i2ce_mcc2
		goto	i2ce_m3
;-------------------------------------------------------------------------------
i2ce_st		call	wm_S
		call	i2c_start
		bsf	i2ce_stage,D'000'
		bcf	i2ce_stage,D'001'
		call	d2
		call	i2ce_mcc2
		goto	i2ce_m3
;-------------------------------------------------------------------------------
;move cursor

i2ce_mcc6	movlw	D'004'
		addwf	i2ce_cursor,1

i2ce_mcc2	movlw	D'002'
		addwf	i2ce_cursor,1

;we want to have at least 5 characters left
		movf	i2ce_cursor,0
		movwf	data_tmp
		movlw	D'011'			;5 needs to be left, but we need 6 more (to be able to compare bit 5...
		addwf	data_tmp,1

		btfss	data_tmp,D'005'		;we passed the column 24...
		goto	i2ce_mcc_f

		call	d2
		call	d2
		movlw	D'001'
		movwf	i2ce_cursor
		call	disp_clr2
		movlw	B'11000000'
		call	send_comm

i2ce_mcc_f	call	wm_sip1
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                      I2C Editor main screen                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_i2c_edit	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_i2ce1
		movwf	TBLPTRH
		movlw	low t_i2ce1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_sip1
		return

;---------------------------------------------------------------
w_i2ce_curr	movlw	B'10000110'
		call	send_comm
		movlw	'#'
		call	send_data
		btfss	i2ce_stage,D'000'
		goto	w_i2ce_closed

		call	wm_o
		call	wm_p
		call	wm_e
		call	wm_n
		btfsc	i2ce_stage,D'001'
		goto	w_i2ce_op
		call	wm_space
		call	wm_space

w_i2ce_curr2	movf	i2ce_cursor,0
		addlw	B'11000000'
		call	send_comm

		return

;------------------------------------------------
w_i2ce_closed	call	wm_c
		call	wm_l
		call	wm_o
		call	wm_s
		call	wm_e
		call	wm_d

		goto	w_i2ce_curr2

;------------------------------------------------
w_i2ce_op	call	wm_minus
		btfss	i2ce_op,D'000'
		goto	w_i2ce_opw
		call	wm_R
		call	wm_D

W_i2ce_op2	call	wm_sls
		movf	i2ce_op,0
		movwf	kbd_buffer
		call	write_bin7

		goto	w_i2ce_curr2


w_i2ce_opw	call	wm_W
		call	wm_R
		goto	W_i2ce_op2
;-------------------------------------------------------------------------------
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                          1-Wire Main Menu                                   X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_main_menu	nop
		call	disp_clr1
		call	disp_clr2

		call	w_ow_mm

ow_mm		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Basic Functions
		btfsc	STATUS,Z
		goto	ow_bafu

		movf	kbd_buffer,0
		xorlw	H'23'		;D = Device Selection
		btfsc	STATUS,Z
		goto	ow_dev_sel

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = ROM Search
		btfsc	STATUS,Z
		goto	ow_rom_search1

		movf	kbd_buffer,0
		xorlw	H'44'		;O = ROM Search "with debugging"
		btfsc	STATUS,Z
		goto	ow_rom_search2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	mm001

		goto	ow_mm
;-------------------------------------------------------------------------------
ow_rom_search1	clrf	v_ows_debug
		goto	ow_rom_search
;-------------------------------------------------------------------------------
ow_rom_search2	bsf	v_ows_debug,D'000'
		goto	ow_rom_search
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; "Debugging Screen for 1-wire ROM Search functionality
;
; first  line shows the last found ROM ID
; second line shows the conflict points
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_tmp_stage	movlw	D'128'
		call	send_comm

		call	wm_dvojbodka

		movf	ow_ROM_7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_6,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_5,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_4,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_3,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	ow_ROM_0,0
		movwf	kbd_buffer
		call	write_hex

		movlw	D'192'
		call	send_comm

		call	wm_hviezdicka

		movf	v_owc7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc6,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc5,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc4,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc3,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_owc0,0
		movwf	kbd_buffer
		call	write_hex

;----------------------------------------------------------------------
ow_tmp_n1	clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER - next device
		btfsc	STATUS,Z
		return

		goto	ow_tmp_n1
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_execute	clrf	v_ow_dc

		clrf	v_owc0
		clrf	v_owc1
		clrf	v_owc2
		clrf	v_owc3
		clrf	v_owc4
		clrf	v_owc5
		clrf	v_owc6
		clrf	v_owc7

		clrf	v_owd0
		clrf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7

		clrf	v_owdc

		lfsr	FSR0, H'200'
;----------------------------------------------------
ow_rs_next	call	ow_rst
		movlw	H'F0'			;Search ROM
		movwf	ow_buffer
		call	ow_write

		movf	v_owd0,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_0
		movf	v_owcc,0
		movwf	v_owc0


		movf	v_owd1,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_1
		movf	v_owcc,0
		movwf	v_owc1


		movf	v_owd2,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_2
		movf	v_owcc,0
		movwf	v_owc2

		movf	v_owd3,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_3
		movf	v_owcc,0
		movwf	v_owc3

		movf	v_owd4,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_4
		movf	v_owcc,0
		movwf	v_owc4

		movf	v_owd5,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_5
		movf	v_owcc,0
		movwf	v_owc5


		movf	v_owd6,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_6
		movf	v_owcc,0
		movwf	v_owc6


		movf	v_owd7,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_7
		movf	v_owcc,0
		movwf	v_owc7

;------------------------------------------------------------
		movf	v_ow_dc,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		return				;no device
;-----------------------------------------------------------

		call	rom_s_store

		btfsc	v_ows_debug,D'000'
		call	w_tmp_stage

;-----------------------------------------------------------
;now find last conflict and change the corresponding direction

		movf	v_owd7,0
		movwf	v_owdc
		movf	v_owc7,0
		btfss	STATUS,Z
		goto	ow_rs_next7
		movf	v_owd6,0
		movwf	v_owdc
		movf	v_owc6,0
		btfss	STATUS,Z
		goto	ow_rs_next6
		movf	v_owd5,0
		movwf	v_owdc
		movf	v_owc5,0
		btfss	STATUS,Z
		goto	ow_rs_next5
		movf	v_owd4,0
		movwf	v_owdc
		movf	v_owc4,0
		btfss	STATUS,Z
		goto	ow_rs_next4
		movf	v_owd3,0
		movwf	v_owdc
		movf	v_owc3,0
		btfss	STATUS,Z
		goto	ow_rs_next3
		movf	v_owd2,0
		movwf	v_owdc
		movf	v_owc2,0
		btfss	STATUS,Z
		goto	ow_rs_next2
		movf	v_owd1,0
		movwf	v_owdc
		movf	v_owc1,0
		btfss	STATUS,Z
		goto	ow_rs_next1
		movf	v_owd0,0
		movwf	v_owdc
		movf	v_owc0,0
		btfss	STATUS,Z
		goto	ow_rs_next0

;search finished
		call	ow_rst

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_next7	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next6	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next5	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next4	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next3	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next2	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next1	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next0	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd0
		clrf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_nextc	btfsc	v_owcc,D'007'
		goto	ow_rs_nextcb7
		btfsc	v_owcc,D'006'
		goto	ow_rs_nextcb6
		btfsc	v_owcc,D'005'
		goto	ow_rs_nextcb5
		btfsc	v_owcc,D'004'
		goto	ow_rs_nextcb4
		btfsc	v_owcc,D'003'
		goto	ow_rs_nextcb3
		btfsc	v_owcc,D'002'
		goto	ow_rs_nextcb2
		btfsc	v_owcc,D'001'
		goto	ow_rs_nextcb1
		goto	ow_rs_nextcb0
;------------------------------------------
ow_rs_nextcb7	bsf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb6	bsf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb5	bsf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb4	bsf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb3	bsf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb2	bsf	v_owdc,D'002'
		bcf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb1	bsf	v_owdc,D'001'
		bcf	v_owdc,D'002'
		bcf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb0	clrf	v_owdc
		bsf	v_owdc,D'000'
		movf	v_owdc,0
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
rom_s_store	incf	v_ow_dc,1		;increment "devices found"

		call	w_ows_prog_i

;store the device ROM ID

		movf	ow_ROM_0,0
		movwf	v_owd0
		movwf	POSTINC0

		movf	ow_ROM_1,0
		movwf	v_owd1
		movwf	POSTINC0

		movf	ow_ROM_2,0
		movwf	v_owd2
		movwf	POSTINC0

		movf	ow_ROM_3,0
		movwf	v_owd3
		movwf	POSTINC0

		movf	ow_ROM_4,0
		movwf	v_owd4
		movwf	POSTINC0

		movf	ow_ROM_5,0
		movwf	v_owd5
		movwf	POSTINC0

		movf	ow_ROM_6,0
		movwf	v_owd6
		movwf	POSTINC0

		movf	ow_ROM_7,0
		movwf	v_owd7
		movwf	POSTINC0

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; ROM Search Progress Indicator
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ows_prog_i	movlw	D'128'
		call	send_comm

		call	wm_D
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e
		call	wm_s
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_u
		call	wm_n
		call	wm_d
		call	wm_dvojbodka
		call	wm_space

		movf	v_ow_dc,0
		movwf	kbd_buffer
		call	write_dec

		call	wm_space
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_byte	nop
		clrf	v_rom_id_c
		clrf	v_owcc

;bit 0 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'000'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfss	ds2484_st,D'005'
		goto	x567_2
		btfsc	ds2484_st,D'006'
		goto	ow_no_device

x567_2		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'000'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'000'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'000'

		btfsc	v_owdc,D'000'
		bcf	v_owcc,D'000'

;bit 1 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'001'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'001'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'001'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'001'

		btfsc	v_owdc,D'001'
		bcf	v_owcc,D'001'
;bit 2 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'002'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'002'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'002'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'002'

		btfsc	v_owdc,D'002'
		bcf	v_owcc,D'002'
;bit 3 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'003'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'003'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'003'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'003'

		btfsc	v_owdc,D'003'
		bcf	v_owcc,D'003'
;bit 4 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'004'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'004'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'004'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'004'

		btfsc	v_owdc,D'004'
		bcf	v_owcc,D'004'
;bit 5 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'005'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'005'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'005'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'005'

		btfsc	v_owdc,D'005'
		bcf	v_owcc,D'005'
;bit 6 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'006'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'006'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'006'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'006'

		btfsc	v_owdc,D'006'
		bcf	v_owcc,D'006'
;bit 7 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'007'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'007'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'007'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'007'

		btfsc	v_owdc,D'007'
		bcf	v_owcc,D'007'
;----------------------------------------------------
x999_f		movf	v_rom_id_c,0
		return
;----------------------------------------------------
ow_no_device	movlw	H'FF'
		movwf	v_ow_dc
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire ROM Search                                     X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;subroutines description:
;
;	ow_rs_execute - scan the 1-wire bus and store ROM IDs into
;                       memory bank 2
;                       each ROM ID needs 8 bytes of memory
;                     - Process:
;                          0. the initialization is done:
;                             all direction bits are set to 0
;                          1. 1-wire reset is executed
;                          2. 1-wire ROM Search command is executed
;                          3. 8 x ow_rs_byte is executed starting with LSB
;			   4. first (on the MSb) conflict is found and the
;                             corresponding direction bit is changed from 0 to 1
;                               - if there is no conflict anymore -- search is finished
;                          5. the cycle is executed again from point 1
;
;
;	ow_rs_byte    - execute the scan for one byte of the ROM ID
;                     - for one cycle this soubroutine is executed 8 times
;                     - internaly the routine ow_triplet is used for every bit
;                     - means, the ow_triplet is called 8 times from this routine
;                     - variable v_owdc defines the direction
;                           - is filled with the corresponding v_owd0 - v_owd7
;                             prior to call
;                     - variable v_owcc contains after the execution "list" of
;                       conflicts - for every bit where at least two devices
;                       response with different bit, the corresponding bit
;                       will be set to 1 - but only if the corresponding
;                       direction bit is 0
;       rom_s_store   - sotred the ROM ID to memory bank 2 
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rom_search	nop
		call	disp_clr1
		call	disp_clr2

		call	ow_rs_execute
		movf	v_ow_dc,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	ow_rs_nd

		call	disp_clr1
		call	disp_clr2

owms3		clrf	v_ow_curr
		incf	v_ow_curr,1

		lfsr	FSR0, H'200'
;--------------------------------------------
owms2		call	w_ow_device

ow_rs_1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER
		btfsc	STATUS,Z
		goto	ow_xx_next

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_rs_1
;----------------------------------------------------------------------
ow_xx_next	movf	v_ow_curr,0
		xorwf	v_ow_dc,0
		btfsc	STATUS,Z
		goto	owms3
;
		incf	v_ow_curr,1
		goto	owms2
;----------------------------------------------------------------------
;----------------------------------------------------------------------
ow_rs_nd	nop
		movlw	B'10000000'
		call	send_comm
		call	wm_N
		call	wm_o
		call	wm_space
		call	wm_d
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_u
		call	wm_n
		call	wm_d

ow_rs_n1	clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_rs_n1
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ow_DS18B20	call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_B
		call	wm_2
		call	wm_0
		return
;-------------------------------------------------------------------------------
w_ow_DS18S20	call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_S
		call	wm_2
		call	wm_0
		return
;-------------------------------------------------------------------------------
w_ow_DS1822	call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_2
		call	wm_2
		call	wm_space
		return
;-------------------------------------------------------------------------------
w_ow_DS2401	call	wm_D
		call	wm_S
		call	wm_2
		call	wm_4
		call	wm_0
		call	wm_1
		call	wm_space
		return
;-------------------------------------------------------------------------------
w_ow_DS24B33	call	wm_D
		call	wm_S
		call	wm_2
		call	wm_4
		call	wm_B
		call	wm_3
		call	wm_3
		return
;-------------------------------------------------------------------------------
w_ow_device	movf	POSTINC0,0
		movwf	ow_ROM_0
		movf	POSTINC0,0
		movwf	ow_ROM_1
		movf	POSTINC0,0
		movwf	ow_ROM_2
		movf	POSTINC0,0
		movwf	ow_ROM_3
		movf	POSTINC0,0
		movwf	ow_ROM_4
		movf	POSTINC0,0
		movwf	ow_ROM_5
		movf	POSTINC0,0
		movwf	ow_ROM_6
		movf	POSTINC0,0
		movwf	ow_ROM_7

;device name -------------------------------
		movlw	B'10000000'
		call	send_comm

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'10000000'
		call	send_comm

		movf	ow_ROM_0,0
		xorlw	H'01'
		btfsc	STATUS,Z
		call	w_ow_DS2401

		movf	ow_ROM_0,0
		xorlw	H'10'
		btfsc	STATUS,Z
		call	w_ow_DS18S20

		movf	ow_ROM_0,0
		xorlw	H'22'
		btfsc	STATUS,Z
		call	w_ow_DS1822

		movf	ow_ROM_0,0
		xorlw	H'23'
		btfsc	STATUS,Z
		call	w_ow_DS24B33

		movf	ow_ROM_0,0
		xorlw	H'28'
		btfsc	STATUS,Z
		call	w_ow_DS18B20
;--------------------------------------------
		movlw	B'10001111'		;line 1 / position 15
		call	send_comm

		call	wm_zat1
		movf	v_ow_curr,0
		movwf	kbd_buffer
		call	write_dec
		call	wm_sls
		movf	v_ow_dc,0
		movwf	kbd_buffer
		call	write_dec
		call	wm_zat2

;Line 2 - ROM ID
		call	w_ds1820i2
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire Basic Functions                                X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_bafu		nop
		call	disp_clr1
		call	disp_clr2

		call	w_ow_bafu

ow_bafu_mm	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'32'			;B = Bridge Registers (DS2484)
		btfsc	STATUS,Z
		goto	onew_s

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = Read ROM id
		btfsc	STATUS,Z
		goto	ow_rom_id

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_bafu_mm

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire Device Selection                               X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_dev_sel	nop
		call	disp_clr1
		call	disp_clr2

		call	w_ow_dev_sel

ow_dsm		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'45'		;0 = DS18S20
		btfsc	STATUS,Z
		goto	ds1820_1

		movf	kbd_buffer,0
		xorlw	H'36'		;6 = DS2406
		btfsc	STATUS,Z
		goto	ds2406

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to level above
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_dsm

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                1-Wire Bridge Registers / DS2484                             X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
onew_s		nop

		call	disp_clr1
		call	disp_clr2

ds2484_r	call	ds2484_i2c_read
		call	w_ds2484

ds2484_m	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read
		btfsc	STATUS,Z
		goto	ds2484_r

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_bafu

		goto	ds2484_m

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                            CRC Check Execute 1                              X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;DOW CRC Check using look-up table
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
crc_check1_exec	nop
		movlw	low ow_crc_lt		;this should be 00h
		movwf	TBLPTRL

		clrf	ow_crc_calc		;starting value for computing is 0

		movf	ow_ROM_0,0
		call	ow_crc_bc1
		movf	ow_ROM_1,0
		call	ow_crc_bc1
		movf	ow_ROM_2,0
		call	ow_crc_bc1
		movf	ow_ROM_3,0
		call	ow_crc_bc1
		movf	ow_ROM_4,0
		call	ow_crc_bc1
		movf	ow_ROM_5,0
		call	ow_crc_bc1
		movf	ow_ROM_6,0
		call	ow_crc_bc1

;now in ow_crc_calc the calculated DOW CRC value is stored...

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ow_crc_bc1	movwf	c_tmp
		movlw	high ow_crc_lt
		movwf	TBLPTRH

;calculated offset for the look-up table --------------------------------------

		movf	c_tmp,0
		xorwf	ow_crc_calc,0

;that's it - offset is now calculated (it's just XOR between current value and considered data byte)

;------------------------------------------------------------------------------------------
;because of the lazy definition of the look-up able we need to multiply the result by 2

		movwf	c_tmp
		rlncf	c_tmp,1			;x2

		btfsc	c_tmp,D'000'
		incf	TBLPTRH,1		;Address MSB

		bcf	c_tmp,D'000'		;clear the LSb of LSB - we read always only LSB from the 16-bit data stored in programm memory
		movf	c_tmp,0
;------------------------------------------------------------------------------------------
		movwf	TBLPTRL			;Address LSB

		tblrd*
		movf	TABLAT,0		;get the corresponding value from the table
		movwf	ow_crc_calc
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                            CRC Check                                        X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_crc_check	nop
		call	disp_clr1
		call	disp_clr2
;----------------------------------------------
		call	crc_check1_exec
		call	w_crc_comp
;----------------------------------------------
ow_crc_chk_k2	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z		;back to the calling sub-program
		return

		goto	ow_crc_chk_k2

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                            ROM id                                           X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rom_id	nop
		call	disp_clr1
		call	disp_clr2
;------------------------------------------------------------
ow_rom_id_r	call	ds1820_read_rom
		call	w_ds1820i
;------------------------------------------------------------

ow_rom_id_im	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read
		btfsc	STATUS,Z
		goto	ow_rom_id_r

		movf	kbd_buffer,0
		xorlw	H'21'		;C = re-read
		btfsc	STATUS,Z
		goto	ow_rid_crc

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_bafu

		goto	ow_rom_id_im

;-----------------------------------------------------------
ow_rid_crc	call	ow_crc_check
		goto	ow_rom_id
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ds2406	movlw	B'10000000'
		call	send_comm
		call	wm_P
		call	wm_I
		call	wm_O
		call	wm_minus
		call	wm_A
		call	wm_dvojbodka
		call	wm_space
		btfss	ow_m2,D'000'
		call	wm_0
		btfsc	ow_m2,D'000'
		call	wm_1

		movlw	B'10001011'
		call	send_comm
		call	wm_zat1
		call	wm_0
		call	wm_sls
		call	wm_1
		call	wm_sls
		call	wm_R
		call	wm_zat2


		movlw	B'11000000'
		call	send_comm
		call	wm_C
		call	wm_h
		call	wm_space
		call	wm_i
		call	wm_n
		call	wm_f
		call	wm_o
		call	wm_space
		call	wm_b
		call	wm_y
		call	wm_t
		call	wm_e
		call	wm_dvojbodka
		call	wm_space
		movf	ow_m1,0
		movwf	kbd_buffer
		call	write_bin

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                   DS2406 - GPIO / TO-92 Version                             X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2406		movlw	H'FF'
		movwf	ow_m2

ds2406_2	call	disp_clr1
		call	disp_clr2

		call	ds2406_read
		call	w_ds2406
;------------------------------------------------------------------------------
ds2406_m	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'45'			;0 = Set PIO-A value to 0
		btfsc	STATUS,Z
		goto	pio_a_0

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = Set PIO-A value to 1
		btfsc	STATUS,Z
		goto	pio_a_1

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = read
		btfsc	STATUS,Z
		goto	ds2406_2

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_dev_sel

		goto	ds2406_m
;-------------------------------------------------------------------------------
pio_a_0		clrf	ow_m2
		call	ds2406_write
		goto	ds2406_2
;-------------------------------------------------------------------------------
pio_a_1		movlw	H'FF'
		movwf	ow_m2
		call	ds2406_write
		goto	ds2406_2
;-------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                   DS1820 - temperature measurement                          X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_1	nop
		call	disp_clr1
		call	disp_clr2

;------------------------------------------------------------
ds1820_2	call	ds1820_read_all

		movf	ow_m1,0
		movwf	v_temp1
		movf	ow_m2,0
		movwf	v_temp2

		rrcf	v_temp1,1
		bcf	v_temp1,D'007'
		call	t_convert2dec

		movf	ow_m1,0
		movwf	v_temp1
		movf	ow_m2,0
		movwf	v_temp2
;------------------------------------------------------------
		call	w_ds1820
;------------------------------------------------------------

ds1820_m	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read temperature
		btfsc	STATUS,Z
		goto	ds1820_t1

		movf	kbd_buffer,0
		xorlw	H'21'		;C = re-read temperature with strong pull-up
		btfsc	STATUS,Z
		goto	ds1820_t2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_dev_sel

		goto	ds1820_m
;-------------------------------------------------------------------------------
ds1820_t1	call	disp_clr1
		call	disp_clr2

		call	ds1820_conv
		goto	ds1820_2
;-------------------------------------------------------------------------------
ds1820_t2	call	disp_clr1
		call	disp_clr2

		call	ds1820_conv_spu
		goto	ds1820_2
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2484_i2c_read	call	ds2484_dc_read
		call	ds2484_pc_read
		call	ds2484_st_read
		call	ds2484_da_read
		return
;-------------------------------------------------------------------------------
ds2484_dc_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'C3'			;address of the Device Configuration Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_dc
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
ds2484_pc_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'B4'			;address of the Port Configuration Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_pc
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
ds2484_st_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'F0'			;address of the Status Register
		call	i2c_send
;
ds2484_st_read2	nop
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_st
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
ds2484_da_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'E1'			;address of the Read Data Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_data
		call	i2c_not_ack
		call	i2c_stop

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_conv	call	ow_rst
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		call	ow_write

		call	ow_rst

		return
;-------------------------------------------------------------------------------
ds1820_conv_spu	call	ow_rst
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		call	ow_write_spu

;wait and display progress in line #2
		movlw	B'11000000'
		call	send_comm
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		movlw	D'255'
		call	send_data
		call	d2		;wait a while...
		call	disp_clr2

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2406_read	call	ow_rst

		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'F5'		;Channel Access
		movwf	ow_buffer
		call	ow_write

		movlw	B'11000100'	;Channel Control Byte 1
		movwf	ow_buffer
		call	ow_write

		movlw	H'FF'		;Channel Control Byte 2
		movwf	ow_buffer
		call	ow_write

		call	ow_read		;read Channel Info Byte
		movf	ow_buffer,0
		movwf	ow_m1

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2406_write	call	ow_rst

		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'F5'		;Channel Access
		movwf	ow_buffer
		call	ow_write

		movlw	B'10000100'	;Channel Control Byte 1
		movwf	ow_buffer
		call	ow_write

		movlw	H'FF'		;Channel Control Byte 2
		movwf	ow_buffer
		call	ow_write

		call	ow_read		;read Channel Info Byte
		movf	ow_buffer,0
		movwf	ow_m1

		movf	ow_m2,0
		movwf	ow_buffer
		call	ow_write

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_read_all	call	ow_rst

		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'BE'		;Read Scratchpad
		movwf	ow_buffer
		call	ow_write

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m1

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m2

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m3

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m4

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m5

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m6

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m7

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m8

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m9

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_read_rom	call	ow_rst

		movlw	H'33'		;Read ROM
		movwf	ow_buffer
		call	ow_write

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_0

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_1

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_2

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_3

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_4

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_5

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_6

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_ROM_7

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;###############################################################################
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;         1-wire communication routines utilizing bridge device DS2484
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;###############################################################################
;-------------------------------------------------------------------------------
;1-wire Reset
;-------------------------------------------------------------------------------
ow_rst		nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'B4'			;command: 1-wire Reset
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop
		return
;-------------------------------------------------------------------------------
;1-wire write byte and use strong pull-up (SPU)
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write_spu	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'D2'			;command: Write Device Configuration
		call	i2c_send
		movlw	B'10110100'		;activate SPU bit of cinfiguration register
		call	i2c_send
		call	d55			;1-wire timing
		call	i2c_stop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire write byte
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire read byte
;                   byte will be stored to: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_read		nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'96'			;command: 1-wire Read Byte
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	ds2484_da_read

		movf	ds2484_data,0
		movwf	ow_buffer
		return
;-------------------------------------------------------------------------------
;1-wire Triplet
;                   used for ROM Search algorithm
;
; content of the Status register is stored in: ds2484_st
;     bit 5 (SBR) is the address bit of the device
;     bit 6 (TSB) is the complement of the device address bit
;     bit 7 (DIR) is the "direction taken bit"
;-------------------------------------------------------------------------------
ow_triplet	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'78'			;command: 1-wire Triplet
		call	i2c_send
		movf	v_ow_act_dir,0		;direction
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	ds2484_st_read2
		return
;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################
;-------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;X            Special Circuits Screens
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_sic001	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_I
		call	wm_zat2
		call	wm_N
		call	wm_A
		call	wm_2
		call	wm_1
		call	wm_9

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	high t_sic2
		movwf	TBLPTRH
		movlw	low t_sic2
		movwf	TBLPTRL
		call	w_send_txt

		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire main screen                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_mm		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_owm1
		movwf	TBLPTRH
		movlw	low t_owm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_owm2
		movwf	TBLPTRL
		call	w_send_txt
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire Basic Functions                             x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_bafu	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ow_bafu1
		movwf	TBLPTRH
		movlw	low t_ow_bafu1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_ow_bafu2
		movwf	TBLPTRL
		call	w_send_txt
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire device selection                            x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_dev_sel	movlw	B'10000000'
		call	send_comm
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_S
		call	wm_2
		call	wm_zat1
		call	wm_0
		call	wm_zat2

		call	wm_sls

		call	wm_D
		call	wm_S
		call	wm_2
		call	wm_4
		call	wm_0
		call	wm_zat1
		call	wm_6
		call	wm_zat2

		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                       CRC Compare  -  display                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_crc_comp	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_ow_tg1
		movwf	TBLPTRH
		movlw	low t_ow_tg1
		movwf	TBLPTRL
		call	w_send_txt

		movf	ow_ROM_7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_h


		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	high t_ow_tg2
		movwf	TBLPTRH
		movlw	low t_ow_tg2
		movwf	TBLPTRL
		call	w_send_txt

		movf	ow_crc_calc,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_h

		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          DS1820  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ds1820i	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ds1820
		movwf	TBLPTRH
		movlw	low t_ds1820
		movwf	TBLPTRL
		call	w_send_txt

w_ds1820i2	movlw	B'11000000'
		call	send_comm

		movf	ow_ROM_7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_hviezdicka

		movf	ow_ROM_6,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_2,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		call	wm_minus
		movf	ow_ROM_0,0
		movwf	kbd_buffer
		call	write_hex

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ds1820	nop

		call	w_temp_t_all
		call	w_tmp100_t

		movlw	B'11000000'
		call	send_comm

		movf	ow_m3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_m4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_m5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_m6,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_m7,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_m8,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_hviezdicka
		movf	ow_m9,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_zat1
		call	wm_X
		call	wm_C
		call	wm_zat2

		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          DS2484  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ds2484	movlw	B'10000000'
		call	send_comm
		call	wm_D
		call	wm_C
		call	wm_dvojbodka
		call	wm_space
		movf	ds2484_dc,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_S
		call	wm_t
		call	wm_a
		call	wm_t
		call	wm_dvojbodka
		call	wm_space

		movf	ds2484_st,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_dvojbodka
		call	wm_space
		movf	ds2484_pc,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_sls
		call	wm_space


		call	wm_D
		call	wm_a
		call	wm_t
		call	wm_a
		call	wm_dvojbodka
		call	wm_space

		movf	ds2484_data,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11010100'
		call	send_comm
		call	wm_zat1
		call	wm_X
		call	wm_zat2

		return

;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################
;-------------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Basic circuits screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
main_escs	call	disp_clr1
		call	disp_clr2

main_es		call	w_i2c_s

mes_001		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = Real Time Clock
		btfsc	STATUS,Z
		goto	rtc_sc

		movf	kbd_buffer,0
		xorlw	H'2C'		;T = Temp sensorts
		btfsc	STATUS,Z
		goto	temp_m

		movf	kbd_buffer,0
		xorlw	H'24'		;E = EEPROM
		btfsc	STATUS,Z
		goto	eeprom_ssm

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = Ports
		btfsc	STATUS,Z
		goto	ports1_3


		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	m001

		goto	mes_001


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          RTC - device selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_sc		nop
		call	disp_clr1
		call	disp_clr2
		call	w_rtcssc

		clrf	v_isl12022

scc001		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = DS1307 / M41T00CAP
		btfsc	STATUS,Z
		goto	rtc_m1

		movf	kbd_buffer,0
		xorlw	H'1E'			;2 = MCP7940N
		btfsc	STATUS,Z
		goto	rtc_m2

		movf	kbd_buffer,0
		xorlw	H'26'			;3 = ISL12022
		btfsc	STATUS,Z
		goto	rtc_m3

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	main_escs

		goto	scc001

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          RTC main screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

rtc_m1		movlw	B'11010000'		;RTC = 1101000x
		movwf	v_i2c_add
		goto	rtc_m
;--------------------------------------------------------
rtc_m2		movlw	B'11011110'		;RTC = 1101111x
		movwf	v_i2c_add
		goto	rtc_m
;--------------------------------------------------------
;--------------------------------------------------------
rtc_m		btfsc	v_xxx_conf,D'000'
		call	AMS_init
;
		call	disp_clr1
		call	disp_clr2
		call	w_rtcm

;----------------- RTC get data -----------------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
		call	d55

rtc_ml		call	rtc_i2c_read
		call	rtc_translate			;translate RTC date from RTC device to display format

		call	w_rtcm_dt

		btfsc	v_xxx_conf,D'000'
		call	w_ams_rtc			;send the clock data to AMS display driver via SPI
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

rwaitclk	btfss	PORTB,D'007'
		goto	rwaitclk

rwaitdat	btfss	PORTB,D'006'
		goto	rwaitdat

rwaitclk2	btfsc	PORTB,D'007'
		goto	rwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S - Set Date / time - keyboard input
		btfsc	STATUS,Z
		goto	rtc_set

		movf	kbd_buffer,0
		xorlw	H'24'			;E - Set Date / time example
		btfsc	STATUS,Z
		goto	rtc_set_e

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC -> exit
		btfsc	STATUS,Z
		goto	rtemp_exit

		goto	rtc_ml

rtemp_exit	nop
		btfsc	v_xxx_conf,D'000'
		call	w_ams_minus			;send the "minus" to AMS display driver via SPI
		goto	rtc_sc

;--------------------------------------------------------
;--------------------------------------------------------
;ISL12022
;--------------------------------------------------------
;--------------------------------------------------------
rtc_m3		btfsc	v_xxx_conf,D'000'
		call	AMS_init

		bsf	v_isl12022,D'000'
		movlw	B'11011110'		;RTC = 1101111x
		movwf	v_i2c_add

		call	disp_clr1
		call	disp_clr2
		call	w_rtcm

;----------------- RTC get data -----------------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
		call	d55

rtc_ml_isl	call	rtc_i2c_read_is
		call	rtc_translate			;translate RTC date from RTC device to display format

		call	w_rtcm_dt

		btfsc	v_xxx_conf,D'000'
		call	w_ams_rtc			;send the clock data to AMS display driver via SPI
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

riwaitclk	btfss	PORTB,D'007'
		goto	riwaitclk

riwaitdat	btfss	PORTB,D'006'
		goto	riwaitdat

riwaitclk2	btfsc	PORTB,D'007'
		goto	riwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S - Set Date / time - keyboard input
		btfsc	STATUS,Z
		goto	rtc_set

		movf	kbd_buffer,0
		xorlw	H'24'			;E - Set Date / time example
		btfsc	STATUS,Z
		goto	rtc_set_e

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC -> exit
		btfsc	STATUS,Z
		goto	ritemp_exit

		goto	rtc_ml_isl

ritemp_exit	nop
		btfsc	v_xxx_conf,D'000'
		call	w_ams_minus			;send the "minus" to AMS display driver via SPI

		goto	rtc_sc

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;                          RTC set - example
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_set_e	nop

		call	rtc_i2c_write_e

		call	disp_clr1
		call	disp_clr2
		call	w_rtc_e

rtcse01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	rtc_ml_p1

		goto	rtcse01


rtc_ml_p1	nop

		call	disp_clr1
		call	disp_clr2
		call	w_rtcm

		btfsc	v_isl12022,D'001'
		goto	spi_ds1306		;back to the SPI RTC routines
		goto	rtc_ml

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;                          RTC set
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_set		nop
		call	disp_clr2
		call	w_rtcm_s
;

		movlw	B'10000000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d1

		movlw	B'10000001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d2

		movlw	B'10000011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m1

		movlw	B'10000100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m2

		movlw	B'10001000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y3

		movlw	B'10001001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y4

		movlw	B'10001011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h1

		movlw	B'10001100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h2

		movlw	B'10001110'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n1

		movlw	B'10001111'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n2

		movlw	B'10010001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s1

		movlw	B'10010010'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s2

		movlw	B'10010100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_wd

		goto	rtc_set			;start cyclic - from the first number again...

rtc_set_set	call	rtc_backtrs		;translate the info on the display to RTC device format
		btfsc	v_isl12022,D'001'
		call	rtc_spi_write		;write data to SPI RTC device
		btfss	v_isl12022,D'001'
		call	rtc_i2c_write		;write data to I2C RTC device
		goto	rtc_set_exit

rtc_set_can	goto	rtc_set_exit		;ESC - no changes to be made

;-----------------------------------------------------------------------------
rtc_set_exit	btfsc	v_isl12022,D'001'
		goto	spi_rtc_in
		btfsc	v_isl12022,D'000'
		goto	rtc_m3
		goto	rtc_m
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_rtcssc	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_1
		call	wm_zat2
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_3
		call	wm_0
		call	wm_7
		call	wm_sls
		call	wm_M
		call	wm_4
		call	wm_1
		call	wm_T
		call	wm_0
		call	wm_0
		call	wm_C
		call	wm_A
		call	wm_P

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_2
		call	wm_zat2
		call	wm_M
		call	wm_C
		call	wm_P
		call	wm_7
		call	wm_9
		call	wm_4
		call	wm_0
		call	wm_N

		call	wm_space
		call	wm_space

		call	wm_zat1
		call	wm_3
		call	wm_zat2
		call	wm_I
		call	wm_S
		call	wm_L
		call	wm_1
		call	wm_2
		call	wm_0
		call	wm_2
		call	wm_2


		return		
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;set clock data to the AMS1107
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_ams_rtc	movf	v_rtc_h1,0
		movwf	ams_v_dig7
		movf	v_rtc_h2,0
		movwf	ams_v_dig6
		bsf	ams_v_dig6,D'007'

		movf	v_rtc_n1,0
		movwf	ams_v_dig5
		movf	v_rtc_n2,0
		movwf	ams_v_dig4

		movlw	' '
		movwf	ams_v_dig3
		movwf	ams_v_dig2

		movf	v_rtc_s1,0
		movwf	ams_v_dig1
		movf	v_rtc_s2,0
		movwf	ams_v_dig0

		movf	ams_v_dig7,0
		xorlw	'0'
		btfsc	STATUS,Z
		call	w_ams_cfd

		movf	ams_v_dig7,0
		btfsc	STATUS,Z
		call	w_ams_cfd

		call	AMS_disp_code

		return
;--------------------------------------------
w_ams_cfd	movlw	' '
		movwf	ams_v_dig7
		return
;---------------------------------------------------------------------------------------------------
w_ams_minus	movlw	'-'
		movwf	ams_v_dig7
		movwf	ams_v_dig6
		movwf	ams_v_dig5
		movwf	ams_v_dig4
		movwf	ams_v_dig3
		movwf	ams_v_dig2
		movwf	ams_v_dig1
		movwf	ams_v_dig0
		call	AMS_disp_code
		return
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;RTC display date/time in first row
;
w_rtcm_dt	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movf	v_rtc_d1,0
		call	send_data
		movf	v_rtc_d2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_m1,0
		call	send_data
		movf	v_rtc_m2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_y1,0
		call	send_data
		movf	v_rtc_y2,0
		call	send_data
		movf	v_rtc_y3,0
		call	send_data
		movf	v_rtc_y4,0
		call	send_data

		call	wm_sls
		
		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_s1,0
		call	send_data
		movf	v_rtc_s2,0
		call	send_data

		call	wm_space

		movf	v_rtc_t1,0
		call	send_data
		movf	v_rtc_t2,0
		call	send_data

		return
;-----------------------------------------------------------------
w_rtcm		call	w_rtcm_dt
;
;
		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_S
		call	wm_zat2
		call	wm_e
		call	wm_t
		call	wm_sls
		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_space
		call	wm_space

		btfss	v_isl12022,D'001'
		goto	w_i2c_add

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_S
		call	wm_P
		call	wm_I
		call	wm_0
		return
;-----------------------------------------------------------------
w_rtcm_s	call	w_rtcm_dt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_S
		call	wm_zat1
		call	wm_e
		call	wm_zat2
		call	wm_t
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C

		return
;-----------------------------------------------------------------
w_rtc_e		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_R
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_s
		call	wm_e
		call	wm_t
		call	wm_space
		call	wm_e
		call	wm_x
		call	wm_e
		call	wm_c
		call	wm_u
		call	wm_t
		call	wm_e
		call	wm_d
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	B'11000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_E
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_R
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock SPI communication routines for PCA2129
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
rtc_spi_read2	call	spi_reinit1
		call	spi_cs0_en

		movlw	H'A3'				;read second register
		call	spi_send
		call	spi_send
		movwf	v_rtc_sec
		bcf	v_rtc_sec,D'007'		;clear OSF-bit
		call	spi_send
		movwf	v_rtc_min
		call	spi_send
		movwf	v_rtc_hrs
		call	spi_send
		movwf	v_rtc_day
		call	spi_send
		movwf	v_rtc_wday
		call	spi_send
		movwf	v_rtc_mnt
		call	spi_send
		movwf	v_rtc_year

		call	spi_cs_all_dis
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock SPI communication routines for DS1306
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
rtc_spi_read	call	spi_init_DS1306
		call	spi_cs0_en

		movlw	H'00'
		call	spi_send
		call	spi_send
		movwf	v_rtc_sec
		call	spi_send
		movwf	v_rtc_min
		call	spi_send
		movwf	v_rtc_hrs
		call	spi_send
		movwf	v_rtc_wday
		call	spi_send
		movwf	v_rtc_day
		call	spi_send
		movwf	v_rtc_mnt
		call	spi_send
		movwf	v_rtc_year

		call	spi_cs_all_dis
		call	spi_reinit1
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_spi_write	btfsc	v_isl12022,D'002'
		goto	rtc_spi_write2
		call	spi_init_DS1306
		call	spi_cs0_en

		movlw	H'80'
		call	spi_send
		movf	v_rtc_sec,0
		call	spi_send
		movf	v_rtc_min,0
		call	spi_send
		movf	v_rtc_hrs,0
		call	spi_send
		movf	v_rtc_wday,0
		call	spi_send
		movf	v_rtc_day,0
		call	spi_send
		movf	v_rtc_mnt,0
		call	spi_send
		movf	v_rtc_year,0
		call	spi_send

		call	spi_cs_all_dis
		call	spi_reinit1
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_spi_write2	call	spi_reinit1
		call	spi_cs0_en

		movlw	H'23'
		call	spi_send
		movf	v_rtc_sec,0
		call	spi_send
		movf	v_rtc_min,0
		call	spi_send
		movf	v_rtc_hrs,0
		call	spi_send
		movf	v_rtc_day,0
		call	spi_send
		movf	v_rtc_wday,0
		call	spi_send
		movf	v_rtc_mnt,0
		call	spi_send
		movf	v_rtc_year,0
		call	spi_send

		call	spi_cs_all_dis
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;write example date / time to RTC: date = 15.11.2013 time = 14:32:45; day = 5 (Friday)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_spi_write_e	call	spi_init_DS1306
		call	spi_cs0_en

		movlw	H'80'
		call	spi_send
		movlw	H'45'
		call	spi_send
		movlw	H'32'
		call	spi_send
		movlw	H'14'
		call	spi_send
		movlw	H'05'
		call	spi_send
		movlw	H'15'
		call	spi_send
		movlw	H'11'
		call	spi_send
		movlw	H'13'
		call	spi_send

		call	spi_cs_all_dis
		call	spi_reinit1
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock I2C communication routines for ISL12022
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read current date/time
;
rtc_i2c_read_is	nop

;Step 1 - Start condition
		call	i2c_start

;Step 2: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send

;Step 4: START condition
		call	i2c_restart

;Step 5: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

;-------------------------------------
;Step 6: read data
;6.1 - sec
		call	i2c_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_ack

;6.2 - min
		call	i2c_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_ack

;6.3 - hours
		call	i2c_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_ack
		bcf	v_rtc_hrs,D'007'	;clear the MIL bit of HR register

;6.5 - day
		call	i2c_receive		;received byte
		movwf	v_rtc_day
		call	i2c_ack

;6.6 - month
		call	i2c_receive		;received byte
		movwf	v_rtc_mnt
		call	i2c_ack

;6.7 - year
		call	i2c_receive		;received byte
		movwf	v_rtc_year
		call	i2c_ack


;6.4 - day in a week
		call	i2c_receive		;received byte
		movwf	v_rtc_wday
		call	i2c_not_ack
		incf	v_rtc_wday,1		;for ISL12022 is DW 0-6; for translation we need 1-7


;Step 7: send STOP condition (stop the read)
		call	i2c_stop

		return

;--------------------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read current date/time
;
rtc_i2c_read	nop

;Step 1 - Start condition
		call	i2c_start

;Step 2: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send

;Step 4: send STOP condition (stop the write)
		call	i2c_stop

;Step 5: START condition
		call	i2c_start

;Step 6: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

;-------------------------------------
;Step 7: read data
;7.1 - sec
		call	i2c_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_ack

;7.2 - min
		call	i2c_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_ack

;7.3 - hours
		call	i2c_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_ack

;7.4 - day in a week
		call	i2c_receive		;received byte
		movwf	v_rtc_wday
		call	i2c_ack

;7.5 - day
		call	i2c_receive		;received byte
		movwf	v_rtc_day
		call	i2c_ack

;7.6 - month
		call	i2c_receive		;received byte
		movwf	v_rtc_mnt
		call	i2c_ack

;7.7 - year
		call	i2c_receive		;received byte
		movwf	v_rtc_year
		call	i2c_not_ack

;Step 8: send STOP condition (stop the read)
		call	i2c_stop

		return

;--------------------------------------------------------------------------------------
;write example date / time to RTC: date = 15.11.2013 time = 14:32:45; day = 5 (Friday)
;--------------------------------------------------------------------------------------
rtc_i2c_write_e	nop

		btfsc	v_isl12022,D'001'
		goto	rtc_spi_write_e

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send



;-------------------------------------
;Step 4: write data
;4.1 - sec
		movlw	B'01000101'		;BCD code of 45
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movlw	B'00110010'		;BCD code of 32
		call	i2c_send

;4.3 - hours
		movlw	B'00010100'		;BCD code of 14
		call	i2c_send

;4.4 - day in a week
		movlw	B'00000101'		;BCD code of 05
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movlw	B'00010101'		;BCD code of 15
		call	i2c_send

;4.6 - month
		movlw	B'00010001'		;BCD code of 11
		call	i2c_send

;4.7 - year
		movlw	B'00010011'		;BCD code of 13
		call	i2c_send

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_i2c_write	nop
		btfsc	v_isl12022,D'000'
		goto	rtc_i2c_wr_isl		;for IS12022 we need different logic

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send
;-------------------------------------
;Step 4: write data
;4.1 - sec
		movf	v_rtc_sec,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movf	v_rtc_min,0
		call	i2c_send

;4.3 - hours
		movf	v_rtc_hrs,0
		call	i2c_send

;4.4 - day in a week
		movf	v_rtc_wday,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movf	v_rtc_day,0
		call	i2c_send

;4.6 - month
		movf	v_rtc_mnt,0
		call	i2c_send

;4.7 - year
		movf	v_rtc_year,0
		call	i2c_send

		btfsc	v_i2c_add,D'001'
		call	MCP7940_calib		;for MCP7940 - write calibration data in addition

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return

;-----------------------------------------------------------------------------
MCP7940_calib	movlw	H'80'
		call	i2c_send		;control register (just use the power-up value)

		movlw	D'000'
		call	i2c_send		;write 120 (DEC) to calibration register to speed-up the clock

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;IS12022 write routine
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_i2c_wr_isl	nop

;1. set the WRTC bit of INT register to 1

		call	i2c_start
		movlw	B'11011110'		;I2C address of ISL12022 - RTC part
		call	i2c_send
		movlw	H'08'
		call	i2c_send
		movlw	B'01000001'
		call	i2c_send
		call	i2c_stop
;----------------------------------------------------
;2. set the date and time

		call	i2c_start
		movlw	B'11011110'		;I2C address of ISL12022 - RTC part
		call	i2c_send
		movlw	H'00'
		call	i2c_send

		movf	v_rtc_sec,0
		call	i2c_send
;
		movf	v_rtc_min,0
		call	i2c_send
;
		movf	v_rtc_hrs,0
		addlw	D'128'			;set MIL bit --> 24 hrs format to be used
		call	i2c_send
;
		movf	v_rtc_day,0
		call	i2c_send
;
		movf	v_rtc_mnt,0
		call	i2c_send
;
		movf	v_rtc_year,0
		call	i2c_send
;
		movf	v_rtc_wday,0
		call	i2c_send

		call	i2c_stop

		call	isl_DST_set

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;Maintain DST registers for ISL12022
;
;European convention:
;	start of daylight saving time: last Sunday in March at 1.00 AM
;       end   of daylight saving time: last Sunday in October at 1.00 AM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
isl_DST_set	call	i2c_start
		movlw	B'11011110'		;I2C address of ISL12022 - RTC part
		call	i2c_send
		movlw	H'20'			;start of DST Forward Registers
		call	i2c_send

;Forward
		movlw	H'03'			;Month = March
		addlw	D'128'			;set bit DSTE (Enable DST)
		call	i2c_send

		movlw	B'01111110'
					;bit  6     - DwFdE = 1   --> Week in Month / Day in Week option
					;bits 5-4-3 = Week  = 111 --> Week = 7 --> last week in the Month
					;bits 2-1-0 = Day   = 110 --> Day =  6 --> Sunday
		call	i2c_send

		movlw	D'000'			;date - not used
		call	i2c_send

		movlw	H'02'			;hour
		call	i2c_send
;-----------------------------------------------------------
;Backward
		movlw	H'10'		;Month = October
		call	i2c_send

		movlw	B'01111110'
					;bit  6     - DwFdE = 1   --> Week in Month / Day in Week option
					;bits 5-4-3 = Week  = 111 --> Week = 7 --> last week in the Month
					;bits 2-1-0 = Day   = 110 --> Day =  6 --> Sunday
		call	i2c_send

		movlw	D'000'			;date - not used
		call	i2c_send

		movlw	H'02'			;hour
		call	i2c_send

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM selection SCREEN
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

eeprom_ssm	movlw	D'001'				;24LC01 EEPROM selected
		movwf	v_ee_type

eeprom_disp	call	disp_clr1
		call	disp_clr2
		call	w_eess

eeprom_ss	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = next EEPROM
		btfsc	STATUS,Z
		goto	eeprom_sel1

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = pervious EEPROM
		btfsc	STATUS,Z
		goto	eeprom_sel2

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eeprom_read

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eeprom_write

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_m_ret

		goto	eeprom_ss
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eeprom_m_ret	btfss	v_ee_type,D'007'
		goto	main_escs	;return to I2C EEPROM menu
		goto	m_spi		;return to SPI EEPROM menu
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eeprom_sel1	btfsc	v_ee_type,D'007'
		goto	eeprom_ssp_s1

		incf	v_ee_type,1
		movf	v_ee_type,0
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	eeprom_ssm

		goto	eeprom_disp
;-----------------------------------------------------------------------------
eeprom_sel2	btfsc	v_ee_type,D'007'
		goto	eeprom_ssp_s2
		
		decf	v_ee_type,1
		btfsc	STATUS,Z
		goto	eeprom_ssm2

		goto	eeprom_disp

eeprom_ssm2	movlw	H'0C'
		movwf	v_ee_type
		goto	eeprom_disp
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eeprom_ssp_s1	incf	v_ee_type,1
		movf	v_ee_type,0
		xorlw	H'82'
		btfss	STATUS,Z
		goto	eeprom_disp

		movlw	H'81'
		movwf	v_ee_type
		goto	eeprom_disp
;-----------------------------------------------------------------------------
eeprom_ssp_s2	decf	v_ee_type,1
		movf	v_ee_type,0
		xorlw	H'80'
		btfss	STATUS,Z
		goto	eeprom_disp

		movlw	H'81'
		movwf	v_ee_type
		goto	eeprom_disp

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM write Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eeprom_write	clrf	v_ad0
		clrf	v_ad1


eep_lw2s	movlw	B'00000001'
		movwf	v_ee_w
		call	disp_clr1
		call	disp_clr2


eep_lw2		call	eep_gen_read8		;EEPROM read 8 bytes from address v_ad1-v_ad0
		call	w_eeprom_data

;------------------------------------------------------------------
eep_rw1		movf	v_eed0,0
		movwf	v_b_in
		movlw	B'11000000'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed0
		call	eep_lw1

		movf	v_eed1,0
		movwf	v_b_in
		movlw	B'11000011'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed1
		call	eep_lw1

		movf	v_eed2,0
		movwf	v_b_in
		movlw	B'11000110'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed2
		call	eep_lw1

		movf	v_eed3,0
		movwf	v_b_in
		movlw	B'11001001'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed3
		call	eep_lw1

		movf	v_eed4,0
		movwf	v_b_in
		movlw	B'11001100'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed4
		call	eep_lw1

		movf	v_eed5,0
		movwf	v_b_in
		movlw	B'11001111'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed5
		call	eep_lw1

		movf	v_eed6,0
		movwf	v_b_in
		movlw	B'11010010'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed6
		call	eep_lw1

		movf	v_eed7,0
		movwf	v_b_in
		movlw	B'11010101'
		call	hex_input
		movf	v_b_in,0
		movwf	v_eed7
		call	eep_lw1


		goto	eep_rw1
;------------------------------------------------------------------

eep_lw1		nop

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = read next
		btfsc	STATUS,Z
		goto	eep_lw5

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = read pervious
		btfsc	STATUS,Z
		goto	eep_lw6

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down = read next starting in next 256 bytes bank at 00
		btfsc	STATUS,Z
		goto	eep_lw3

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up = read next starting 256 bytes before at 00
		btfsc	STATUS,Z
		goto	eep_lw4

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eep_l2s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_disp

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw3		call	ee_save

		clrf	v_ad0
		incf	v_ad1,1
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw4		call	ee_save

		clrf	v_ad0
		decf	v_ad1,1
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw5		call	ee_save

		call	eeprom_inc_ad8
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw6		call	ee_save

		call	eeprom_dec_ad8
		goto	eep_lw2
;-----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM Read Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eeprom_read	clrf	v_ad0
		clrf	v_ad1

eep_l2s		clrf	v_ee_w
		call	disp_clr1
		call	disp_clr2


eep_l2		call	eep_gen_read8		;EEPROM read 8 bytes from address v_ad1-v_ad0
		call	w_eeprom_data

eep_l1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = read next
		btfsc	STATUS,Z
		goto	eep_l5

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = read pervious
		btfsc	STATUS,Z
		goto	eep_l6

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down = read next starting in next 256 bytes bank at 00
		btfsc	STATUS,Z
		goto	eep_l3

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up = read next starting 256 bytes before at 00
		btfsc	STATUS,Z
		goto	eep_l4

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eep_lw2s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_disp

		goto	eep_l1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l3		clrf	v_ad0
		incf	v_ad1,1
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l4		clrf	v_ad0
		decf	v_ad1,1
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l5		call	eeprom_inc_ad8
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l6		call	eeprom_dec_ad8
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

ee_save		nop

		movf	o_eed0,0
		xorwf	v_eed0,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed1,0
		xorwf	v_eed1,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed2,0
		xorwf	v_eed2,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed3,0
		xorwf	v_eed3,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed4,0
		xorwf	v_eed4,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed5,0
		xorwf	v_eed5,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed6,0
		xorwf	v_eed6,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed7,0
		xorwf	v_eed7,0
		btfss	STATUS,Z
		goto	ee_save_y

		return


ee_save_y	nop
		call	disp_clr2
		call	w_ee_save


		call	eep_gen_write8
		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;write 8 bytes to SPI EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data to EEPROM will be stored from: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eep_spi_write8	call	spi_ee_seq_op

;1. activate CS line
		call	spi_cs0_en		;activate -CS0

;2. send device instruction
		movlw	B'00000010'		;send WRITE
		call	spi_send

;the rest is exactly the same as for read operation...
		goto	eep_spi_data

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;write 8 bytes to I2C EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data to EEPROM will be stored from: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eep_i2c_write8	nop
;initialize the write operation
;
		call	i2c_start
;
		movlw	B'10100000'	;I2C address of the EEPROM device
		movwf	v_ee_i2c
;
;we have to merge the high address bits for EEPROMs 24LC04 - 24LC16 with the I2C address
;we will do it for 24LC01 and 02 as well to be able to work with then on different
;I2C addresses -- "finta"
		movf	v_ee_type,0
		xorlw	D'01'		;24LC01
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'02'		;24LC02
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'03'		;24LC04
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'04'		;24LC08
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'05'		;24LC16
		btfss	STATUS,Z
		goto	ee_i2c_cnt52x

addr_merge2	movf	v_ad1,0
		movwf	v_ee_i2c
		rlcf	v_ee_i2c,0
		andlw	B'00001110'	;we are using only the 3 LSB bits for those devices
		movwf	v_ee_i2c

		movlw	B'10100000'	;I2C address of the EEPROM device
		addwf	v_ee_i2c,1
;----------------------------------------------------------------------
;now it is time to consider DS1624...
ee_i2c_cnt52x	movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	ee_i2c_cnt52

;DS1624 - for now just change the device I2C address
		movlw	B'10010100'
		movwf	v_ee_i2c
;----------------------------------------------------------------------

;----------------------------------------------------------------------
ee_i2c_cnt52	movf	v_ee_i2c,0
		call	i2c_send


; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		call	ee_DS1624_comm
; ----------------------------------------

		btfsc	v_ee_add,D'000'
		call	ee_sah
		movf	v_ad0,0
		call	i2c_send

;
;write the sequence of 8 bytes
		movf	v_eed0,0
		call	i2c_send
		call	fwpe

		movf	v_eed1,0
		call	i2c_send
		call	fwpe

		movf	v_eed2,0
		call	i2c_send
		call	fwpe

		movf	v_eed3,0
		call	i2c_send
		call	fwpe

		movf	v_eed4,0
		call	i2c_send
		call	fwpe

		movf	v_eed5,0
		call	i2c_send
		call	fwpe

		movf	v_eed6,0
		call	i2c_send
		call	fwpe

		movf	v_eed7,0
		call	i2c_send


		call	i2c_stop

		call	dr2

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
fwpe		movf	v_ee_type,0
		xorlw	H'0C'
		btfsc	STATUS,Z
		call	dr2			;wait a while for EEPROM AT28C512
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;read 8 bytes of data from I2C or SPI EEPROM
;-----------------------------------------------------------------------------
eep_gen_read8	btfss	v_ee_type,D'007'
		goto	eep_i2c_read8
		goto	eep_spi_read8
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;write 8 bytes of data from I2C or SPI EEPROM
;-----------------------------------------------------------------------------
eep_gen_write8	btfss	v_ee_type,D'007'
		goto	eep_i2c_write8
		goto	eep_spi_write8
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;initialize SPI sequencial opeation
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_ee_seq_op	call	spi_cs0_en		;activate -CS0
		movlw	B'00000001'		;send instruction WRMR (Write Mode Rgister)
		call	spi_send
		movlw	B'01000000'		;'01xxxxx' = sequencial mode (power-up default)
		call	spi_send
		call	spi_cs_all_dis		;deactivate -CS0
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;read 8 bytes from SPI EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data from EEPROM will be stored in: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
eep_spi_read8	call	spi_ee_seq_op

;1. activate CS line
		call	spi_cs0_en		;activate -CS0

;2. send device instruction
		movlw	B'00000011'		;send READ
		call	spi_send

;-----------------------------------------------------------------------------
;this part is used for read and for write operations
;during write we destroy the content of the internal registers x_eedn
;with dummy data received from the device; but doesn't matter, because
;in this Firmware after write operation we will read the content of the device
;before displaying something again anyway...
;-----------------------------------------------------------------------------
;3. send memory address
eep_spi_data	movf	v_ad1,0			;Address MSB
		call	spi_send
		movf	v_ad0,0			;Address LSB
		call	spi_send

;4. read data
		movf	v_eed0,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed0
		movwf	o_eed0

		movf	v_eed1,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed1
		movwf	o_eed1

		movf	v_eed2,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed2
		movwf	o_eed2

		movf	v_eed3,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed3
		movwf	o_eed3

		movf	v_eed4,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed4
		movwf	o_eed4

		movf	v_eed5,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed5
		movwf	o_eed5

		movf	v_eed6,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed6
		movwf	o_eed6

		movf	v_eed7,0		;to be writte / don't care for read
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_eed7
		movwf	o_eed7

;5. deactivate CS line

		call	spi_cs_all_dis		;deactivate all CS

		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;read 8 bytes from I2C EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data from EEPROM will be stored in: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eep_i2c_read8	nop
;initialize the read operation
;
		call	i2c_start
;
		movlw	B'10100000'	;I2C address of the EEPROM device
		movwf	v_ee_i2c
;
;we have to merge the high address bits for EEPROMs 24LC04 - 24LC16 with the I2C address
;we will do it for 24LC01 and 02 as well to be able to work with then on different
;I2C addresses -- "finta"
		movf	v_ee_type,0
		xorlw	D'01'		;24LC01
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'02'		;24LC02
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'03'		;24LC04
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'04'		;24LC08
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'05'		;24LC16
		btfss	STATUS,Z
		goto	ee_i2c_cnt5x

addr_merge	movf	v_ad1,0
		movwf	v_ee_i2c
		rlcf	v_ee_i2c,0
		andlw	B'00001110'	;we are using only the 3 LSB bits for those devices
		movwf	v_ee_i2c

		movlw	B'10100000'	;I2C address of the EEPROM device
		addwf	v_ee_i2c,1
;----------------------------------------------------------------------
;now it is time to consider DS1624...
ee_i2c_cnt5x	movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	ee_i2c_cnt5

;DS1624 - for now just change the device I2C address
		movlw	B'10010100'
		movwf	v_ee_i2c
;----------------------------------------------------------------------
ee_i2c_cnt5	movf	v_ee_i2c,0
		call	i2c_send

; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		call	ee_DS1624_comm
; ----------------------------------------

		btfsc	v_ee_add,D'000'
		call	ee_sah		;send memory location address (MSB), if required
		movf	v_ad0,0		;memory location address (LSB)
		call	i2c_send

		call	i2c_start
		movf	v_ee_i2c,0	;I2C address of the EEPROM device
		addlw	D'001'		;set the LSB to 1 (--> read operation)
		call	i2c_send
;
;read the sequence of 8 bytes
		call	i2c_receive
		movwf	v_eed0
		movwf	o_eed0
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed1
		movwf	o_eed1
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed2
		movwf	o_eed2
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed3
		movwf	o_eed3
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed4
		movwf	o_eed4
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed5
		movwf	o_eed5
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed6
		movwf	o_eed6
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed7
		movwf	o_eed7

; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
; ----------------------------------------
;send NOT ACK - for all EEPROM types except DS1624

		call	i2c_not_ack

		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;send high address byte
ee_sah		movf	v_ad1,0
		call	i2c_send
		return
;-----------------------------------------------------------------------------
;send EEPROM access command for DS1624 device
ee_DS1624_comm	movlw	H'17'
		call	i2c_send
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-----------------------------------------------------------------------------
;increment the address to be displayed
eeprom_inc_ad8	movlw	D'008'
		movwf	TIMER1
eaa1x		incf	v_ad0,1
		btfsc	STATUS,Z
		goto	inc_ah
cnt_lw		decfsz	TIMER1,1
		goto	eaa1x

		return
;-------------------------------------------
inc_ah		incf	v_ad1,1
		goto	cnt_lw
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;increment the address to be displayed
eeprom_dec_ad8	movlw	D'008'
		movwf	TIMER1
eaa1y		decf	v_ad0,1
		movf	v_ad0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	dec_ah
cnt_lw2		decfsz	TIMER1,1
		goto	eaa1y

		return
;-------------------------------------------
dec_ah		decf	v_ad1,1
		goto	cnt_lw2


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;
; EEPROM w_screens
;
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
w_eess		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	'<'
		call	send_data
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_4
		call	wm_L
		call	wm_C

		movlw	B'10010110'		;row: 1 / col: 22
		call	send_comm

		call	wm_minus
		movlw	'>'
		call	send_data


		btfsc	v_ee_type,D'007'
		goto	w_eess_c_spi

;second row currently only for I2C devices
		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	high t_ee_addr
		movwf	TBLPTRH
		movlw	low t_ee_addr
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11010001'		;row: 2 / col: 17
		call	send_comm

		call	wm_sls
		call	wm_space
		call	wm_M
		call	wm_a
		call	wm_x
		call	wm_dvojbodka

w_eess_c_spi	movlw	B'10000111'		;row: 1 / col: 7
		call	send_comm
		call	w_eeprom_type


		movlw	B'11011000'		;row: 2 / col: 24 (out of display)
		call	send_comm

		return
;-------------------------------------------------------------------------------
w_ee_save	movlw	B'11000000'
		call	send_comm
		movlw	high t_ee_save
		movwf	TBLPTRH
		movlw	low t_ee_save
		movwf	TBLPTRL
		call	w_send_txt
		return
;-------------------------------------------------------------------------------
w_eeprom_type	nop
		movf	v_ee_type,0
		xorlw	H'01'
		btfsc STATUS,Z
		goto	w_eeprom01

		movf	v_ee_type,0
		xorlw	H'02'
		btfsc STATUS,Z
		goto	w_eeprom02

		movf	v_ee_type,0
		xorlw	H'03'
		btfsc STATUS,Z
		goto	w_eeprom04

		movf	v_ee_type,0
		xorlw	H'04'
		btfsc STATUS,Z
		goto	w_eeprom08

		movf	v_ee_type,0
		xorlw	H'05'
		btfsc STATUS,Z
		goto	w_eeprom16

		movf	v_ee_type,0
		xorlw	H'06'
		btfsc STATUS,Z
		goto	w_eeprom32

		movf	v_ee_type,0
		xorlw	H'07'
		btfsc STATUS,Z
		goto	w_eeprom64

		movf	v_ee_type,0
		xorlw	H'08'
		btfsc STATUS,Z
		goto	w_eeprom128

		movf	v_ee_type,0
		xorlw	H'09'
		btfsc STATUS,Z
		goto	w_eeprom256

		movf	v_ee_type,0
		xorlw	H'0A'
		btfsc STATUS,Z
		goto	w_eeprom512

		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc STATUS,Z
		goto	w_eepromDS

		movf	v_ee_type,0
		xorlw	H'0C'
		btfsc STATUS,Z
		goto	w_eepromp

		movf	v_ee_type,0
		xorlw	H'81'
		btfsc STATUS,Z
		goto	w_eeprom512


w_eeprom_err	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_r
		return

w_eeprom01	movlw	high t_eeprom01
		movwf	TBLPTRH
		movlw	low t_eeprom01
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		clrf	v_ee_add		;one address byte

		return

w_eeprom02	movlw	high t_eeprom02
		movwf	TBLPTRH
		movlw	low t_eeprom02
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		clrf	v_ee_add		;one address byte

		return

w_eeprom04	movlw	high t_eeprom04
		movwf	TBLPTRH
		movlw	low t_eeprom04
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_4

		movlw	B'11001000'		;row: 2 / col: 8
		call	send_comm
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom08	movlw	high t_eeprom08
		movwf	TBLPTRH
		movlw	low t_eeprom08
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_2

		movlw	B'11000111'		;row: 2 / col: 7
		call	send_comm
		call	wm_x
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom16	movlw	high t_eeprom16
		movwf	TBLPTRH
		movlw	low t_eeprom16
		movwf	TBLPTRL
		call	w_send_txt


		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_1

		movlw	B'11000110'		;row: 2 / col: 6
		call	send_comm
		call	wm_x
		call	wm_x
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom32	movlw	high t_eeprom32
		movwf	TBLPTRH
		movlw	low t_eeprom32
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom64	movlw	high t_eeprom64
		movwf	TBLPTRH
		movlw	low t_eeprom64
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom128	movlw	high t_eeprom128
		movwf	TBLPTRH
		movlw	low t_eeprom128
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom256	movlw	high t_eeprom256
		movwf	TBLPTRH
		movlw	low t_eeprom256
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom512	movlw	high t_eeprom512
		movwf	TBLPTRH
		movlw	low t_eeprom512
		movwf	TBLPTRL
		call	w_send_txt

		btfss	v_ee_type,D'007'
		goto	w_ee512_i2c

;additional info for SPI device
		movlw	B'10000100'
		call	send_comm
		call	wm_3

		movlw	D'001'
		movwf	v_ee_add		;two address bytes
		return

;additional info for I2C device
w_ee512_i2c	movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return


w_eepromDS	movlw	B'10000011'		;row:1 / col: 3
		call	send_comm

		movlw	high t_ds1624
		movwf	TBLPTRH
		movlw	low t_ds1624
		movwf	TBLPTRL
		call	w_send_txt


		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	B'11000010'		;row:2 / col: 2
		call	send_comm
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_1
		call	wm_0

		movlw	D'000'
		movwf	v_ee_add		;one address bytes

		return
;-------------------------------------------------------------------------------
;28C256

w_eepromp	movlw	B'10000011'		;row:1 / col: 3
		call	send_comm

		movlw	high t_28c256
		movwf	TBLPTRH
		movlw	low t_28c256
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_minus

		movlw	B'11000010'		;row:2 / col: 2
		call	send_comm
		call	wm_1
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_eeprom_data	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_A
		call	wm_d
		call	wm_d
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		movlw	':'
		btfsc	v_ee_w,D'000'
		movlw	'*'
		call	send_data
		call	wm_space

;first write the address (2 bytes)
		movf	v_ad1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus

		movf	v_ad0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		btfss	v_ee_type,D'007'
		goto	wed_i2c

		call	wm_S
		call	wm_P
		call	wm_I
		call	wm_dvojbodka
		call	wm_space
		call	wm_C
		call	wm_S
		call	wm_0
		goto	wed2

wed_i2c		call	wm_A
		call	wm_dvojbodka
		call	write_bin2
;

wed2		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movf	v_eed0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed2,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed6,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed7,0
		movwf	kbd_buffer
		call	write_hex

;
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX






;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;               Temperature Sensor Selection Sreen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
temp_m		call	disp_clr1
		call	disp_clr2
		call	w_tm

tmm2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = TMP100 / AT30TSE754
		btfsc	STATUS,Z
		goto	tmp100c_m

		movf	kbd_buffer,0
		xorlw	H'1E'		;2 = TMP100 / AT30TSE754
		btfsc	STATUS,Z
		goto	tmp_atmel

		movf	kbd_buffer,0
		xorlw	H'26'		;3 = LM76
		btfsc	STATUS,Z
		goto	tmp_lm76

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = DS1624
		btfsc	STATUS,Z
		goto	tmp_ds1624

		movf	kbd_buffer,0
		xorlw	H'2E'		;5 = ISL12022
		btfsc	STATUS,Z
		goto	tmp_isl12022

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	main_escs

		goto	tmm2

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          ISL12022 - temperature
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_isl12022	nop

		call	disp_clr1
		call	disp_clr2
		call	w_tmp_isl12022

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

;clear kbd buffer----------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
;--------------------------------
isl12022_temp	call	isl_temp_en		;trigger temperature meassurement cycle
		call	isl_temp_read		;read the temperature
		call	t_isl_conv2dec		;convert the data
		call	w_isl12_2		;display the data
		call	dr2			;wait a bit
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

iwaitclk	btfss	PORTB,D'007'
		goto	iwaitclk

iwaitdat	btfss	PORTB,D'006'
		goto	iwaitdat

iwaitclk2	btfsc	PORTB,D'007'
		goto	iwaitclk2

		clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	isl12022_temp
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
isl_temp_en	call	i2c_start
		movlw	B'11011110'
		call	i2c_send
		movlw	H'0D'			;BETA-Register
		call	i2c_send
		movlw	H'80'			;set TSE bit to 1
		call	i2c_send
		call	i2c_stop
		return
;-----------------------------------------------------------------------------
isl_temp_read	call	i2c_start
		movlw	B'11011110'
		call	i2c_send
		movlw	H'28'			;TK0L register
		call	i2c_send
		call	i2c_stop
		call	i2c_start
		movlw	B'11011111'
		call	i2c_send
		call	i2c_receive
		movwf	v_temp2			;LSB
		call	i2c_ack
		call	i2c_receive
		movwf	v_temp1			;MSB
		call	i2c_not_ack
		call	i2c_stop
		return
;-----------------------------------------------------------------------------
t_isl_conv2dec	movf	v_temp1,0
		movwf	v_temp1s
		movf	v_temp2,0
		movwf	v_temp2s
;---------------------------------------------------
;K
		movlw	'0'
		movwf	v_tst1k
		movwf	v_tst2k
		movwf	v_tst3k
;---------------------------------------------------
;C
		movlw	'2'
		movwf	v_tst1
		movlw	'7'
		movwf	v_tst2
		movlw	'2'
		movwf	v_tst3

		bsf	v_tst1,D'007'		;minus
;-------------------------------------------------
		rrcf	v_temp1s,1
		bcf	v_temp1s,D'007'
		rrcf	v_temp2s,1
;-------------------------------------------------
		incf	v_temp1s,1

tcisl_l1	incf	v_tst3k,1
		movf	v_tst3k,0
		xorlw	':'
		btfss	STATUS,Z
		goto	icisl_next
		movlw	'0'
		movwf	v_tst3k
		incf	v_tst2k,1
		movf	v_tst2k,0
		xorlw	':'
		btfss	STATUS,Z
		goto	icisl_next
		movlw	'0'
		movwf	v_tst2k
		incf	v_tst1k,1

icisl_next	call	tmp_isl_sc

		decfsz	v_temp2s,1
		goto	tcisl_l1
		decfsz	v_temp1s,1
		goto	tcisl_l1

;--------------------------------------------------
		movf	v_tst1,0
		xorlw	'+'
		btfsc	STATUS,Z
		return
		movlw	'-'
		movwf	v_tst1
;--------------------------------------------------
		return
;-----------------------------------------------------------------------------
tmp_isl_sc	btfss	v_tst1,D'007'
		goto	tmp_isl_sc2
;decrement (minus values)
		decf	v_tst3,1
		movf	v_tst3,0
		xorlw	'/'			;'/' = 2Fh
		btfss	STATUS,Z
		return
		movlw	'9'
		movwf	v_tst3
;
		decf	v_tst2,1
		movf	v_tst2,0
		xorlw	'/'
		btfss	STATUS,Z
		return
		movlw	'9'
		movwf	v_tst2
;
		decf	v_tst1,1
		movf	v_tst1,0
		xorlw	H'AF'
		btfss	STATUS,Z
		return
;
		movlw	'+'
		movwf	v_tst1
		movlw	'0'
		movwf	v_tst2
		movwf	v_tst3

		return
;----------------------------------------------
tmp_isl_sc2	nop
;increment (plus values)

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_tmp_isl12022	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_R
		call	wm_e
		call	wm_g
		call	wm_dvojbodka
		call	wm_space

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	high t_tmp_isl12022
		movwf	TBLPTRH
		movlw	low t_tmp_isl12022
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_isl12_2	movlw	B'10000101'		;row: 1 / col: 5
		call	send_comm

		movf	v_temp1,0		;MSB
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus

		movf	v_temp2,0		;LSB
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
		call	wm_space

		movf	v_tst1k,0
		call	send_data
		movf	v_tst2k,0
		call	send_data
		movf	v_tst3k,0
		call	send_data
		call	wm_space
		call	wm_K

		movlw	B'10010011'		;row 1 / col: 19
		call	send_comm
		movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		call	wm_stup
		call	wm_C

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          TMP100c screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp100c_m	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		call	disp_clr1
		call	disp_clr2
		call	w_tmp100

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

;clear kbd buffer----------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
;--------------------------------
tmp100c_rt	call	tmp100_read_t1
		call	t_convert2dec
		call	w_tmp100_t

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

waitclk		btfss	PORTB,D'007'
		goto	waitclk

waitdat		btfss	PORTB,D'006'
		goto	waitdat

waitclk2	btfsc	PORTB,D'007'
		goto	waitclk2

		clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp100c_rt

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          AT30TSE754 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_atmel	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		call	disp_clr1
		call	disp_clr2
		call	w_tmp_atmel

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

;clear kbd buffer----------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
;--------------------------------

tmp_atm_rt	call	tmp100_read_t2
		call	t_convert2dec
		call	w_tmp100_t

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

gwaitclk	btfss	PORTB,D'007'
		goto	gwaitclk

gwaitdat	btfss	PORTB,D'006'
		goto	gwaitdat

gwaitclk2	btfsc	PORTB,D'007'
		goto	gwaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_atm_rt


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          LM76 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_lm76	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		call	disp_clr1
		call	disp_clr2
		call	w_tmp_lm76

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

;clear kbd buffer----------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
;--------------------------------

tmp_ti_rt	call	tmp100_read_t2

		movf	v_temp1,0
		movwf	v_temp1s
		movf	v_temp2,0
		movwf	v_temp2s

		rlcf	v_temp2,1
		rlcf	v_temp1,1
		call	t_convert2dec

		movf	v_temp1s,0
		movwf	v_temp1
		movf	v_temp2s,0
		movwf	v_temp2

		call	w_tmp100_t

		call	dr2			;LM76 seems to be slow

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

twaitclk	btfss	PORTB,D'007'
		goto	twaitclk

twaitdat	btfss	PORTB,D'006'
		goto	twaitdat

twaitclk2	btfsc	PORTB,D'007'
		goto	twaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_ti_rt

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          DS1624 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_ds1624	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		call	disp_clr1
		call	disp_clr2
		call	w_tmp_ds1624

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

		call	ds1624_init

;clear kbd buffer----------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
;--------------------------------

tmp_mx_rt	call	tmp100_read_t3
		call	t_convert2dec
		call	w_tmp100_t

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

mwaitclk	btfss	PORTB,D'007'
		goto	mwaitclk

mwaitdat	btfss	PORTB,D'006'
		goto	mwaitdat

mwaitclk2	btfsc	PORTB,D'007'
		goto	mwaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_mx_rt


;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
t_convert2dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3
;
		clrf	v_tempx
;
		movlw	D'255'
		xorwf	v_temp2,0		;if LSB is eq to 255 --- probably no sensor is connected
		btfsc	STATUS,Z
		goto	temp_no_value
;
		btfsc	v_temp1,D'007'		;negative value?
		goto	negative_temp
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;positive value conversion
;
		incf	v_temp1,0		;we convert only MSB now
		movwf	TIMER1			;store "to be converted" value to data location TIMER1
		
t_next		decfsz	TIMER1,1
		goto	t_inc_t3
		goto	finito7

t_inc_t3	call	t_inc_t
		goto	t_next
;we are done.....----
finito7		movf	v_tst1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
;
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst1
;dblchk 2nd position as well
		movf	v_tst2,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst2

finito7_2	movlw	'-'
		btfsc	v_temp1,D'007'
		movwf	v_tst1		;minus sign
		return
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;negative value conversion
;
negative_temp	movf	v_temp1,0
		movwf	TIMER1

nt_next		incf	TIMER1,1
		btfsc	STATUS,Z
		goto	finito9
		call	t_inc_t
		goto	nt_next
;------------------------------------
finito9		call	t_inc_t
		goto	finito7
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
t_inc_t		nop

		incf	v_tempx,1

		incf	v_tst3,1
;----------------------------------------
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst3
;----------------------------------------
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

		return
;-----------------------------------
t_c_fin		nop		
		return

;-----------------------------------
temp_no_value	movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

		return
;-----------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                 PORTS - device first selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ports1_3	call	disp_clr1
		call	disp_clr2
		call	w_ports_m

p002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1C'		;A = Quasi bi-directional GPIO
		btfsc	STATUS,Z
		goto	p_ti

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Totem - Pole GPIO
		btfsc	STATUS,Z
		goto	p_g_b

		movf	kbd_buffer,0
		xorlw	H'21'		;C = LED drivers
		btfsc	STATUS,Z
		goto	p_nxp

		movf	kbd_buffer,0
		xorlw	H'22'		;X = other I/O devices
		btfsc	STATUS,Z
		goto	p_others

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z

		goto	p002

		goto	main_escs

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - LED drivers - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_nxp		call	disp_clr1
		call	disp_clr2
		call	w_ports_nxp


p_nxp02		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'45'		;0 = PCA9530
		btfsc	STATUS,Z
		goto	p_pca9530_s

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = PCA9531
		btfsc	STATUS,Z
		goto	p_pca9531_s

		movf	kbd_buffer,0
		xorlw	H'1E'		;2 = PCA9532
		btfsc	STATUS,Z
		goto	p_pca9532_s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_nxp02
		goto	ports1_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 device selection
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca9530_s	movlw	D'001'
		movwf	v_pca953x
		goto	p_pca9530_m
;----------------------------------------
p_pca9531_s	movlw	D'002'
		movwf	v_pca953x
		goto	p_pca9530_m
;----------------------------------------
p_pca9532_s	movlw	D'003'
		movwf	v_pca953x
		goto	p_pca9530_m

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 main screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca9530_m	call	disp_clr1
		call	disp_clr2
		call	w_pca9530m

pca9530_02	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = PCA9530 - read
		btfsc	STATUS,Z
		goto	pca9530_r

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = PCA9530 - PSC0/1 + PWM0/1 set
		btfsc	STATUS,Z
		goto	pca9530_pwm

		movf	kbd_buffer,0
		xorlw	H'4B'		;L = LEDs - register LS0
		btfsc	STATUS,Z
		goto	pca9530_L0

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	pca9530_02
		goto	p_nxp

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 PWM maintenance screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_pwm	call	disp_clr1
		call	disp_clr2

pwm_33		call	p_pxx_i2c_read
		call	w_ppwm
;-------------------------------------------------------------
;PSC0
		movf	v_psc0,0
		movwf	v_b_in
		movlw	B'10000111'
		call	hex_input
		movf	v_b_in,0
		movwf	v_psc0

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
;PWM0
		movf	v_pwm0,0
		movwf	v_b_in
		movlw	B'10010101'
		call	hex_input
		movf	v_b_in,0
		movwf	v_pwm0

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
;PSC1
		movf	v_psc1,0
		movwf	v_b_in
		movlw	B'11000111'
		call	hex_input
		movf	v_b_in,0
		movwf	v_psc1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;--------------------------------------------------------------
;PWM1
		movf	v_pwm1,0
		movwf	v_b_in
		movlw	B'11010101'
		call	hex_input
		movf	v_b_in,0
		movwf	v_pwm1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
		goto	pwm_33


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 read screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_r	call	disp_clr2

;--- read operation & display.....
		call	p9530_i2c_read
		call	w_pca9530r

pca9530r_02	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = PCA9530 - read
		btfsc	STATUS,Z
		goto	pca9530_r

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	pca9530r_02
		goto	p_pca9530_m
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 LED0/1 screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_L0	call	disp_clr1
		call	disp_clr2
n_nxp_l1	call	p_pxx_i2c_read
		call	w_LED0

;------ LS0 ------------------------------------
		movf	v_ls0,0
		movwf	v_b_in
		movlw	B'10000110'
		call	hex_input
		movf	v_b_in,0
		movwf	v_ls0

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x998			;leave now for PCA9530 device

		call	p_pxx_i2c_write

		call	w_LED0

;------ LS1 ------------------------------------
		movf	v_ls1,0
		movwf	v_b_in
		movlw	B'10001010'
		call	hex_input
		movf	v_b_in,0
		movwf	v_ls1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write

		call	w_LED0

;------ LS2 ------------------------------------
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x998			;leave now for PCA9531 device

		movf	v_ls2,0
		movwf	v_b_in
		movlw	B'10001110'
		call	hex_input
		movf	v_b_in,0
		movwf	v_ls2

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write

		call	w_LED0
;------ LS3 ------------------------------------
		movf	v_ls3,0
		movwf	v_b_in
		movlw	B'10010010'
		call	hex_input
		movf	v_b_in,0
		movwf	v_ls3

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

x998		nop
		call	p_pxx_i2c_write

		goto	n_nxp_l1

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group X - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_others	call	disp_clr1
		call	disp_clr2
		call	w_ports_oth

p_ot002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'21'		;C = TPIC2810
		btfsc	STATUS,Z
		goto	p_tpic2810

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_ot002
		goto	ports1_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group B - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_g_b		call	disp_clr1
		call	disp_clr2
		call	w_ports_gb

p_gb002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'36'		;6 = PCA6107
		btfsc	STATUS,Z
		goto	p_pca6107

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = PCA9534A
		btfsc	STATUS,Z
		goto	p_pca9534A

		movf	kbd_buffer,0
		xorlw	H'3A'		;M = MCP23008
		btfsc	STATUS,Z
		goto	p_mcp23008

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_gb002
		goto	ports1_3

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group A - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_ti		call	disp_clr1
		call	disp_clr2
		call	w_ports_ti

p_ti002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = PCF8574
		btfsc	STATUS,Z
		goto	p_pcf8574

		movf	kbd_buffer,0
		xorlw	H'2E'		;5 = PCF8575
		btfsc	STATUS,Z
		goto	p_pcf8575

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_ti002
		goto	ports1_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------


;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_tpic2810	movlw	D'010'			;device = TPIC2810
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		call	disp_clr1
		call	disp_clr2

		goto	p_75_1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca6107	movlw	D'003'			;device = PCA6107
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		call	disp_clr1
		call	disp_clr2

		goto	p_75_1
;-----------------------------------------------------------------------------
p_pca9534A	movlw	D'004'			;device = PCA9534A
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		call	disp_clr1
		call	disp_clr2

		goto	p_75_1

;-----------------------------------------------------------------------------
p_mcp23008	movlw	D'005'			;device = MCP23008
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		call	disp_clr1
		call	disp_clr2

		goto	p_75_1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pcf8574	nop
		movlw	D'001'			;device = PCF8574
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		goto	p_75_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pcf8575	nop

		movlw	D'002'			;device = PCF8575
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0
		movwf	v_p1

		movlw	D'015'
		movwf	v_cursor2
p_75_3		nop

		call	disp_clr1
		call	disp_clr2

		call	p_send1		;set all ports to "1"
;
		call	p_read1		;read the port values

;------------------------------------------------------
p_75_1		call	w_pcf8575

p_75_2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'		;S - Send data to the device
		btfsc	STATUS,Z
		call	p_send1

		movf	kbd_buffer,0
		xorlw	H'2D'		;R - Read data from the device
		btfsc	STATUS,Z
		goto	p_read2

		movf	kbd_buffer,0
		xorlw	H'23'		;D - PCF8574 Demo / MCP23008 Detail Info
		btfsc	STATUS,Z
		goto	p_demo7

		movf	kbd_buffer,0
		xorlw	H'45'		;0 - 0 was pressed
		btfsc	STATUS,Z
		goto	p_key0

		movf	kbd_buffer,0
		xorlw	H'16'		;1 - 1 was pressed
		btfsc	STATUS,Z
		goto	p_key1

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfss	STATUS,Z
		goto	p_75_2

;------------------------------------------------------
;return procedure
;------------------------------------------------------
		movf	v_p_device,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	p_others		;return to "Others" sub-menu

		movf	v_p_device,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	p_g_b

		movf	v_p_device,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	p_g_b

		movf	v_p_device,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	p_g_b


		goto	p_ti

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read2		call	p_read1
		goto	p_75_1
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_demo7		movf	v_p_device,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	p_demo8574

		movf	v_p_device,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	p_detail23008

		goto	p_75_2
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

p_detail23008	call	disp_clr1
		call	disp_clr2

		call	read_mcp23008

		goto	w_r7
;1 -------------------------------------------
		movlw	B'10000000'
		call	send_comm
		call	wm_I
		call	wm_O
		call	wm_D
		call	wm_I
		call	wm_R
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg00,0
		movwf	kbd_buffer
		call	write_bin
;
		movlw	B'11000000'
		call	send_comm
		call	wm_I
		call	wm_P
		call	wm_O
		call	wm_L
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'11001001'
		call	send_comm
		movf	v_reg01,0
		movwf	kbd_buffer
		call	write_bin

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


;2 -------------------------------------------
		movlw	B'10000000'
		call	send_comm
		call	wm_G
		call	wm_P
		call	wm_I
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_N
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg02,0
		movwf	kbd_buffer
		call	write_bin
;
		movlw	B'11000000'
		call	send_comm
		call	wm_D
		call	wm_E
		call	wm_F
		call	wm_V
		call	wm_A
		call	wm_L
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'11001001'
		call	send_comm
		movf	v_reg03,0
		movwf	kbd_buffer
		call	write_bin

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


;3 -------------------------------------------
		movlw	B'10000000'
		call	send_comm
		call	wm_I
		call	wm_N
		call	wm_T
		call	wm_C
		call	wm_O
		call	wm_N
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg04,0
		movwf	kbd_buffer
		call	write_bin
;
		movlw	B'11000000'
		call	send_comm
		call	wm_I
		call	wm_O
		call	wm_C
		call	wm_O
		call	wm_N
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'11001001'
		call	send_comm
		movf	v_reg05,0
		movwf	kbd_buffer
		call	write_bin

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


;4 -------------------------------------------
		movlw	B'10000000'
		call	send_comm
		call	wm_G
		call	wm_P
		call	wm_P
		call	wm_U
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg06,0
		movwf	kbd_buffer
		call	write_bin
;
		call	disp_clr2

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


;5 -------------------------------------------
w_r7		movlw	B'10000000'
		call	send_comm
		call	wm_I
		call	wm_N
		call	wm_T
		call	wm_F
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg07,0
		movwf	kbd_buffer
		call	write_bin


		movlw	B'11000000'
		call	send_comm
		call	wm_I
		call	wm_N
		call	wm_T
		call	wm_C
		call	wm_A
		call	wm_P
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'11001001'
		call	send_comm
		movf	v_reg08,0
		movwf	kbd_buffer
		call	write_bin

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


;6 -------------------------------------------
		movlw	B'10000000'
		call	send_comm
		call	wm_G
		call	wm_P
		call	wm_I
		call	wm_O
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'10001001'
		call	send_comm
		movf	v_reg09,0
		movwf	kbd_buffer
		call	write_bin


		movlw	B'11000000'
		call	send_comm
		call	wm_O
		call	wm_L
		call	wm_A
		call	wm_T
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_dvojbodka
		movlw	B'11001001'
		call	send_comm
		movf	v_reg10,0
		movwf	kbd_buffer
		call	write_bin

		call	pdx_wfk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	p_75_3


		goto	p_detail23008
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
pdx_wfk		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER -> exit
		btfsc	STATUS,Z
		return

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		return

		goto	pdx_wfk
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
read_mcp23008	call	i2c_start
		movlw	B'01000000'
		call	i2c_send
		movlw	H'07'			;first register to be readed
		call	i2c_send
		call	i2c_stop
;
		call	i2c_start
		movlw	B'01000001'
		call	i2c_send

		goto	read_r7

		call	i2c_receive
		movwf	v_reg00
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg01
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg02
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg03
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg04
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg05
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg06
		call	i2c_ack

read_r7		call	i2c_receive
		movwf	v_reg07
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg08
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg09
		call	i2c_ack
		call	i2c_receive
		movwf	v_reg10
		call	i2c_not_ack
		call	i2c_stop

		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_demo8574	call	disp_clr1
		call	disp_clr2

;start I2C communication
		call	i2c_start
		movlw	B'01000000'
		call	i2c_send

		movlw	B'11111110'
		call	demo8574_send
		movlw	B'11111101'
		call	demo8574_send
		movlw	B'11111011'
		call	demo8574_send
		movlw	B'11110111'
		call	demo8574_send
		movlw	B'11101111'
		call	demo8574_send
		movlw	B'11011111'
		call	demo8574_send
		movlw	B'10111111'
		call	demo8574_send
		movlw	B'01111111'
		call	demo8574_send
		movlw	B'11111111'
		call	demo8574_send

;finish the communication
		call	i2c_stop

		goto	p_75_3
;------------------------------------------------------------------------------------
demo8574_send	movwf	v_p0
		call	w_demo8574
		movf	v_p0,0
		call	i2c_send
		call	d1333
		return
;---------------------------------------------
;---------------------------------------------
d1332		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1331	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP1332	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP1332
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1331
		return
;---------------------------------------------
d1333		call	d1332
		call	d1332
		return
;---------------------------------------------
d332		MOVLW   D'001'
		MOVWF   TIMER2
DEL_LOOP331	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP332	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP332
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP331
		return

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

p_key0		nop

;-- H -----------------------------------
		movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		bcf	v_p1,D'007'
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		bcf	v_p1,D'006'
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		bcf	v_p1,D'005'
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		bcf	v_p1,D'004'
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		bcf	v_p1,D'003'
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		bcf	v_p1,D'002'
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		bcf	v_p1,D'001'
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		bcf	v_p1,D'000'
;-- L -----------------------------------
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		bcf	v_p0,D'007'
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		bcf	v_p0,D'006'
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		bcf	v_p0,D'005'
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		bcf	v_p0,D'004'
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		bcf	v_p0,D'003'
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bcf	v_p0,D'002'
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		bcf	v_p0,D'001'
;
		movf	v_cursor2,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bcf	v_p0,D'000'
;
		goto	inc_c_pos
;----------------------------------------------------------
;----------------------------------------------------------
p_key1		nop

;-- H -----------------------------------
		movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		bsf	v_p1,D'007'
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		bsf	v_p1,D'006'
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		bsf	v_p1,D'005'
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		bsf	v_p1,D'004'
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		bsf	v_p1,D'003'
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		bsf	v_p1,D'002'
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		bsf	v_p1,D'001'
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		bsf	v_p1,D'000'
;-- L -----------------------------------
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		bsf	v_p0,D'007'
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		bsf	v_p0,D'006'
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		bsf	v_p0,D'005'
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		bsf	v_p0,D'004'
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		bsf	v_p0,D'003'
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bsf	v_p0,D'002'
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		bsf	v_p0,D'001'
;
		movf	v_cursor2,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bsf	v_p0,D'000'
;
		goto	inc_c_pos
;--------------------------------------------------
inc_c_pos	call	d332		;wait a while...

		decf	v_cursor2,1
		movf	v_cursor2,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	set_c1
		goto	p_75_1

set_c1		movlw	D'015'		;PCF8575 (16-bit)
		movwf	v_cursor2

		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	set_c2

		goto	p_75_1

set_c2		movlw	D'007'		;not PCF8575 (8-bit)
		movwf	v_cursor2

		goto	p_75_1

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
w_pcf8575	nop
		movf	v_p_device,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	w_pcf8574

		movf	v_p_device,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	w_pca6107

		movf	v_p_device,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	w_pca9534A

		movf	v_p_device,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	w_mcp23008

		movf	v_p_device,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	w_tpic2810

		movlw	B'10001100'		;1/12
		call	send_comm

		call	wm_minus
		movlw	'<'
		call	send_data
		movlw	B'10010111'		;1/23
		call	send_comm
		movlw	'>'
		call	send_data

w_p0_main	movlw	B'11001100'		;2/12
		call	send_comm

		call	wm_minus
		movlw	'<'
		call	send_data
		movlw	B'11010111'		;2/23
		call	send_comm
		movlw	'>'
		call	send_data

		call	w_p_data
		call	w_r_data

;---------------------------------------------
;set the cursor for "binary editor"
		call	set_cursor2
		call	send_comm
;---------------------------------------------
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_r_data	movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	w_rd2

		movlw	B'10001110'		;1/14
		call	send_comm

		movf	v_r1,0
		movwf	c_tmp

		call	w_p_data_s

w_rd2		movlw	B'11001110'		;2/14
		call	send_comm

		movf	v_r0,0
		movwf	c_tmp

		call	w_p_data_s

		return

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_p_data	movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	w_p2

		movlw	B'10000000'		;1/0
		call	send_comm

		call	wm_P
		call	wm_1
		call	wm_dvojbodka

		movf	v_p1,0
		movwf	c_tmp

		call	w_p_data_s


w_p2		movlw	B'11000000'		;2/0
		call	send_comm

		call	wm_P
		call	wm_0
		call	wm_dvojbodka

		movf	v_p0,0
		movwf	c_tmp

		call	w_p_data_s

		return
		
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_p_data_s	movlw	'1'
		btfss	c_tmp,D'007'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'006'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'005'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'004'
		movlw	'0'
		call	send_data

		call	wm_space

		movlw	'1'
		btfss	c_tmp,D'003'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'002'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'001'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'000'
		movlw	'0'
		call	send_data

		return

;------------------------------------------------------
set_cursor2	movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		goto	wpdsc_15
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		goto	wpdsc_14
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		goto	wpdsc_13
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		goto	wpdsc_12
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		goto	wpdsc_11
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	wpdsc_10
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	wpdsc_09
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	wpdsc_08
;
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	wpdsc_07
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	wpdsc_06
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	wpdsc_05
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	wpdsc_04
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	wpdsc_03
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	wpdsc_02
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	wpdsc_01
;------------------------------------------
		movlw	B'11001011'	;11
		return
;------------------------------------------
wpdsc_01	movlw	B'11001010'	;10
		return
;------------------------------------------
wpdsc_02	movlw	B'11001001'	;09
		return
;------------------------------------------
wpdsc_03	movlw	B'11001000'	;08
		return
;------------------------------------------
wpdsc_04	movlw	B'11000110'	;06
		return
;------------------------------------------
wpdsc_05	movlw	B'11000101'	;05
		return
;------------------------------------------
wpdsc_06	movlw	B'11000100'	;04
		return
;------------------------------------------
wpdsc_07	movlw	B'11000011'	;03
		return
;------------------------------------------
wpdsc_08	movlw	B'10001011'	;11
		return
;------------------------------------------
wpdsc_09	movlw	B'10001010'	;10
		return
;------------------------------------------
wpdsc_10	movlw	B'10001001'	;09
		return
;------------------------------------------
wpdsc_11	movlw	B'10001000'	;08
		return
;------------------------------------------
wpdsc_12	movlw	B'10000110'	;06
		return
;------------------------------------------
wpdsc_13	movlw	B'10000101'	;05
		return
;------------------------------------------
wpdsc_14	movlw	B'10000100'	;04
		return
;------------------------------------------
wpdsc_15	movlw	B'10000011'	;03
		return

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_pcf8574	movlw	B'10000000'		;1/0
		call	send_comm

		movlw	high t_pcf8574
		movwf	TBLPTRH
		movlw	low t_pcf8574
		movwf	TBLPTRL
		call	w_send_txt

		goto	w_p0_main
;-------------------------------------------------------------------------
w_tpic2810	movlw	B'10000000'		;1/0
		call	send_comm

		movlw	high t_tpic2810
		movwf	TBLPTRH
		movlw	low t_tpic2810
		movwf	TBLPTRL
		call	w_send_txt

		goto	w_p0_main
;-------------------------------------------------------------------------
w_pca9534A	movlw	B'10000000'		;1/0
		call	send_comm

		movlw	high t_pca9534A
		movwf	TBLPTRH
		movlw	low t_pca9534A
		movwf	TBLPTRL
		call	w_send_txt

		goto	w_p0_main
;-------------------------------------------------------------------------
w_mcp23008	movlw	B'10000000'		;1/0
		call	send_comm

		movlw	high t_mcp23008
		movwf	TBLPTRH
		movlw	low t_mcp23008
		movwf	TBLPTRL
		call	w_send_txt

		goto	w_p0_main
;-------------------------------------------------------------------------
w_pca6107	movlw	B'10000000'		;1/0
		call	send_comm

		movlw	high t_pca6107
		movwf	TBLPTRH
		movlw	low t_pca6107
		movwf	TBLPTRL
		call	w_send_txt

		goto	w_p0_main
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ppwm		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_S
		call	wm_C
		call	wm_0
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		movf	v_psc0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_space

		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_0
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		movf	v_pwm0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_S
		call	wm_C
		call	wm_1
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		movf	v_psc1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_space

		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_1
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		movf	v_pwm1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------

w_pca9530m	movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3

;------------------------------------
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		call	wm_0

		movf	v_pca953x,0
		xorlw	D'002'
		btfsc	STATUS,Z
		call	wm_1
;------------------------------------

		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_1
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		movlw	B'11000000'
		call	send_comm
		call	wm_zat1
		call	wm_R
		call	wm_zat2
		call	wm_e
		call	wm_a
		call	wm_d

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_zat1
		call	wm_P
		call	wm_zat2
		call	wm_W
		call	wm_M

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_zat1
		call	wm_L
		call	wm_zat2
		call	wm_E
		call	wm_D
		call	wm_s

		return
;-------------------------------------------------------------------------
w_pca9530r	movlw	B'11000000'
		call	send_comm

		movlw	'<'
		call	send_data

		movf	v_r0,0
		movwf	c_tmp

		call	w_p_data_s

		movlw	'>'
		call	send_data

		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		return

;for PCA9532 display the second input register
		call	wm_minus
		movlw	'<'
		call	send_data

		movf	v_r1,0
		movwf	c_tmp

		call	w_p_data_s

		movlw	'>'
		call	send_data

		return
;-------------------------------------------------------------------------
w_LED0		movlw	B'10000000'
		call	send_comm

		call	wm_L
		call	wm_S
		call	wm_x
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		movf	v_ls0,0
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x977
;not PCA9530
		call	wm_minus
		movf	v_ls1,0
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x977
;PCA9532
		call	wm_minus
		movf	v_ls2,0
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		call	wm_minus
		movf	v_ls3,0
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

;-------------------------------------------------
x977		nop
;second line - LED status ------------------------
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	w_LED0_1
;PCA9531 & PCA9532 part --------------------------
		movlw	B'11000000'
		call	send_comm
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_dvojbodka
		call	wm_space

		movf	v_ls0,0
		movwf	v_lsx
		call	w_LED_st4

		movf	v_ls1,0
		movwf	v_lsx
		call	w_LED_st4
;PCA9532 part ------------------------------------
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		return				;leave now for PCA9531

		movf	v_ls2,0
		movwf	v_lsx
		call	w_LED_st4

		movf	v_ls3,0
		movwf	v_lsx
		call	w_LED_st4
		return
;PCA9530 part ------------------------------------
w_LED0_1	movlw	B'11000000'
		call	send_comm
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_0
		call	wm_dvojbodka
		call	wm_space

		call	w_LEDx

		movlw	B'11001011'
		call	send_comm
		movlw	'*'
		call	send_data
		call	wm_space
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_1
		call	wm_dvojbodka
		call	wm_space

		movf	v_ls0,0
		movwf	c_tmp
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		call	w_LEDx

		return
;-------------------------------------------------------------------------
w_LED_st4	movf	v_lsx,0
		movwf	c_tmp
		call	w_LEDx2

		movf	v_lsx,0
		movwf	c_tmp
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		call	w_LEDx2

		movf	v_lsx,0
		movwf	c_tmp
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		call	w_LEDx2

		movf	v_lsx,0
		movwf	c_tmp
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		rrcf	c_tmp,1
		call	w_LEDx2
		call	wm_space
		return
;-------------------------------------------------------------------------
w_LEDx2		movlw	B'00000011'
		andwf	c_tmp,1
		movf	c_tmp,0
		btfsc	STATUS,Z
		call	wm_0
		movf	c_tmp,0
		xorlw	B'00000001'
		btfsc	STATUS,Z
		call	wm_1
		movf	c_tmp,0
		xorlw	B'00000010'
		btfsc	STATUS,Z
		call	wm_2
		movf	c_tmp,0
		xorlw	B'00000011'
		btfsc	STATUS,Z
		call	wm_3
		return
;-------------------------------------------------------------------------
w_LEDx		movlw	B'00000011'
		andwf	c_tmp,1
		movf	c_tmp,0
		btfsc	STATUS,Z
		call	w_LED_off
		movf	c_tmp,0
		xorlw	B'00000001'
		btfsc	STATUS,Z
		call	w_LED_on
		movf	c_tmp,0
		xorlw	B'00000010'
		btfsc	STATUS,Z
		call	w_LED_PWM0
		movf	c_tmp,0
		xorlw	B'00000011'
		btfsc	STATUS,Z
		call	w_LED_PWM1
		return
;----------------------------------------------------
w_LED_off	nop
		call	wm_O
		call	wm_f
		call	wm_f
		call	wm_space
		return
;----------------------------------------------------
w_LED_on	nop
		call	wm_O
		call	wm_n
		call	wm_space
		call	wm_space
		return
;----------------------------------------------------
w_LED_PWM0	nop
		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_0
		return
;----------------------------------------------------
w_LED_PWM1	nop
		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_1
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;general entry for I2C send routines for Texas Instruments Expanders
;data send routines
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_send1		movf	v_p_device,0
		xorlw	D'003'		;PCA6107
		btfsc	STATUS,Z
		goto	p_send6

		movf	v_p_device,0
		xorlw	D'004'		;PCA9534A
		btfsc	STATUS,Z
		goto	p_send9534A

		movf	v_p_device,0
		xorlw	D'005'		;MCP23008
		btfsc	STATUS,Z
		goto	p_send23008

		movf	v_p_device,0
		xorlw	D'010'		;TPIC2810
		btfsc	STATUS,Z
		goto	p_send010

;------------------------------------------------------------------------------------
;PCF8574 / PCF8575 data send routines
;------------------------------------------------------------------------------------

		call	i2c_start
		movlw	B'01000000'
		call	i2c_send

		movf	v_p0,0		;first transmitted byte is LSB (P0x)
		call	i2c_send

		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z	;continue (send second byte) only for PCF8575
		goto	ps1_1		;stop communication for PCF8574(A)

		movf	v_p1,0		;second transmitted byte is LSB (P1x)
		call	i2c_send

ps1_1		call	i2c_stop

		return

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;general entry for I2C read routines for Texas Instruments Expanders
;data receive routines
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read1		movf	v_p_device,0
		xorlw	D'003'		;PCA6107
		btfsc	STATUS,Z
		goto	p_read6

		movf	v_p_device,0
		xorlw	D'004'		;PCA9534A
		btfsc	STATUS,Z
		goto	p_read9534A

		movf	v_p_device,0
		xorlw	D'005'		;MCP23008
		btfsc	STATUS,Z
		goto	p_read23008

		movf	v_p_device,0
		xorlw	D'010'		;TPIC2810
		btfsc	STATUS,Z
		goto	p_read010

;------------------------------------------------------------------------------------
;PCF8574 / PCF8575 data send routines
;------------------------------------------------------------------------------------
		call	i2c_start
		movlw	B'01000001'
		call	i2c_send

		call	i2c_receive
		movwf	v_r0		;first transmitted byte is LSB (P0x)

		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z	;continue (ACK + receive second byte) only for PCF8575
		goto	pr1_1		;stop communication for PCF8574(A)

		call	i2c_ack

		call	i2c_receive
		movwf	v_r1		;second transmitted byte is LSB (P1x)
pr1_1		call	i2c_not_ack
		call	i2c_stop

		return

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_demo8574	movlw	B'10000000'
		call	send_comm

		call	wm_D
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_space
		call	wm_P
		call	wm_C
		call	wm_F
		call	wm_8
		call	wm_5
		call	wm_7
		call	wm_4

		goto	w_p2

;------------------------------------------------------------------------------------
;PCA6107 data send routines
;------------------------------------------------------------------------------------
p_send6_int	nop

;1 - set all 8 bits to output ------
		call	xi2c_start
		movlw	B'00111110'		;PCA6107 I2C address + write
		call	xi2c_send
		movlw	B'00000011'		;pointer to config register
		call	xi2c_send
		movlw	B'00000000'		;all bits as output
		call	xi2c_send
		call	i2c_stop

;2 - write v_p0 to the device ------
		call	xi2c_start
		movlw	B'00111110'		;PCA6107 I2C address + write
		call	xi2c_send
		movlw	B'00000001'		;pointer to output register
		call	xi2c_send
		movf	v_board_LEDs,0		;data to be send
		call	xi2c_send
		call	i2c_stop
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
p_send9534A	movlw	B'01110000'
		movwf	v_i2c_add
		goto	p_send6_gen
;---------------------------------------------------------------------------------
p_send6		movlw	B'00110000'
		movwf	v_i2c_add
		goto	p_send6_gen
;---------------------------------------------------------------------------------
p_send6_gen	nop

;1 - set all 8 bits to output ------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000011'		;pointer to config register
		call	i2c_send
		movlw	B'00000000'		;all bits as output
		call	i2c_send
		call	i2c_stop

;2 - write v_p0 to the device ------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000001'		;pointer to output register
		call	i2c_send
		movf	v_p0,0			;data to be send
		call	i2c_send
		call	i2c_stop
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;MCP23008 communication routines
;  we are using BANK = 0 and autoincrement feature (power on defaults)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;GPIO as output
;-----------------------------------------------------------------------------
;configuration
;-----------------------------------------------------------------------------

p_send23008	call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	B'00000000'		;address of the IODIR register
		call	i2c_send
		movlw	B'00000000'		;all bits of GPIO to output
		call	i2c_send
		call	i2c_stop
;-----------------------------------------------------------------------------
;send_data
;-----------------------------------------------------------------------------
		call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'09'			;GPIO register
		call	i2c_send
		movf	v_p0,0			;data to be send
		call	i2c_send
		call	i2c_stop
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;GPIO as input
;-----------------------------------------------------------------------------
;configuration
;-----------------------------------------------------------------------------

p_read23008	call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	B'00000000'		;address of the IODIR register
		call	i2c_send
		movlw	B'11111111'		;all bits of GPIO to input
		call	i2c_send
		call	i2c_stop
;enable pull-ups
		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'06'			;address of the GPPU register
		call	i2c_send
		movlw	B'11111111'		;enable all pull-up resistors
		call	i2c_send
		call	i2c_stop
;enable interrupt on change
		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'02'			;address of the GPINTEN register
		call	i2c_send
		movlw	B'11111111'		;enable all pull-up resistors
		call	i2c_send
		call	i2c_stop
;-----------------------------------------------------------------------------
;read data
;-----------------------------------------------------------------------------
		call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'09'			;GPIO register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	B'01000001'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		call	i2c_receive
		movwf	v_r0
		call	i2c_not_ack
		call	i2c_stop
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;TPIC2810 data send routines
;------------------------------------------------------------------------------------
p_send010	nop
		call	i2c_start
		movlw	B'11000000'		;TPIC2810 I2C address + write
		call	i2c_send
		movlw	H'44'			;subaddress= 44h
		call	i2c_send

		movf	v_p0,0			;data to be send

		call	i2c_send
		call	i2c_stop
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read6		movlw	B'00110000'		;PCA6107
		movwf	v_i2c_add
		goto	p_read6_gen
;------------------------------------------------------------------------------------
p_read9534A	movlw	B'01110000'		;PCA9534A
		movwf	v_i2c_add
		goto	p_read6_gen

;------------------------------------------------------------------------------------
;TI Group B data read routines
;------------------------------------------------------------------------------------
p_read6_gen	nop

;1 - set all 8 bits to input ------------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000011'		;pointer to config register
		call	i2c_send
		movlw	B'11111111'		;all bits as input
		call	i2c_send
		call	i2c_stop

;2 - set inversion register ------------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000010'		;pointer to polarity inversion register
		call	i2c_send
		movlw	B'00000000'		;all bits w/o polarity inversion
		call	i2c_send
		call	i2c_stop

;3 - receive data from the device ------
;    and write to v_r0
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000000'		;pointer to input register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movf	v_i2c_add,0		;device I2C address
		addlw	D'001'			;+ read
		call	i2c_send

		call	i2c_receive
		movwf	v_r0

		call	i2c_not_ack
		call	i2c_stop
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read6_int		nop

;1 - set all 8 bits to input ------------
		call	xi2c_start
		movlw	B'00111110'		;PCA6107 I2C address + write
		call	xi2c_send
		movlw	B'00000011'		;pointer to config register
		call	xi2c_send
		movlw	B'11111111'		;all bits as input
		call	xi2c_send
		call	i2c_stop

;2 - set inversion register ------------
		call	xi2c_start
		movlw	B'00111110'		;PCA6107 I2C address + write
		call	xi2c_send
		movlw	B'00000010'		;pointer to polarity inversion register
		call	xi2c_send
		movlw	B'00000000'		;all bits w/o polarity inversion
		call	xi2c_send
		call	i2c_stop

;3 - receive data from the device ------
;    and write to v_r0
		call	xi2c_start
		movlw	B'00111110'		;PCA6107 I2C address + write
		call	xi2c_send
		movlw	B'00000001'		;pointer to output register
		call	xi2c_send
		call	i2c_stop

		call	xi2c_start
		movlw	B'00111111'		;PCA6107 I2C address + read
		call	xi2c_send

		call	i2c_receive
		movwf	v_r0

		call	i2c_not_ack
		call	i2c_stop
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;TPIC2810 data read routines
;------------------------------------------------------------------------------------
p_read010	nop

;1 - set all 8 bits to input ------------
		call	i2c_start
		movlw	B'11000000'		;TPIC2810 I2C address + write
		call	i2c_send
		movlw	H'11'			;subaddress
		call	i2c_send

		call	i2c_restart
		movlw	B'11000001'		;TPIC2810 I2C address + read
		call	i2c_send
		call	i2c_receive
		movwf	v_r0

		call	i2c_not_ack
		call	i2c_stop
		return

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_gb	movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_zat1
		call	wm_6
		call	wm_zat2
		call	wm_1
		call	wm_0
		call	wm_7

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_zat1
		call	wm_4
		call	wm_zat2
		call	wm_A

		movlw	B'11000000'
		call	send_comm

		call	wm_zat1
		call	wm_M
		call	wm_zat2
		call	wm_C
		call	wm_P
		call	wm_2
		call	wm_3
		call	wm_0
		call	wm_0
		call	wm_8

		return


;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH





;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
; "LED blink" routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

LED_blink	movlw	D'003'
		movwf	v_ttt

nn		nop
		bsf	PORTA,D'007'
		call	d2
		bcf	PORTA,D'007'
		call	d2

		decfsz	v_ttt,1
		goto	nn

		call	d2
		call	d2
		call	d2
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX




;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5
;R/W = RE1 -- N/C
;E   = RE0 -- RA4

;D0 - D3 -- not connected

;D4  = RD0 -- RA0
;D5  = RD1 -- RA1
;D6  = RD2 -- RA2
;D7  = RD3 -- RA3


disp_init	nop

		clrf	TBLPTRU

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTA
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTA,D'005'
		bcf	PORTA,D'004'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------

set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTA,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTA,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTA,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTA,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTA,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTA,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTA,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTA,D'003'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTA,D'004'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTA,D'005'		;RS -> 1 (data)
		bsf	PORTA,D'004'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTA,D'004'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTA,D'004'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTA,D'005'		;RS -> 0 (command)
		bsf	PORTA,D'004'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTA,D'004'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


disp_clr1_p4	movlw	B'10000100'		;line 1 / char4
		call	send_comm
		goto	disp_clrx
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; sending text string to LCD; end of string is recognized with FEh
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
w_send_txt	tblrd*+
		movf	TABLAT,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		return			;end of string reached

		movf	TABLAT,0
		btfss	STATUS,Z	;do not display 00h
		call	send_data
		goto	w_send_txt

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_release1
		movwf	TBLPTRH
		movlw	low t_release1
		movwf	TBLPTRL

		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	low t_release2
		movwf	TBLPTRL
		call	w_send_txt

		return
;-------------------------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_brd_setup	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_s1
		movwf	TBLPTRH
		movlw	low t_s1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_s2
		movwf	TBLPTRH
		movlw	low t_s2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------
w_mm001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_mm1
		movwf	TBLPTRH
		movlw	low t_mm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_mm2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------
w_m001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_menu_main1
		movwf	TBLPTRH
		movlw	low t_menu_main1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_menu_main2
		movwf	TBLPTRL
		call	w_send_txt
		return

;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_wkbd		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_kbd_wait
		movwf	TBLPTRH
		movlw	low t_kbd_wait
		movwf	TBLPTRL
		call	w_send_txt

		return

;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_tm		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_tm1
		movwf	TBLPTRH
		movlw	low t_tm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_tm2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------

;-------------------------------------------------------------------------------
w_tmp100_t	movlw	B'10000101'		;row: 1 / col: 5
		call	send_comm

		movf	v_temp1,0		;MSB
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus

		movf	v_temp2,0		;LSB
		movwf	kbd_buffer
		call	write_hex

		movlw	B'10010011'		;row 1 / col: 19
		call	send_comm
		movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		call	wm_stup
		call	wm_C

		return
;-------------------------------------------------------------------------------
w_tmp100	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_T
		call	wm_M
		call	wm_P
		call	wm_hvizdicka
		call	wm_sls
		call	wm_A
		call	wm_T
		call	wm_hvizdicka
		call	wm_and
		call	wm_i
		call	wm_n
		call	wm_i
		call	wm_t
		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_atmel	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_T
		call	wm_M
		call	wm_P
		call	wm_hvizdicka
		call	wm_sls
		call	wm_A
		call	wm_T
		call	wm_hvizdicka
		call	wm_and
		call	wm_e
		call	wm_a
		call	wm_s
		call	wm_y
		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_lm76	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_L
		call	wm_M
		call	wm_7
		call	wm_6

		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_ds1624	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_6
		call	wm_2
		call	wm_4

		goto	w_temp_all
;-------------------------------------------------------------------------------
w_temp_all	movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	w_i2c_add

w_temp_t_all	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_R
		call	wm_e
		call	wm_g
		call	wm_dvojbodka
		call	wm_space

		movlw	B'10001011'		;row: 1 / col: 11
		call	send_comm
		call	wm_minus
		call	wm_sip1
		call	wm_space
		call	wm_T
		call	wm_e
		call	wm_m
		call	wm_p
		call	wm_dvojbodka

		return
;-------------------------------------------------------------------------------

;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_i2c_s		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_i2c_s1
		movwf	TBLPTRH
		movlw	low t_i2c_s1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_i2c_s2
		movwf	TBLPTRH
		movlw	low t_i2c_s2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_ports_m	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ports_m1
		movwf	TBLPTRH
		movlw	low t_ports_m1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_ports_m2
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_ports_ti	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ports_ti1
		movwf	TBLPTRH
		movlw	low t_ports_ti1
		movwf	TBLPTRL
		call	w_send_txt

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_ports_nxp	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ports_nxp1
		movwf	TBLPTRH
		movlw	low t_ports_nxp1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	low t_ports_nxp2
		movwf	TBLPTRL
		call	w_send_txt

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_oth	movlw	B'10000000'
		call	send_comm

		call	wm_T
		call	wm_P
		call	wm_I
		call	wm_zat1
		call	wm_C
		call	wm_zat2
		call	wm_2
		call	wm_8
		call	wm_1
		call	wm_0

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin2	movf	v_ee_i2c,0
		movwf	kbd_buffer
		goto	write_bin7
;--------------------------------------------

w_i2c_add	call	wm_A
		call	wm_dvojbodka
		movf	v_i2c_add,0
		movwf	kbd_buffer

;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;---------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; send data to the keyboard
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;RB7 = kbd clock
;RB6 = kbd data
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
send_ck		nop

;IR-keyboard buffer handling -------------------------------------------
		movf	kbd_out,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		bsf	v_ir_kbd_new,D'000'		;Re-send command

		movf	kbd_out,0
		xorlw	H'F4'
		btfsc	STATUS,Z
		clrf	v_ir_kbd			;clear buffer
;----------------------------------------------------------------------

		clrf	kbd_parity
		bsf	kbd_parity,D'000'

;-----------------------------------------------------------
;1.1. set RB7 (keaboard clock) to output ----------------------
		bcf	TRISB,D'007'
;-----------------------------------------------------------
;1.2. put KBD CLK to 0 ----------------------------------------
		bcf	PORTB,D'007'
		call	d57		;wait a while
;-----------------------------------------------------------
;2. put KBD DATA to 0 ---------------------------------------
		bcf	TRISB,D'006'

		bcf	PORTB,D'006'
;-----------------------------------------------------------
;3. release KBD CLK -----------------------------------------
		bsf	PORTB,D'007'

		bsf	TRISB,D'007'
;-----------------------------------------------------------
;4. wait for keayboard to bring the kbd clk low ------------
sck01		btfsc	PORTB,D'007'
		goto	sck01

		movf	kbd_out,0
		andlw	B'00000001'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000010'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000100'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00001000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00010000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00100000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'01000000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'10000000'
		call	send_kbd_bit

;now send the parity bit...
		movf	kbd_parity,0
		andlw	B'00000001'
		call	send_kbd_bit

;		movlw	B'00000001'		;stop bit
;		andlw	B'00000001'
;		call	send_kbd_bit

		bsf	TRISB,D'006'		;data as input

w0079		btfsc	PORTB,D'007'
		goto	w0079

w0080		btfss	PORTB,D'007'
		goto	w0080

w0081		btfsc	PORTB,D'007'
		goto	w0081

w0082		btfss	PORTB,D'007'
		goto	w0082

		return

;-----------------------------------------------------------
send_kbd_bit	btfsc	STATUS,Z
		goto	send_kbd0
		goto	send_kbd1

send_kbd0	bcf	PORTB,D'006'

w0077		btfss	PORTB,D'007'
		goto	w0077

w0078		btfsc	PORTB,D'007'
		goto	w0078

		return

send_kbd1	bsf	PORTB,D'006'
		movlw	B'00000001'
		xorwf	kbd_parity,1
		goto	w0077
;-----------------------------------------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;get a transmission from the keyboard
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

get_datafk	nop
		btfsc	PORTB,D'007'
		goto	kbd_err01		;clock level is 1 instead of expected 0
		btfsc	PORTB,D'006'
		goto	kbd_err02		;data level is 1 instead of expected 0 (start bit)
;
get_datafk2	nop
		clrf	kbd_buffer

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'000'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'001'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'002'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'003'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'004'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'005'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'006'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'007'

;get parity bit

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		nop

;compare parity bit

		nop				;not now.... :-)

;wait for communication end - last bit should be "1"

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		nop

		nop				;no check right now...

		movf	kbd_buffer,0
		xorlw	H'FA'			;keaboard returns ACK
		btfsc	STATUS,Z
		goto	ir_kbd2

		movf	kbd_buffer,0
		btfss	STATUS,Z
		return

;code is 00 or FA - consider IR keyboard
ir_kbd2		btfss	v_ir_kbd_new,D'000'
		return
		movf	v_ir_kbd,0
		movwf	kbd_buffer
		clrf	v_ir_kbd_new

		return				;keyboard code is in kbd_buffer

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;get bit from the keyboard during transmission
get_bitfk	nop
kbd_l2		btfss	PORTB,D'007'
		goto	kbd_l2			;wait to 1 on clock line
kbd_l3		btfsc	PORTB,D'007'
		goto	kbd_l3			;and now wait to 0 on clock
;
		movf	PORTB,0
		andlw	B'01000000'		;now we have the bit in W1
		movwf	kbd_bb

		return
;-----------------------------------------------------------------------------
kbd_err01	nop
kbd_err02	nop

;consider IR keyboard -----------------------
		btfss	v_ir_kbd_new,D'000'
		return
		movf	v_ir_kbd,0
		movwf	kbd_buffer
		clrf	v_ir_kbd_new

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;in W is the command to be send to display (posintion in display)
;
;output will be shifted to the variable		'v_b_in'
;value of the variable				'v_comm' will be:
;
;							ESC		01
;							ENTER		02
;							F10		03
;							other		FF
;------------------------------------------------------------------------------
hex_input	movwf	v_cursor
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'11110000'
		andwf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1

		call	sa_key_a
		movf	v_comm,0
		btfss	STATUS,Z
;exit - something happens
		goto	hi_exit_h

		movf	b_ax,0
		call	send_data
		call	hi_trans_h

		incf	v_cursor,0
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'00001111'
		andwf	b_ax_h,1

		call	sa_key_a
		movf	v_comm,0
		btfss	STATUS,Z
		goto	hi_exit_l

		movf	b_ax,0
		call	send_data
		call	hi_trans_l
		return

;now we have to translate the value back to HEX

hi_exit_h	call	hi_trans_h
		return

hi_exit_l	call	hi_trans_l
		return


hi_trans_h	rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		movlw	B'11110000'
		andwf	b_ax_h,1
		movlw	B'00001111'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

hi_trans_l	movlw	B'00001111'
		andwf	b_ax_h,1
		movlw	B'11110000'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

;-------------------------------------------------
hi_leave_esc	movlw	H'01'
		movwf	v_comm
		call	dr77
		return

hi_leave_enter	movlw	H'02'
		movwf	v_comm
		call	dr77
		return

hi_leave_F10	movlw	H'03'
		movwf	v_comm
		call	dr77
		return

hi_leave_oth	movlw	H'FE'
		movwf	v_comm
		call	dr77
		return
;-------------------------------------------------

;--------------------------------------------------------------------------
sa_key_a	movlw	H'00'
		movwf	v_comm

		movf	tmp_ot,0
		call	send_comm

sa_key3_a	btfss	PORTB,D'007'
		goto	sa_key2_a
		goto	sa_key3_a

;
sa_key2_a	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC
		btfsc	STATUS,Z
		goto	hi_leave_esc

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	hi_leave_enter

		movf	kbd_buffer,0
		xorlw	H'09'		;F10
		btfsc	STATUS,Z
		goto	hi_leave_F10

		movf	kbd_buffer,0
		xorlw	H'74'		;->
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'6B'		;<-
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth



		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		movf	kbd_buffer,0
		xorlw	H'1C'		;A
		btfsc	STATUS,Z
		goto	set_avA

		movf	kbd_buffer,0
		xorlw	H'32'		;B
		btfsc	STATUS,Z
		goto	set_avB

		movf	kbd_buffer,0
		xorlw	H'21'		;C
		btfsc	STATUS,Z
		goto	set_avC

		movf	kbd_buffer,0
		xorlw	H'23'		;D
		btfsc	STATUS,Z
		goto	set_avD

		movf	kbd_buffer,0
		xorlw	H'24'		;E
		btfsc	STATUS,Z
		goto	set_avE

		movf	kbd_buffer,0
		xorlw	H'2B'		;F
		btfsc	STATUS,Z
		goto	set_avF

		goto	sa_key3_a
;-----------------------------------
set_av0		movlw	'0'
		movwf	b_ax
		movlw	H'00'
		movwf	b_ax_h
		call	dr77
		return

set_av1		movlw	'1'
		movwf	b_ax
		movlw	H'01'
		movwf	b_ax_h
		call	dr77
		return

set_av2		movlw	'2'
		movwf	b_ax
		movlw	H'02'
		movwf	b_ax_h
		call	dr77
		return

set_av3		movlw	'3'
		movwf	b_ax
		movlw	H'03'
		movwf	b_ax_h
		call	dr77
		return

set_av4		movlw	'4'
		movwf	b_ax
		movlw	H'04'
		movwf	b_ax_h
		call	dr77
		return

set_av5		movlw	'5'
		movwf	b_ax
		movlw	H'05'
		movwf	b_ax_h
		call	dr77
		return

set_av6		movlw	'6'
		movwf	b_ax
		movlw	H'06'
		movwf	b_ax_h
		call	dr77
		return

set_av7		movlw	'7'
		movwf	b_ax
		movlw	H'07'
		movwf	b_ax_h
		call	dr77
		return

set_av8		movlw	'8'
		movwf	b_ax
		movlw	H'08'
		movwf	b_ax_h
		call	dr77
		return

set_av9		movlw	'9'
		movwf	b_ax
		movlw	H'09'
		movwf	b_ax_h
		call	dr77
		return

set_avA		movlw	'A'
		movwf	b_ax
		movlw	H'0A'
		movwf	b_ax_h
		call	dr77
		return

set_avB		movlw	'B'
		movwf	b_ax
		movlw	H'0B'
		movwf	b_ax_h
		call	dr77
		return

set_avC		movlw	'C'
		movwf	b_ax
		movlw	H'0C'
		movwf	b_ax_h
		call	dr77
		return

set_avD		movlw	'D'
		movwf	b_ax
		movlw	H'0D'
		movwf	b_ax_h
		call	dr77
		return

set_avE		movlw	'E'
		movwf	b_ax
		movlw	H'0E'
		movwf	b_ax_h
		call	dr77
		return

set_avF		movlw	'F'
		movwf	b_ax
		movlw	H'0F'
		movwf	b_ax_h
		call	dr77
		return


dr77		goto	d2
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------

;---------------------------------------------------------------------------
w_otaznik	call	send_comm
;
		call	wm_otaznik
;
		movf	tmp_ot,0
		call	send_comm
		return
;---------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sa_key		nop

		call	w_rtcm_dt
		movf	tmp_ot,0
		call	w_otaznik

sa_key3		btfss	PORTB,D'007'
		goto	sa_key2
		goto	sa_key3

;
sa_key2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC = return to menu
		btfsc	STATUS,Z
		goto	rtc_set_can		;"RTC set" cancel

		movf	kbd_buffer,0
		xorlw	H'24'			;E = Set
		btfsc	STATUS,Z
		goto	rtc_set_set		;RTC set the date and time

		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		goto	sa_key3

;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------




;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; NXP devices (port) I2C communication routines
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

p9530_i2c_read	call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

;1. set outputs to high impedance
		movlw	B'00010101'
		movwf	v_F0
		movf	v_pca953x,0
		xorlw	D'003'
		btfsc	STATUS,Z
		incf	v_F0,1			;LS0 starts at other address for PCA9532

		movf	v_F0,0			;select LS0 register + autoincrement
		call	i2c_send

		movlw	H'F0'
		movwf	v_F0
		movf	v_pca953x,0
		xorlw	D'001'
		btfss	STATUS,Z
		clrf	v_F0			;for all devices except of PCA9530
		movf	v_F0,0
		call	i2c_send		;set LS1 to 00000000 (or 11110000 for PCA9530)

;for PCA9531 and PCA9532 we have to continue
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x789			;leave for PCA9530 now

		movlw	H'00'
		call	i2c_send		;set LS1 to 00000000
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x789

;for PCA9532 set registers LS2 & LS3 to 00000000
		movlw	H'00'
		call	i2c_send
		movlw	H'00'
		call	i2c_send

x789		call	i2c_stop

;2. read the input value from INPUT0
		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movlw	B'00000000'		;select INPUT register
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_r0
		call	i2c_not_ack
		call	i2c_stop

		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		return

;read the INPUT2 register for device PCA9532

		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movlw	B'00000001'		;select INPUT1 (2nd) register
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_r1
		call	i2c_not_ack
		call	i2c_stop

		return

;-------------------------------------------------------------------------------
; PCA9530 - all registers read (I2C):
; PWM, RSC and LS0
;-------------------------------------------------------------------------------
p_pxx_i2c_read	call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movf	v_pca953x,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	x593
;-----------------------------------------------
;PCA9530 & PCA9531
		movlw	B'00010001'		;select PSC0 register + autoincrement
		call	i2c_send
		goto	x594
;-----------------------------------------------
;PCA9532
x593		movlw	B'00010010'		;select PSC0 register + autoincrement
		call	i2c_send
;-----------------------------------------------
x594		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_psc0
		call	i2c_ack

		call	i2c_receive
		movwf	v_pwm0
		call	i2c_ack

		call	i2c_receive
		movwf	v_psc1
		call	i2c_ack

		call	i2c_receive
		movwf	v_pwm1
		call	i2c_ack

		call	i2c_receive
		movwf	v_ls0
		call	i2c_ack

;we do not need to leave for PCA9530 now
;we just start to read something what doesn't make sense anymore
;but doesn't hurt as well....

		call	i2c_receive
		movwf	v_ls1
		call	i2c_ack

		call	i2c_receive
		movwf	v_ls2
		call	i2c_ack

		call	i2c_receive
		movwf	v_ls3
		call	i2c_not_ack

		call	i2c_stop
		return

;-------------------------------------------------------------------------------
; PWM & PSC registers I2C write
;-------------------------------------------------------------------------------
p_pxx_i2c_write	call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movf	v_pca953x,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	x578
;--------------------------------------------------------		
;PCA9530 & PCA9531
		movlw	B'00010001'		;select PSC0 register + autoincrement
		call	i2c_send
		goto	x577
;--------------------------------------------------------
;PCA9532 - PSC0 register starts at 0010 instead of 0001
x578		movlw	B'00010010'		;select PSC0 register + autoincrement
		call	i2c_send
;--------------------------------------------------------
x577		movf	v_psc0,0
		call	i2c_send

		movf	v_pwm0,0
		call	i2c_send

		movf	v_psc1,0
		call	i2c_send

		movf	v_pwm1,0
		call	i2c_send

		movf	v_pca953x,0
		xorlw	D'001'
		btfss	STATUS,Z
		goto	next22
;only for PCA9530 - set the first highest bits to 1
		movlw	B'11110000'
		iorwf	v_ls0,1
next22		movf	v_ls0,0
		call	i2c_send

		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x777

;send LS1 for devices above PCA9530
		movf	v_ls1,0
		call	i2c_send

		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x777

;in addition send LS2 and LS3 for PCA9532
		movf	v_ls2,0
		call	i2c_send

		movf	v_ls3,0
		call	i2c_send

x777		call	i2c_stop
		return


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; TMP100 I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;---------------------------------------------------------------------------------
;read routine for temperature sensor TMP100, LM76 and AT30TSE754
;with pointer setting
;---------------------------------------------------------------------------------


tmp100_read_t1	nop

;1. write the address of required registerto the device
		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	B'00000000'		;Register Addess = 00 => temperature register
		call	i2c_send

		call	i2c_stop

;2. read the temperature
		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return
;---------------------------------------------------------------------------------
;read routine for temperature sensor TMP100, LM76 and AT30TSE754
;w/o pointer setting
;---------------------------------------------------------------------------------
tmp100_read_t2	nop

;1. read the temperature
		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return
;---------------------------------------------------------------------------------
;read routine for temperature sensor DS1624
;---------------------------------------------------------------------------------

ds1624_init	call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'EE'			;command = start temperature conversion
		call	i2c_send

		call	i2c_stop
		return
;---------------------------------------------------
tmp100_read_t3	nop
;1. send command protocol
		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'AA'			;command = read temeprature
		call	i2c_send

;2. read temeprature
		call	i2c_restart

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines - new
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

i2c_time_out	goto	i2c_ee

;-----------------------------------------------------------------------------
i2c_start	movlw	D'255'
		movwf	TIMER2

		bcf	PIR1,SSP1IF
		bsf	SSP1CON2,SEN

wait1		dcfsnz	TIMER2,1
		goto	i2c_time_out
		btfss	PIR1,SSP1IF
		goto	wait1
		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bcf	PIR1,SSP1IF
		movwf	SSP1BUF
wait2		btfss	PIR1,SSP1IF
		goto	wait2
		bcf	PIR1,SSP1IF
		btfss	SSP1CON2,ACKSTAT
		goto	ack_received
		goto	ack_not_rec
;------------------------------------------------------------------
ack_received	bsf	PORTA,D'006'		;turn on green LED
		goto	ccc2
;------------------------------------------------------------------
ack_not_rec	bsf	PORTA,D'007'		;turn on red LED
		goto	ccc2
;------------------------------------------------------------------
ccc2		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
xi2c_start	bsf	SSP1CON2,SEN
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	SSP1CON2,PEN
		call	d55
		bcf	PORTA,D'006'
		bcf	PORTA,D'007'
		return
;-----------------------------------------------------------------------------
i2c_ack		bcf	SSP1CON2,ACKDT			;ACK
		bsf	SSP1CON2,ACKEN
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	SSP1CON2,ACKDT			;NOT ACK
		bsf	SSP1CON2,ACKEN
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	SSP1CON2,RCEN
		call	d55
		movf	SSP1BUF,0
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
xi2c_send	movwf	SSP1BUF
		bsf	PORTA,D'006'
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	SSP1CON2,RSEN
		call	d55
		return
;-----------------------------------------------------------------------------


;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;----------------------------------------------------------------------------
dr01s		MOVLW   D'010'		;original value was 010
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;SPI Interface initialization
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;reset SPI based on settings

spi_reinit1	clrf	SSP2CON1
		movlw	B'00000010'		;see Datasheet Page 260
		movwf	SSP2CON1
		btfsc	v_spi_conf,D'004'
		bsf	SSP2CON1,D'004'

		clrf	SSP2STAT
		btfsc	v_spi_conf,D'006'
		bsf	SSP2STAT,D'006'

		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2

;set pull-up / down for SDI
		btfss	v_spi_conf,D'000'
		call	sdi_float		;default = floating
		btfss	v_spi_conf,D'000'
		return				;--> exit

		btfss	v_spi_conf,D'001'
		call	sdi_pull_down
		btfsc	v_spi_conf,D'001'
		call	sdi_pull_up
		return
;---------------------------------------------------------------------------------
sdi_pull_up	bcf	TRISB,D'004'
		bsf	PORTB,D'004'		;use pull-up for SDI line (MISO)
		return
;---------------------------------------------------------------------------------
sdi_pull_down	bcf	TRISB,D'004'
		bcf	PORTB,D'004'		;use pull-down for SDI line (MISO)
		return
;---------------------------------------------------------------------------------
sdi_float	bsf	TRISB,D'004'		;PB4 = high impedance
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;reset SPI for EEPROM
spi_reinit2	clrf	SSP2CON1
		nop
		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
spi_init	nop

;check the Chip Select functionality
;P7 of PCF8574 is just for doublecheck of the functionality (with connected LED)
;just switch on the LED for now...
		call	xi2c_start
		movlw	B'01001110'		;I2C address of the internal PCF8574
		call	xi2c_send
		movlw	B'01111111'
		call	xi2c_send
		call	i2c_stop

; ----------------------------------------------------
; SPI initilization on MSSP2
;
;  RB1 = SCK2
;  RB2 = SDI2
;  RB3 = SDO2
;
; ----------------------------------------------------

		bsf	TRISB,D'002'		;SDI2 (RB2 -- input)
		bcf	TRISB,D'003'		;SDO2 (RB3 -- output)
		bcf	TRISB,D'001'		;SCK2 (RB1 -- output for master mode)


;set up SPI BUS for communication as a master for MSSP2
		clrf	v_spi_conf

		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1


		clrf	SSP2STAT
						;bit 7   = 0 -> Input data sampled at middle of data output time
						;bit 6   = 0 -> Transmit occurs on transition from Idle to active clock state

		bsf	SSP2STAT,D'006'		;bit 6   = 1 -> Transmit occurs on transition from Active to Idle clock state
		bsf	v_spi_conf,D'006'

		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
spi_init_AMS	movlw	B'00000010'		;see Datasheet Page 260
		movwf	SSP2CON1
		clrf	SSP2STAT
		bsf	SSP2STAT,D'006'
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
spi_init_DS1306	movlw	B'00000010'		;see Datasheet Page 260
		movwf	SSP2CON1
		clrf	SSP2STAT
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C Interface initialization
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_init	nop
;set up I2C BUS for communication as a master for MSSP1
		movlw	B'00101000'		;see Datasheet Page 260
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;internal I2C chanels initialization
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_setup	call	i2c_bus_rst

		clrf	brd_i2c_config

		call	i2c_bus_test
		btfsc	c_tmp,D'000'
		goto	i2c_cee				;I2c internal communication doesn't work


		call	disp_clr1
		call	disp_clr2

;I2C channel #0 -----------------------------

i2c_setup_r	movlw	B'10000000'
		call	send_comm
		call	wm_I
		call	wm_2
		call	wm_C
		call	wm_kk
		call	wm_0
		call	wm_space
		call	wm_minus
		call	wm_space

		btfsc	brd_i2c_config,D'004'
		goto	i2c_ch0_erc
		bsf	brd_i2c_config,D'000'		;enable I2C #0
		call	send_i2c_config
		call	i2c_bus_test
		btfsc	c_tmp,D'000'
		goto	i2c_ch0_err			;I2c channel #0 error
		call	wm_o
		call	wm_k
		call	wm_space
		call	wm_space
		call	wm_space
		goto	i2c_ch1_c

i2c_ch0_erc	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_space
		call	wm_space

;I2C channel #1 -----------------------------

i2c_ch1_c	call	wm_I
		call	wm_2
		call	wm_C
		call	wm_kk
		call	wm_1
		call	wm_space
		call	wm_minus
		call	wm_space

		btfsc	brd_i2c_config,D'005'
		goto	i2c_ch1_erc
		bsf	brd_i2c_config,D'001'		;enable I2C #1
		call	send_i2c_config
		call	i2c_bus_test
		btfsc	c_tmp,D'000'
		goto	i2c_ch1_err			;I2c channel #1 error
		call	wm_o
		call	wm_k
		call	wm_space
		call	wm_space
		call	wm_space
		goto	i2c_ch2_c

i2c_ch1_erc	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_space
		call	wm_space

;I2C channel #2 -----------------------------

i2c_ch2_c	movlw	B'11000000'
		call	send_comm

		call	wm_I
		call	wm_2
		call	wm_C
		call	wm_kk
		call	wm_2
		call	wm_space
		call	wm_minus
		call	wm_space

		btfsc	brd_i2c_config,D'006'
		goto	i2c_ch2_erc
		bsf	brd_i2c_config,D'002'		;enable I2C #2
		call	send_i2c_config
		call	i2c_bus_test
		btfsc	c_tmp,D'000'
		goto	i2c_ch2_err			;I2c channel #2 error
		call	wm_o
		call	wm_k
		call	wm_space
		call	wm_space
		call	wm_space
		goto	i2c_ch3_c

i2c_ch2_erc	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_space
		call	wm_space

;I2C channel #3 -----------------------------

i2c_ch3_c	call	wm_I
		call	wm_2
		call	wm_C
		call	wm_kk
		call	wm_3
		call	wm_space
		call	wm_minus
		call	wm_space

		btfsc	brd_i2c_config,D'007'
		goto	i2c_ch3_erc
		bsf	brd_i2c_config,D'003'		;enable I2C #3
		call	send_i2c_config
		call	i2c_bus_test
		btfsc	c_tmp,D'000'
		goto	i2c_ch3_err			;I2c channel #3 error
		call	wm_o
		call	wm_k
		call	wm_space
		call	wm_space
		call	wm_space
		goto	i2c_ch_fin

i2c_ch3_erc	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_space
		call	wm_space

;-----------------------------------------
i2c_ch_fin	call	show_LEDs

		call	dr4
		call	dr4
		call	dr4
		call	dr4

		return

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
show_LEDs	bsf	v_board_LEDs,D'000'
		btfsc	brd_i2c_config,D'000'
		bcf	v_board_LEDs,D'000'

		bsf	v_board_LEDs,D'001'
		btfsc	brd_i2c_config,D'001'
		bcf	v_board_LEDs,D'001'

		bsf	v_board_LEDs,D'002'
		btfsc	brd_i2c_config,D'002'
		bcf	v_board_LEDs,D'002'

		bsf	v_board_LEDs,D'003'
		btfsc	brd_i2c_config,D'003'
		bcf	v_board_LEDs,D'003'

		call	p_send6_int			;LEDs
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;send I2C configuratio to PCA9546 ---------

send_i2c_config	call	xi2c_start
		movlw	B'11101110'
		call	xi2c_send
		movf	brd_i2c_config,0
		call	xi2c_send
		call	i2c_stop
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_bus_rst	bcf	PORTC,D'002'		;RESET PCA6107 & PCA9546
		call	d2
		bsf	PORTC,D'002'
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_bus_test	movf	v_board_LEDs,0
		movwf	data_tmp

		clrf	c_tmp

; -- test the data transfer through the bus ---
;1. write data to PCA6107
		movlw	H'AA'
		movwf	v_board_LEDs
		call	p_send6_int
;2. now read the data written
		call	p_read6_int
		movf	v_r0,0
		xorlw	H'AA'
		btfss	STATUS,Z
		bsf	c_tmp,D'000'			;communication fails

		movf	data_tmp,0
		movwf	v_board_LEDs
		call	p_send6_int			;LEDs
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C channel error -- communication doesn't work
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_ch0_err	bsf	brd_i2c_config,D'004'		;I2c channel #0 error
		bcf	brd_i2c_config,D'000'
		call	i2c_bus_rst
		goto	i2c_setup_r
;---------------------------------------------------------------------------------
i2c_ch1_err	bsf	brd_i2c_config,D'005'		;I2c channel #1 error
		bcf	brd_i2c_config,D'001'
		call	i2c_bus_rst
		goto	i2c_setup_r
;---------------------------------------------------------------------------------
i2c_ch2_err	bsf	brd_i2c_config,D'006'		;I2c channel #2 error
		bcf	brd_i2c_config,D'002'
		call	i2c_bus_rst
		goto	i2c_setup_r
;---------------------------------------------------------------------------------
i2c_ch3_err	bsf	brd_i2c_config,D'007'		;I2c channel #3 error
		bcf	brd_i2c_config,D'003'
		call	i2c_bus_rst
		goto	i2c_setup_r
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C critical error - internal communication doesn't work
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_cee		call	disp_clr1
		call	disp_clr2
		call	w_i2c_cee

cee02		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	mm001

		goto	cee02

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
w_i2c_cee	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_i2c_cee1
		movwf	TBLPTRH
		movlw	low t_i2c_cee1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_zat2

		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C communication error - time out occured during I2C I/O operations
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_ee		call	disp_clr1
		call	disp_clr2
		call	w_i2c_ee

ee02		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	ee02

		call	i2c_setup
		goto	m001

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
w_i2c_ee	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_i2c_ee1
		movwf	TBLPTRH
		movlw	low t_i2c_ee1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_zat2

		return

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; SPI communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_cs0_en	call	i2c_start
		movlw	B'01001110'
		call	i2c_send
		movlw	B'01111110'
		call	i2c_send
		call	i2c_stop
		return		
;--------------------------------------
spi_cs_all_dis	call	i2c_start
		movlw	B'01001110'
		call	i2c_send
		movlw	B'01111111'
		call	i2c_send
		call	i2c_stop
		return		
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_cs1_en	call	i2c_start
		movlw	B'01001110'
		call	i2c_send
		movlw	B'01111101'
		call	i2c_send
		call	i2c_stop
		return		
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_send	bcf	SSP2CON1,SSPOV		;just for sure...
		bcf	SSP2STAT,BF		;just for sure...

		movwf	SSP2BUF
spil2		btfss	SSP2STAT,BF
		goto	spil2
		movf	SSP2BUF,0
		movwf	v_spi_buffi

		return
;-----------------------------------------------------------------------------



;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Not official part - test routines
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

out_p_msb	EQU	H'2B'
out_p_csb	EQU	H'2C'
out_p_lsb	EQU	H'2D'

out_t_msb	EQU	H'2E'
out_t_lsb	EQU	H'2F'


;------------------------------------------------------------------------------
t_mpl3115	da	"MPL3115A2 - ", H'FE'
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_MPL3115A2	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_mpl3115
		movwf	TBLPTRH
		movlw	low t_mpl3115
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
w_xpa_results	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_P
		call	wm_dvojbodka
		movf	out_p_msb,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	out_p_csb,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	out_p_lsb,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_T
		call	wm_dvojbodka

		movf	out_t_msb,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	out_t_lsb,0
		movwf	kbd_buffer
		call	write_hex

		return
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; MPL3115A2 initialization - Barometer mode
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
xpa_init1	call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'26'		;CTRL_REG1
		call	i2c_send
		movlw	H'B8'
		call	i2c_send
;
		call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'13'		;P_DATA_CFG
		call	i2c_send
		movlw	H'07'
		call	i2c_send

		call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'26'		;CTRL_REG1
		call	i2c_send
		movlw	H'B9'
		call	i2c_send

		call	i2c_stop
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; MPL3115A2 initialization - read meassurement results
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
xpa_read	call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'00'		;STATUS
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'	;I2C address of the device + read
		call	i2c_send
		call	i2c_receive
		movwf	c_tmp
		call	i2c_not_ack
		call	i2c_stop

		btfss	c_tmp,D'003'
		goto	xpa_read	;try again...

;now we need to read the data

		call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'01'		;OUT_P_MSB
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'	;I2C address of the device + read
		call	i2c_send
		call	i2c_receive
		movwf	out_p_msb
		call	i2c_ack

		call	i2c_receive
		movwf	out_p_csb
		call	i2c_ack

		call	i2c_receive
		movwf	out_p_lsb
		call	i2c_ack

		call	i2c_receive
		movwf	out_t_msb
		call	i2c_ack

		call	i2c_receive
		movwf	out_t_lsb
		call	i2c_not_ack

		call	i2c_stop

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; MPL3115A2 presence check (read register "who am I")
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
xpa_pc		call	i2c_start
		movlw	B'11000000'	;I2C address of the device + write
		call	i2c_send
		movlw	H'0C'		;address of the "WHO_AM_I" register
		call	i2c_send
;
		call	i2c_start
		movlw	B'11000001'	;I2C address of the device + read
		call	i2c_send
		call	i2c_receive
		xorlw	H'C4'		;MPL3115A2 shall replay with C4 hex
		btfss	STATUS,Z
		goto	no_resp
;-----------------------------------------
;responose was ok
		call	i2c_not_ack
		call	i2c_stop
		movlw	B'10001100'
		call	send_comm
		call	wm_o
		call	wm_k
		return
;-----------------------------------------
no_resp		call	i2c_not_ack
		call	i2c_stop
		movlw	B'10001100'
		call	send_comm
		call	wm_f
		call	wm_a
		call	wm_i
		call	wm_l
		call	wm_s
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;		main test for MPL3115A2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
test_fnc01	call	disp_clr1
		call	disp_clr2
		call	w_MPL3115A2

		call	xpa_pc			;check if the device is connected
		call	xpa_init1

		call	xpa_read
		call	w_xpa_results

xx_t001		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = Re-run
		btfsc	STATUS,Z
		goto	test_fnc01


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	xx_t001

		goto	m001

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;		AMS1107 Routines (SPI communication)
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;initialize the AMS1107
AMS_init	movlw	B'11111100'		;register XCh --> Shutdown register
		movwf	ams_v_MSB
		movlw	H'01'			;normal operation
		movwf	ams_v_LSB
		call	AMS_send_data
;
		movlw	H'F9'
		movwf	ams_v_MSB		;decode enable register
		movlw	H'00'
		movwf	ams_v_LSB		;no decode mode
		call	AMS_send_data
;
		movlw	H'FB'
		movwf	ams_v_MSB		;scan limit register
		movlw	H'07'
		movwf	ams_v_LSB		;all 8 digits
		call	AMS_send_data
;
		movlw	H'0F'			;default - maximal intensity
		call	AMS_intensity

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_intensity	nop
		movwf	ams_v_LSB		;xFh = max / x0h = min
		movlw	H'FA'
		movwf	ams_v_MSB		;intensity register

		call	AMS_send_data
		return		
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_send_data	call	spi_init_AMS
		call	spi_cs1_en

		movf	ams_v_MSB,0
		movwf	ams_v_value
		call	AMS_spi_send

		movf	ams_v_LSB,0
		movwf	ams_v_value
		call	AMS_spi_send

		call	spi_cs_all_dis

		call	spi_reinit1
		return
;----------------------------------------------------------------------------
AMS_spi_send	nop
		bcf	SSP2CON1,SSPOV
		bcf	SSP2STAT,BF

		movf	ams_v_value
		movwf	SSP2BUF
AMS_spil2	btfss	SSP2STAT,BF
		goto	AMS_spil2
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
AMS_disp_direct	movlw	H'F1'			;DIG0
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movf	ams_v_dig0,0
		call	AMS_send_data

		movlw	H'F2'			;DIG1
		movwf	ams_v_MSB
		movf	ams_v_dig1,0
		call	AMS_send_data

		movlw	H'F3'			;DIG2
		movwf	ams_v_MSB
		movf	ams_v_dig2,0
		call	AMS_send_data

		movlw	H'F4'			;DIG3
		movwf	ams_v_MSB
		movf	ams_v_dig3,0
		call	AMS_send_data

		movlw	H'F5'			;DIG4
		movwf	ams_v_MSB
		movf	ams_v_dig4,0
		call	AMS_send_data

		movlw	H'F6'			;DIG5
		movwf	ams_v_MSB
		movf	ams_v_dig5,0
		call	AMS_send_data

		movlw	H'F7'			;DIG6
		movwf	ams_v_MSB
		movf	ams_v_dig6,0
		call	AMS_send_data

		movlw	H'F8'			;DIG7
		movwf	ams_v_MSB
		movf	ams_v_dig7,0
		call	AMS_send_data
		return
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
AMS_disp_code	movlw	H'F1'			;DIG0
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movf	ams_v_dig0,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F2'			;DIG1
		movwf	ams_v_MSB
		movf	ams_v_dig1,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F3'			;DIG2
		movwf	ams_v_MSB
		movf	ams_v_dig2,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F4'			;DIG3
		movwf	ams_v_MSB
		movf	ams_v_dig3,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F5'			;DIG4
		movwf	ams_v_MSB
		movf	ams_v_dig4,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F6'			;DIG5
		movwf	ams_v_MSB
		movf	ams_v_dig5,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F7'			;DIG6
		movwf	ams_v_MSB
		movf	ams_v_dig6,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F8'			;DIG7
		movwf	ams_v_MSB
		movf	ams_v_dig7,0
		call	AMS_get_char
		call	AMS_send_data
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
AMS_get_char	movwf	ams_v_tmp
		movwf	ams_v_tmp2
		bcf	ams_v_tmp,D'007'
		call	AMS_get_char2

		movf	ams_v_LSB,0
		btfsc	ams_v_tmp2,D'007'
		addlw	D'128'

		movwf	ams_v_LSB

		return
;----------------------------------------------------------------------------
AMS_get_char2	movf	ams_v_tmp,0
		xorlw	D'000'
		btfsc	STATUS,Z
		goto	AMS_chr0

		movf	ams_v_tmp,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	AMS_chr1

		movf	ams_v_tmp,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	AMS_chr2

		movf	ams_v_tmp,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	AMS_chr3

		movf	ams_v_tmp,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	AMS_chr4

		movf	ams_v_tmp,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	AMS_chr5

		movf	ams_v_tmp,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	AMS_chr6

		movf	ams_v_tmp,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	AMS_chr7

		movf	ams_v_tmp,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	AMS_chr8

		movf	ams_v_tmp,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	AMS_chr9

		movf	ams_v_tmp,0
		xorlw	D'010'		;eq H'0A'
		btfsc	STATUS,Z
		goto	AMS_chrA

		movf	ams_v_tmp,0
		xorlw	D'011'		;eq H'0B'
		btfsc	STATUS,Z
		goto	AMS_chrb

		movf	ams_v_tmp,0
		xorlw	D'012'		;eq H'0C'
		btfsc	STATUS,Z
		goto	AMS_chrC

		movf	ams_v_tmp,0
		xorlw	D'013'		;eq H'0D'
		btfsc	STATUS,Z
		goto	AMS_chrd


		movf	ams_v_tmp,0
		xorlw	D'014'		;eq H'0E'
		btfsc	STATUS,Z
		goto	AMS_chrE

		movf	ams_v_tmp,0
		xorlw	D'015'		;eq H'0F'
		btfsc	STATUS,Z
		goto	AMS_chrF

		movf	ams_v_tmp,0
		xorlw	D'016'
		btfsc	STATUS,Z
		goto	AMS_chr_s5

		movf	ams_v_tmp,0
		xorlw	D'017'
		btfsc	STATUS,Z
		goto	AMS_chr_so

		movf	ams_v_tmp,0
		xorlw	D'018'
		btfsc	STATUS,Z
		goto	AMS_chr_stupen

		movf	ams_v_tmp,0
		xorlw	D'019'
		btfsc	STATUS,Z
		goto	AMS_chr_s1

		movf	ams_v_tmp,0
		xorlw	D'020'
		btfsc	STATUS,Z
		goto	AMS_chr_s2

		movf	ams_v_tmp,0
		xorlw	D'021'
		btfsc	STATUS,Z
		goto	AMS_chr_s3

		movf	ams_v_tmp,0
		xorlw	D'022'
		btfsc	STATUS,Z
		goto	AMS_chr_s4

		movf	ams_v_tmp,0
		xorlw	'L'
		btfsc	STATUS,Z
		goto	AMS_chrL

		movf	ams_v_tmp,0
		xorlw	'U'
		btfsc	STATUS,Z
		goto	AMS_chrU

		movf	ams_v_tmp,0
		xorlw	'C'
		btfsc	STATUS,Z
		goto	AMS_chrC

		movf	ams_v_tmp,0
		xorlw	'I'
		btfsc	STATUS,Z
		goto	AMS_chrI

		movf	ams_v_tmp,0
		xorlw	'-'
		btfsc	STATUS,Z
		goto	AMS_chrm

		movf	ams_v_tmp,0
		xorlw	'0'
		btfsc	STATUS,Z
		goto	AMS_chr0

		movf	ams_v_tmp,0
		xorlw	'1'
		btfsc	STATUS,Z
		goto	AMS_chr1

		movf	ams_v_tmp,0
		xorlw	'2'
		btfsc	STATUS,Z
		goto	AMS_chr2

		movf	ams_v_tmp,0
		xorlw	'3'
		btfsc	STATUS,Z
		goto	AMS_chr3

		movf	ams_v_tmp,0
		xorlw	'4'
		btfsc	STATUS,Z
		goto	AMS_chr4

		movf	ams_v_tmp,0
		xorlw	'5'
		btfsc	STATUS,Z
		goto	AMS_chr5

		movf	ams_v_tmp,0
		xorlw	'6'
		btfsc	STATUS,Z
		goto	AMS_chr6

		movf	ams_v_tmp,0
		xorlw	'7'
		btfsc	STATUS,Z
		goto	AMS_chr7

		movf	ams_v_tmp,0
		xorlw	'8'
		btfsc	STATUS,Z
		goto	AMS_chr8

		movf	ams_v_tmp,0
		xorlw	'9'
		btfsc	STATUS,Z
		goto	AMS_chr9

		movf	ams_v_tmp,0
		xorlw	' '
		btfsc	STATUS,Z
		goto	AMS_chr_space

		movf	ams_v_tmp,0
		xorlw	'A'
		btfsc	STATUS,Z
		goto	AMS_chrA

		movf	ams_v_tmp,0
		xorlw	'E'
		btfsc	STATUS,Z
		goto	AMS_chrE

		movf	ams_v_tmp,0
		xorlw	'F'
		btfsc	STATUS,Z
		goto	AMS_chrF

		movf	ams_v_tmp,0
		xorlw	'P'
		btfsc	STATUS,Z
		goto	AMS_chrP

		movf	ams_v_tmp,0
		xorlw	'S'
		btfsc	STATUS,Z
		goto	AMS_chrS

		movf	ams_v_tmp,0
		xorlw	'b'
		btfsc	STATUS,Z
		goto	AMS_chrb

		movf	ams_v_tmp,0
		xorlw	'd'
		btfsc	STATUS,Z
		goto	AMS_chrd

		movf	ams_v_tmp,0
		xorlw	'i'
		btfsc	STATUS,Z
		goto	AMS_chri

		movf	ams_v_tmp,0
		xorlw	'r'
		btfsc	STATUS,Z
		goto	AMS_chrr

		movf	ams_v_tmp,0
		xorlw	't'
		btfsc	STATUS,Z
		goto	AMS_chrt

		movf	ams_v_tmp,0
		xorlw	'o'
		btfsc	STATUS,Z
		goto	AMS_chro

		goto	AMS_chr_err
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_chr0	movlw	B'01111110'
		movwf	ams_v_LSB
		return

AMS_chr1	movlw	B'00110000'
		movwf	ams_v_LSB
		return

AMS_chr2	movlw	B'01101101'
		movwf	ams_v_LSB
		return

AMS_chr3	movlw	B'01111001'
		movwf	ams_v_LSB
		return

AMS_chr4	movlw	B'00110011'
		movwf	ams_v_LSB
		return

AMS_chr5	movlw	B'01011011'
		movwf	ams_v_LSB
		return

AMS_chr6	movlw	B'01011111'
		movwf	ams_v_LSB
		return

AMS_chr7	movlw	B'01110000'
		movwf	ams_v_LSB
		return

AMS_chr8	movlw	B'01111111'
		movwf	ams_v_LSB
		return

AMS_chr9	movlw	B'01111011'
		movwf	ams_v_LSB
		return

AMS_chrm	movlw	B'00000001'		;minus
		movwf	ams_v_LSB
		return

AMS_chr_space	movlw	B'00000000'
		movwf	ams_v_LSB
		return

AMS_chr_s1	movlw	B'01000000'
		movwf	ams_v_LSB
		return

AMS_chr_s2	movlw	B'00000001'
		movwf	ams_v_LSB
		return

AMS_chr_s3	movlw	B'00001000'
		movwf	ams_v_LSB
		return

AMS_chr_s4	movlw	B'00010010'
		movwf	ams_v_LSB
		return

AMS_chr_s5	movlw	B'00100000'
		movwf	ams_v_LSB
		return

AMS_chr_so	movlw	B'01100011'
		movwf	ams_v_LSB
		return

AMS_chrL	movlw	B'00001110'
		movwf	ams_v_LSB
		return

AMS_chrU	movlw	B'00111110'
		movwf	ams_v_LSB
		return

AMS_chrA	movlw	B'01110111'
		movwf	ams_v_LSB
		return

AMS_chrC	movlw	B'01001110'
		movwf	ams_v_LSB
		return

AMS_chrI	movlw	B'00110000'
		movwf	ams_v_LSB
		return

AMS_chrE	movlw	B'01001111'
		movwf	ams_v_LSB
		return

AMS_chrF	movlw	B'01000111'
		movwf	ams_v_LSB
		return

AMS_chrP	movlw	B'01100111'
		movwf	ams_v_LSB
		return

AMS_chrS	movlw	B'01011011'
		movwf	ams_v_LSB
		return

AMS_chrb	movlw	B'00011111'
		movwf	ams_v_LSB
		return

AMS_chrd	movlw	B'00111101'
		movwf	ams_v_LSB
		return

AMS_chri	movlw	B'00010000'
		movwf	ams_v_LSB
		return

AMS_chrr	movlw	B'00000101'
		movwf	ams_v_LSB
		return

AMS_chrt	movlw	B'00001111'
		movwf	ams_v_LSB
		return

AMS_chro	movlw	B'00011101'
		movwf	ams_v_LSB
		return

AMS_chr_stupen	movlw	B'01100011'
		movwf	ams_v_LSB
		return

AMS_chr_err	movlw	B'01001000'
		movwf	ams_v_LSB
		bsf	ams_err_flag,D'000'
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END